@@ -8,16 +8,6 @@ rlJournalStart
         rlRun 'set -o pipefail'
     rlPhaseEnd
 
-    rlPhaseStartTest "Filter by test name"
-        plan='plan --name fmf/nourl/noref/nopath'
-        discover='discover --how fmf --test discover1'
-        rlRun "tmt run -dvr $discover $plan finish 2>&1 >/dev/null | tee output"
-        rlAssertGrep '1 test selected' output
-        rlAssertGrep '/tests/discover1' output
-        rlAssertNotGrep '/tests/discover2' output
-        rlAssertNotGrep '/tests/discover3' output
-    rlPhaseEnd
-
     rlPhaseStartTest "Filter by advanced filter"
         plan='plan --name fmf/nourl/noref/nopath'
         discover='discover --how fmf --filter tier:1,2'
@@ -28,25 +18,6 @@ rlJournalStart
         rlAssertNotGrep '/tests/discover3' output
     rlPhaseEnd
 
-    for exclude in '-x' '--exclude'; do
-        rlPhaseStartTest "Exclude tests using $exclude <regex>"
-            plan='plan --name fmf/nourl/noref/nopath'
-            discover='discover --how fmf'
-            rlRun "tmt run -dvr $discover $plan 2>&1 >/dev/null | tee output"
-            rlAssertGrep '/tests/discover1' output
-            rlRun "tmt run -dvr $discover $exclude discover1 $plan 2>&1 >/dev/null | tee output"
-            rlAssertNotGrep '/tests/discover1' output
-        rlPhaseEnd
-    done
-
-    rlPhaseStartTest "Exclude tests via exclude option within a plan metadata"
-        plan='plan --name fmf/exclude'
-        discover='discover --how fmf'
-        rlRun "tmt run -dvr $discover $plan 2>&1 >/dev/null | tee output"
-        rlAssertNotGrep '/tests/discover1' output
-        rlAssertGrep '/tests/discover2' output
-    rlPhaseEnd
-
     rlPhaseStartTest "Filter by link"
         plan='plans --default'
         for link_relation in "" "relates:" "rel.*:"; do
@@ -190,24 +161,6 @@ in plan \`/plans/a-non-url\` can be used only within git repo." output
         rlRun "tmt run -rdvvv discover -h fmf --fmf-id \
                --url https://github.com/teemtee/fmf finish 2>&1 >/dev/null | tee output" 0
         rlRun "rm -rf $tmp_dir1 $tmp_dir2"
-
-        # 2: w/ url in plan AND w/o url in plan: w/o url in CLI - w/ url in CLI
-        tmp_dir1="$(mktemp -d)"
-        tmp_dir2="$(mktemp -d)"
-        rlRun "cd $tmp_dir1"
-        rlRun "tmt init --template full"
-        rlRun "cd $tmp_dir2"
-        rlRun "tmt init --template base"
-        rlRun "cp plans/example.fmf $tmp_dir1/plans/z-non-url.fmf"
-        rlRun "cd $tmp_dir1"
-        rlRun "tmt run -rdvvv discover -h fmf --fmf-id finish 2>&1 >/dev/null \
-               | tee output" 2
-        rlAssertGrep "\`tmt run discover --fmf-id\` without \`url\` option \
-in plan \`/plans/z-non-url\` can be used only within git repo." output
-
-        rlRun "tmt run -rdvvv discover -h fmf --fmf-id \
-               --url https://github.com/teemtee/fmf finish 2>&1 >/dev/null | tee output" 0
-        rlRun "rm -rf $tmp_dir1 $tmp_dir2"
     rlPhaseEnd
 
     # If current dir has .fmf and --url CLI option exists
