@@ -47,7 +47,7 @@ rlJournalStart
         fi
 
         rlRun "package_cache=\$(mktemp -d)" 0 "Create cache directory for downloaded packages"
-        rlRun "run=\$(mktemp -d)" 0 "Create run directory"
+        rlRun "run=\$(mktemp -d -p /var/tmp)" 0 "Create run directory"
         rlRun "pushd data"
 
         rlRun "export TMT_BOOT_TIMEOUT=300"
@@ -64,14 +64,14 @@ rlJournalStart
                 rlRun "distro=fedora-rawhide"
                 rlRun "package_manager=dnf5"
 
+            elif is_fedora_42 "$image"; then
+                rlRun "distro=fedora-42"
+                rlRun "package_manager=dnf5"
+
             elif is_fedora_41 "$image"; then
                 rlRun "distro=fedora-41"
                 rlRun "package_manager=dnf5"
 
-            elif is_fedora_40 "$image"; then
-                rlRun "distro=fedora-40"
-                rlRun "package_manager=dnf"
-
             elif is_centos_stream_9 "$image"; then
                 rlRun "distro=centos-stream-9"
                 rlRun "package_manager=dnf"
@@ -96,7 +96,14 @@ rlJournalStart
                 rlRun "distro=fedora-coreos"
 
                 if is_ostree "$image"; then
-                    rlRun "package_manager=rpm-ostree"
+
+                    if [ "$PROVISION_HOW" = "virtual" ]; then
+                      rlRun "package_manager=bootc"
+
+                    else
+                      rlRun "package_manager=rpm-ostree"
+
+                    fi
 
                 elif [ "$PROVISION_HOW" = "virtual" ]; then
                     rlRun "package_manager=dnf"
@@ -118,7 +125,7 @@ rlJournalStart
                 rlFail "Cannot infer distro for image $image"
             fi
 
-            tmt="tmt -vvv -c distro=$distro run --id $run --scratch finish discover provision --how $PROVISION_HOW --image $image prepare"
+            tmt="tmt -vvv -c distro=$distro run --id $run --scratch cleanup discover provision --how $PROVISION_HOW --image $image prepare"
         rlPhaseEnd
 
         # TODO: find out whether all those exceptions can be simplified and parametrized...
@@ -225,7 +232,11 @@ rlJournalStart
                 rlAssertGrep "out: no package provides tree-but-spelled-wrong" $rlRun_LOG
 
             elif is_ostree "$image"; then
-                rlAssertGrep "err: error: Packages not found: tree-but-spelled-wrong" $rlRun_LOG
+                if [ "$PROVISION_HOW" = "virtual" ]; then
+                    rlAssertGrep "err: No match for argument: tree-but-spelled-wrong" $rlRun_LOG
+                else
+                    rlAssertGrep "err: error: Packages not found: tree-but-spelled-wrong" $rlRun_LOG
+                fi
 
             elif is_fedora_coreos "$image"; then
                 rlAssertGrep "err: No match for argument: tree-but-spelled-wrong" $rlRun_LOG
@@ -233,11 +244,11 @@ rlJournalStart
             elif is_fedora_rawhide "$image"; then
                 rlAssertGrep "err: No match for argument: tree-but-spelled-wrong" $rlRun_LOG
 
-            elif is_fedora_41 "$image"; then
+            elif is_fedora_42 "$image"; then
                 rlAssertGrep "err: No match for argument: tree-but-spelled-wrong" $rlRun_LOG
 
-            elif is_fedora_40 "$image"; then
-                rlAssertGrep "err: Error: Unable to find a match: tree-but-spelled-wrong" $rlRun_LOG
+            elif is_fedora_41 "$image"; then
+                rlAssertGrep "err: No match for argument: tree-but-spelled-wrong" $rlRun_LOG
 
             elif is_ubuntu "$image" || is_debian "$image"; then
                 rlAssertGrep "err: E: Unable to locate package tree-but-spelled-wrong" $rlRun_LOG
@@ -259,7 +270,11 @@ rlJournalStart
                 rlAssertGrep "out: no package provides tree-but-spelled-wrong" $rlRun_LOG
 
             elif is_ostree "$image"; then
-                rlAssertGrep "err: error: Packages not found: tree-but-spelled-wrong" $rlRun_LOG
+                if [ "$PROVISION_HOW" = "virtual" ]; then
+                    rlAssertGrep "err: No match for argument: tree-but-spelled-wrong" $rlRun_LOG
+                else
+                    rlAssertGrep "err: error: Packages not found: tree-but-spelled-wrong" $rlRun_LOG
+                fi
 
             elif is_fedora_coreos "$image"; then
                 rlAssertGrep "err: No match for argument: tree-but-spelled-wrong" $rlRun_LOG
@@ -267,11 +282,11 @@ rlJournalStart
             elif is_fedora_rawhide "$image"; then
                 rlAssertGrep "err: No match for argument: tree-but-spelled-wrong" $rlRun_LOG
 
-            elif is_fedora_41 "$image"; then
+            elif is_fedora_42 "$image"; then
                 rlAssertGrep "err: No match for argument: tree-but-spelled-wrong" $rlRun_LOG
 
-            elif is_fedora_40 "$image"; then
-                rlAssertGrep "err: Error: Unable to find a match: tree-but-spelled-wrong" $rlRun_LOG
+            elif is_fedora_41 "$image"; then
+                rlAssertGrep "err: No match for argument: tree-but-spelled-wrong" $rlRun_LOG
 
             elif is_ubuntu "$image" || is_debian "$image"; then
                 rlAssertGrep "err: E: Unable to locate package tree-but-spelled-wrong" $rlRun_LOG
@@ -284,6 +299,10 @@ rlJournalStart
             fi
         rlPhaseEnd
 
+        rlPhaseStartTest "$phase_prefix Empty prepare install with exclude"
+            rlRun "$tmt execute plan --name empty-with-exclude"
+        rlPhaseEnd
+
         # TODO: at least copr is RH-specific, but package name escaping and debuginfo should be
         # possible to extend to other distros.
         if (is_fedora "$image" && ! is_fedora_coreos "$image") || is_centos "$image" || is_ubi "$image"; then
