@@ -20,6 +20,12 @@ rlJournalStart
             prepare -h shell -s 'touch /tmp/first' -s 'touch /tmp/second'"
     rlPhaseEnd
 
+    rlPhaseStartTest "Remote Script"
+        rlRun -s "tmt -vvv run provision prepare finish cleanup plan -n url" 0 "Prepare using a remote script"
+        rlAssertGrep "Hello world" "$rlRun_LOG" #check for the prepare script
+        rlAssertGrep "third" "$rlRun_LOG" # check for the finish script
+    rlPhaseEnd
+
     rlPhaseStartCleanup
         rlRun "popd"
     rlPhaseEnd
