@@ -44,8 +44,8 @@ from . import (  # noqa: E402
     CONTAINER_CENTOS_STREAM_9,
     CONTAINER_CENTOS_STREAM_10,
     CONTAINER_DEBIAN_127,
-    CONTAINER_FEDORA_40,
     CONTAINER_FEDORA_41,
+    CONTAINER_FEDORA_42,
     CONTAINER_FEDORA_COREOS,
     CONTAINER_FEDORA_COREOS_OSTREE,
     CONTAINER_FEDORA_RAWHIDE,
@@ -76,6 +76,7 @@ def has_legacy_dnf(container: ContainerData) -> bool:
 
     return container.image_url_or_id not in (
         CONTAINER_FEDORA_RAWHIDE.url,
+        CONTAINER_FEDORA_42.url,
         CONTAINER_FEDORA_41.url,
         CONTAINER_FEDORA_COREOS.url,
         CONTAINER_FEDORA_COREOS_OSTREE.url,
@@ -89,6 +90,7 @@ def has_dnf5_preinstalled(container: ContainerData) -> bool:
 
     return container.image_url_or_id in (
         CONTAINER_FEDORA_RAWHIDE.url,
+        CONTAINER_FEDORA_42.url,
         CONTAINER_FEDORA_41.url,
         CONTAINER_FEDORA_COREOS.url,
         CONTAINER_FEDORA_COREOS_OSTREE.url,
@@ -123,15 +125,11 @@ CONTAINER_BASE_MATRIX = [
     # Fedora
     (CONTAINER_FEDORA_RAWHIDE, PACKAGE_MANAGER_DNF5),
     (CONTAINER_FEDORA_41, PACKAGE_MANAGER_DNF5),
-    (CONTAINER_FEDORA_40, PACKAGE_MANAGER_DNF5),
-    (CONTAINER_FEDORA_40, PACKAGE_MANAGER_DNF),
-    (CONTAINER_FEDORA_40, PACKAGE_MANAGER_YUM),
+    (CONTAINER_FEDORA_42, PACKAGE_MANAGER_DNF5),
     # CentOS Stream
     (CONTAINER_CENTOS_STREAM_10, PACKAGE_MANAGER_DNF),
     (CONTAINER_CENTOS_STREAM_9, PACKAGE_MANAGER_DNF),
     (CONTAINER_CENTOS_STREAM_9, PACKAGE_MANAGER_YUM),
-    # CentOS
-    (CONTAINER_CENTOS_7, PACKAGE_MANAGER_YUM),
     # UBI
     (CONTAINER_UBI_8, PACKAGE_MANAGER_DNF),
     (CONTAINER_UBI_8, PACKAGE_MANAGER_YUM),
@@ -146,6 +144,11 @@ CONTAINER_BASE_MATRIX = [
     (CONTAINER_ALPINE, PACKAGE_MANAGER_APK),
 ]
 
+# CentOS 7 is not available for aarch64 architecture,
+# so we add it to CONTAINER_BASE_MATRIX for non-aarch64.
+if CONTAINER_CENTOS_7 is not None:
+    CONTAINER_BASE_MATRIX.append((CONTAINER_CENTOS_7, PACKAGE_MANAGER_YUM))
+
 CONTAINER_MATRIX_IDS = [
     f'{container.url} / {package_manager_class.__name__.lower()}'
     for container, package_manager_class in CONTAINER_BASE_MATRIX
@@ -161,15 +164,6 @@ for container, package_manager_class in CONTAINER_BASE_MATRIX:
     CONTAINER_DISCOVERY_MATRIX[container.url] = (container, package_manager_class)
 
 
-def is_dnf5_preinstalled(container: ContainerData) -> bool:
-    return container.image_url_or_id in (
-        CONTAINER_FEDORA_RAWHIDE.url,
-        CONTAINER_FEDORA_41.url,
-        CONTAINER_FEDORA_COREOS.url,
-        CONTAINER_FEDORA_COREOS_OSTREE.url,
-    )
-
-
 def create_package_manager(
     container: ContainerData,
     guest: GuestContainer,
@@ -188,7 +182,7 @@ def create_package_manager(
 
     if package_manager_class is tmt.package_managers.dnf.Dnf5:
         # Note that our custom images contain `dnf5` already
-        if is_dnf5_preinstalled(container):
+        if has_dnf5_preinstalled(container):
             pass
 
         else:
@@ -224,7 +218,7 @@ def test_discovery(
 
         assert guest.facts.package_manager == expected
 
-        assert_log(caplog, message=MATCH(rf"^Discovered package managers: {expected_discovery}$"))
+        assert_log(caplog, message=MATCH(rf"^Discovered package manager: {expected_discovery}$"))
 
     # Images in which `dnf5`` would be the best possible choice, do not
     # come with `dnf5`` pre-installed. Therefore run the discovery first,
@@ -236,22 +230,22 @@ def test_discovery(
         guest.info(f'{has_dnf5_preinstalled(container)=}')
 
         if has_legacy_dnf(container):
-            _test_discovery(tmt.package_managers.dnf.Dnf.NAME, 'dnf and yum')
+            _test_discovery(tmt.package_managers.dnf.Dnf.NAME, 'dnf')
 
             if not has_dnf5_preinstalled(container):
                 guest.execute(ShellScript('dnf install --nogpgcheck -y dnf5'))
 
         if has_legacy_dnf(container):
-            _test_discovery(expected_package_manager.NAME, 'dnf5, dnf and yum')
+            _test_discovery(expected_package_manager.NAME, 'dnf5')
 
         else:
             _test_discovery(expected_package_manager.NAME, 'dnf5')
 
     elif expected_package_manager is tmt.package_managers.dnf.Dnf:
-        _test_discovery(expected_package_manager.NAME, 'dnf and yum')
+        _test_discovery(expected_package_manager.NAME, 'dnf')
 
     elif expected_package_manager is tmt.package_managers.rpm_ostree.RpmOstree:
-        _test_discovery(expected_package_manager.NAME, 'rpm-ostree and dnf5')
+        _test_discovery(expected_package_manager.NAME, 'rpm-ostree')
 
     else:
         _test_discovery(expected_package_manager.NAME, expected_package_manager.NAME)
@@ -1121,7 +1115,7 @@ def _generate_test_check_presence() -> Iterator[
                     r'\s+out:\s+util-linux-',
                 )
 
-            elif 'centos/stream9' in container.url or 'fedora/40' in container.url:
+            elif 'centos/stream9' in container.url:
                 yield (
                     container,
                     package_manager_class,
@@ -1206,7 +1200,7 @@ def _generate_test_check_presence() -> Iterator[
                     r'\s+out:\s+util-linux-',
                 )
 
-            elif 'centos/stream9' in container.url or 'fedora/40' in container.url:
+            elif 'centos/stream9' in container.url:
                 yield (
                     container,
                     package_manager_class,
