@@ -3,8 +3,8 @@ import functools
 from typing import TYPE_CHECKING, Any, Callable, Generic, Optional, TypedDict, TypeVar, cast
 
 import tmt.log
-import tmt.steps.provision
 import tmt.utils
+import tmt.utils.hints
 from tmt.container import (
     SerializableContainer,
     SpecBasedContainer,
@@ -27,14 +27,21 @@ CheckT = TypeVar('CheckT', bound='Check')
 
 CheckPluginClass = type['CheckPlugin[Any]']
 
-_CHECK_PLUGIN_REGISTRY: PluginRegistry[CheckPluginClass] = PluginRegistry()
+_CHECK_PLUGIN_REGISTRY: PluginRegistry[CheckPluginClass] = PluginRegistry('test.check')
 
 
-def provides_check(check: str) -> Callable[[CheckPluginClass], CheckPluginClass]:
+def provides_check(
+    check: str, hints: Optional[dict[str, str]] = None
+) -> Callable[[CheckPluginClass], CheckPluginClass]:
     """
     A decorator for registering test checks.
 
     Decorate a test check plugin class to register its checks.
+
+    :param check: name of the check.
+    :param hints: a mapping containing hints the check wishes to
+        register. Hints are registered with keys prefixed by
+        ``test-checks/$check/`` string.
     """
 
     def _provides_check(check_cls: CheckPluginClass) -> CheckPluginClass:
@@ -44,6 +51,9 @@ def provides_check(check: str) -> Callable[[CheckPluginClass], CheckPluginClass]
             logger=tmt.log.Logger.get_bootstrap_logger(),
         )
 
+        for hint_id, hint in (hints or {}).items():
+            tmt.utils.hints.register_hint(f'test-checks/{check}/{hint_id}', hint)
+
         return check_cls
 
     return _provides_check
@@ -134,6 +144,17 @@ class Check(
     def plugin(self) -> 'CheckPluginClass':
         return find_plugin(self.how)
 
+    @classmethod
+    def create_internal(cls, logger: tmt.log.Logger) -> Optional['Check']:
+        """
+        Create an internal check instance.
+
+        :param logger: logger to use for logging.
+        :returns: a new instance of the check.
+        """
+
+        return None
+
     # ignore[override]: expected, we need to accept one extra parameter, `logger`.
     @classmethod
     def from_spec(  # type: ignore[override]
@@ -224,6 +245,24 @@ class CheckPlugin(tmt.utils._CommonBase, Generic[CheckT]):
             raw_data, logger
         )
 
+    @classmethod
+    def internal_checks(
+        cls,
+        logger: tmt.log.Logger,
+    ) -> list['Check']:
+        """
+        Create internal check data instances
+        """
+
+        checks: list[Check] = []
+
+        for plugin in _CHECK_PLUGIN_REGISTRY.iter_plugins():
+            check = cast(CheckPlugin[CheckT], plugin)._check_class.create_internal(logger)
+            if check:
+                checks.append(check)
+
+        return checks
+
     @classmethod
     def essential_requires(
         cls,
