@@ -7,14 +7,11 @@ from typing import TYPE_CHECKING, Any, Optional, Union
 import jinja2
 
 import tmt.log
-import tmt.steps.execute
-import tmt.steps.provision
 import tmt.utils
-import tmt.utils.templates
 import tmt.utils.themes
 from tmt.checks import Check, CheckPlugin, _RawCheck, provides_check
 from tmt.container import container, field
-from tmt.result import CheckResult, ResultOutcome
+from tmt.result import CheckResult, ResultOutcome, save_failures
 from tmt.utils import (
     CommandOutput,
     Path,
@@ -22,6 +19,7 @@ from tmt.utils import (
     format_timestamp,
     render_command_report,
 )
+from tmt.utils.hints import hints_as_notes
 
 if TYPE_CHECKING:
     import tmt.base
@@ -229,7 +227,7 @@ def create_final_report(
     invocation: 'TestInvocation',
     check: 'AvcCheck',
     logger: tmt.log.Logger,
-) -> tuple[ResultOutcome, Path]:
+) -> tuple[ResultOutcome, list[Path]]:
     """
     Collect the data, evaluate and create the final report
     """
@@ -249,6 +247,7 @@ def create_final_report(
     # Collect all report components
     report_timestamp = datetime.datetime.now(datetime.timezone.utc)
     report: list[str] = []
+    failures: list[str] = []
 
     # Flags indicating whether we were able to successfully fetch report components
     got_sestatus, got_rpm, got_ausearch, got_denials = False, False, False, False
@@ -264,7 +263,9 @@ def create_final_report(
         report += _report_success('sestatus', output)
 
     else:
-        report += _report_failure('sestatus', exc)
+        failure = _report_failure('sestatus', exc)
+        report += failure
+        failures.append('\n'.join(failure))
 
     # Record NVRs of interesting packages.
     interesting_packages = ' '.join(INTERESTING_PACKAGES)
@@ -280,7 +281,9 @@ def create_final_report(
         report += _report_success(f'rpm -q {interesting_packages}', output)
 
     else:
-        report += _report_failure(f'rpm -q {interesting_packages}', exc)
+        failure = _report_failure(f'rpm -q {interesting_packages}', exc)
+        report += failure
+        failures.append('\n'.join(failure))
 
     # Finally, run `ausearch`, to list AVC denials from the time the test started.
     script = ShellScript(
@@ -297,13 +300,18 @@ def create_final_report(
         got_ausearch = True
         got_denials = True
 
-        report += list(render_command_report(label='ausearch', output=output))
+        failure = list(render_command_report(label='ausearch', output=output))
+        report += failure
+        failures.append('\n'.join(failure))
 
     else:
+        failure = _report_failure('ausearch', exc)
+        report += failure
+
         if exc.returncode == 1 and exc.stderr and '<no matches>' in exc.stderr.strip():
             got_ausearch = True
-
-        report += _report_failure('ausearch', exc)
+        else:
+            failures.append('\n'.join(failure))
 
     # If we were able to fetch all components successfully, pick the result based on `ausearch`
     # output.
@@ -314,9 +322,14 @@ def create_final_report(
     else:
         outcome = ResultOutcome.ERROR
 
+    assert invocation.phase.step.workdir is not None  # narrow type
     report_filepath = _save_report(invocation, report, report_timestamp, append=True)
+    paths = [
+        report_filepath.relative_to(invocation.phase.step.workdir),
+        save_failures(invocation, invocation.check_files_path, failures),
+    ]
 
-    return outcome, report_filepath
+    return outcome, paths
 
 
 @container
@@ -357,7 +370,14 @@ class AvcCheck(Check):
         return spec
 
 
-@provides_check('avc')
+@provides_check(
+    'avc',
+    hints={
+        'detection-skipped': """
+            The detection of AVC denials was skipped because the guest was not compatible.
+            """,
+    },
+)
 class AvcDenials(CheckPlugin[AvcCheck]):
     #
     # This plugin docstring has been reviewed and updated to follow
@@ -437,17 +457,27 @@ class AvcDenials(CheckPlugin[AvcCheck]):
         logger: tmt.log.Logger,
     ) -> list[CheckResult]:
         if not invocation.guest.facts.has_selinux:
-            return [CheckResult(name='avc', result=ResultOutcome.SKIP)]
+            return [
+                CheckResult(
+                    name='avc',
+                    result=ResultOutcome.SKIP,
+                    note=hints_as_notes(
+                        'test-checks/avc/detection-skipped', 'selinux-not-available'
+                    ),
+                )
+            ]
 
         if not invocation.is_guest_healthy:
-            return [CheckResult(name='dmesg', result=ResultOutcome.SKIP)]
+            return [
+                CheckResult(
+                    name='avc',
+                    result=ResultOutcome.SKIP,
+                    note=hints_as_notes('guest-not-healthy'),
+                )
+            ]
 
         assert invocation.phase.step.workdir is not None  # narrow type
 
-        outcome, path = create_final_report(invocation, check, logger)
+        outcome, paths = create_final_report(invocation, check, logger)
 
-        return [
-            CheckResult(
-                name='avc', result=outcome, log=[path.relative_to(invocation.phase.step.workdir)]
-            )
-        ]
+        return [CheckResult(name='avc', result=outcome, log=paths)]
