@@ -4,16 +4,13 @@ from time import sleep
 from typing import TYPE_CHECKING, Optional
 
 import tmt.log
-import tmt.steps.execute
-import tmt.steps.provision
 import tmt.utils
 from tmt.checks import Check, CheckPlugin, _RawCheck, provides_check
 from tmt.container import container, field
-from tmt.result import CheckResult, ResultOutcome
+from tmt.result import CheckResult, ResultOutcome, save_failures
 from tmt.utils import Command, Path, ShellScript
 
 if TYPE_CHECKING:
-    import tmt.base
     from tmt.steps.execute import TestInvocation
     from tmt.steps.provision import Guest
 
@@ -81,8 +78,9 @@ class CoredumpCheck(Check):
             guest.execute(
                 ShellScript("mkdir -p /etc/systemd/coredump.conf.d")
                 & ShellScript(
-                    f"echo '{COREDUMP_CONFIG}' > /etc/systemd/coredump.conf.d/50-tmt.conf"
-                )
+                    f"echo '{COREDUMP_CONFIG}' > /etc/systemd/coredump.conf.d/50-tmt.conf",
+                ),
+                silent=True,
             )
             return
         except tmt.utils.RunError:
@@ -96,7 +94,8 @@ class CoredumpCheck(Check):
                     & ShellScript(
                         f"echo '{COREDUMP_CONFIG}' | sudo tee "
                         "/etc/systemd/coredump.conf.d/50-tmt.conf > /dev/null"
-                    )
+                    ),
+                    silent=True,
                 )
                 logger.debug("Configured coredump with sudo")
                 return
@@ -263,15 +262,15 @@ class CoredumpCheck(Check):
 
         logger.debug(f"Timed out after {total_wait}s waiting for systemd-coredump processes")
 
-    def _check_for_crashes(
+    def _get_crashes(
         self,
         guest: "Guest",
         logger: tmt.log.Logger,
         check_files_path: Path,
         previous_dumps_file: Path,
-    ) -> bool:
+    ) -> list[str]:
         """
-        Check if any new crashes have been detected since the test started.
+        Get any new crashes that have been detected since the test started.
 
         Uses the timestamp of the latest coredump before the test to identify
         new coredumps created during test execution.
@@ -312,13 +311,13 @@ class CoredumpCheck(Check):
                 cmd = f"{sudo_prefix}coredumpctl list --all --no-legend --no-pager"
                 logger.debug("No prior coredumps found, checking all available coredumps")
 
-            output = guest.execute(ShellScript(cmd)).stdout
+            output = guest.execute(ShellScript(cmd), silent=True).stdout
 
             if not output:
-                return False
+                return []
 
             # Process each crash entry
-            has_new_crashes = False
+            crashes: list[str] = []
             for line in output.splitlines():
                 fields = line.split()
                 if len(fields) < 10:  # Ensure we have enough fields
@@ -380,13 +379,13 @@ class CoredumpCheck(Check):
                     logger.debug(f"Skipping dump for PID {pid}, corefile status: {corefile}")
 
                 # This is a new, non-ignored crash
-                has_new_crashes = True
+                crashes.append(crash_info)
 
-            return has_new_crashes
+            return crashes
 
         except tmt.utils.RunError as exc:
             logger.debug(f"Failed to check for crashes: {exc}")
-            return False
+            return []
 
     def _check_coredump(
         self, invocation: "TestInvocation", logger: tmt.log.Logger
@@ -395,12 +394,12 @@ class CoredumpCheck(Check):
         Check coredump status and return appropriate result.
 
         :returns: A tuple of (outcome, log_files) where log_files is a list of
-                 paths to files with coredump information.
+                 paths to files with coredump information and potential failures.
         """
         log_files: list[Path] = []
 
         # Check for crashes by comparing with the saved file of previous dumps
-        has_crashes = self._check_for_crashes(
+        crashes = self._get_crashes(
             invocation.guest,
             logger,
             invocation.check_files_path,
@@ -422,7 +421,9 @@ class CoredumpCheck(Check):
         except tmt.utils.RunError:
             logger.debug("Failed to list coredump files")
 
-        if has_crashes:
+        log_files.append(save_failures(invocation, invocation.check_files_path, crashes))
+
+        if crashes:
             return ResultOutcome.FAIL, log_files
 
         return ResultOutcome.PASS, log_files
