@@ -4,19 +4,19 @@ from re import Pattern
 from typing import TYPE_CHECKING, Optional
 
 import tmt.log
-import tmt.steps.execute
 import tmt.steps.provision
 import tmt.utils
 import tmt.utils.themes
 from tmt.checks import Check, CheckEvent, CheckPlugin, _RawCheck, provides_check
 from tmt.container import container, field
-from tmt.result import CheckResult, ResultOutcome
+from tmt.result import CheckResult, ResultOutcome, save_failures
 from tmt.steps.provision import GuestCapability
 from tmt.utils import (
     Path,
     format_timestamp,
     render_command_report,
 )
+from tmt.utils.hints import hints_as_notes
 
 if TYPE_CHECKING:
     import tmt.base
@@ -63,6 +63,13 @@ class DmesgCheck(Check):
     def to_minimal_spec(self) -> _RawCheck:
         return self.to_spec()
 
+    def _extract_failures(self, text: str) -> list[str]:
+        return [
+            line
+            for line in text.splitlines()
+            if any(pattern.search(line) for pattern in self.failure_pattern)
+        ]
+
     @classmethod
     def _fetch_dmesg(
         cls,
@@ -90,11 +97,11 @@ class DmesgCheck(Check):
         if not guest.facts.is_superuser:
             script = tmt.utils.ShellScript(f'sudo {script.to_shell_command()}')
 
-        return guest.execute(script, log=_test_output_logger)
+        return guest.execute(script, log=_test_output_logger, silent=True)
 
     def _save_dmesg(
         self, invocation: 'TestInvocation', event: CheckEvent, logger: tmt.log.Logger
-    ) -> tuple[ResultOutcome, Path]:
+    ) -> tuple[ResultOutcome, list[Path]]:
         assert invocation.phase.step.workdir is not None  # narrow type
 
         timestamp = format_timestamp(datetime.datetime.now(datetime.timezone.utc))
@@ -102,17 +109,16 @@ class DmesgCheck(Check):
         path = invocation.check_files_path / TEST_POST_DMESG_FILENAME.format(event=event.value)
 
         try:
+            outcome = ResultOutcome.PASS
             output = self._fetch_dmesg(invocation.guest, logger)
 
         except tmt.utils.RunError as exc:
             outcome = ResultOutcome.ERROR
             output = exc.output
 
-        else:
-            outcome = ResultOutcome.PASS
-
-            if any(pattern.search(output.stdout or '') for pattern in self.failure_pattern):
-                outcome = ResultOutcome.FAIL
+        failures = self._extract_failures(output.stdout or '')
+        if failures and outcome == ResultOutcome.PASS:
+            outcome = ResultOutcome.FAIL
 
         invocation.phase.write(
             path,
@@ -120,10 +126,25 @@ class DmesgCheck(Check):
             mode='a',
         )
 
-        return outcome, path.relative_to(invocation.phase.step.workdir)
+        log_paths = [
+            path.relative_to(invocation.phase.step.workdir),
+            save_failures(invocation, invocation.check_files_path, failures),
+        ]
+
+        return outcome, log_paths
 
 
-@provides_check('dmesg')
+@provides_check(
+    'dmesg',
+    hints={
+        'fetching-skipped': """
+            Saving of the kernel ring buffer was skipped because of missing privileges.
+
+            The account, used by tmt, lacks necessary privileges. See ``man 2 syslog``
+            for more details on capabilities required for the kernel ring buffer access.
+            """
+    },
+)
 class Dmesg(CheckPlugin[DmesgCheck]):
     #
     # This plugin docstring has been reviewed and updated to follow
@@ -192,11 +213,16 @@ class Dmesg(CheckPlugin[DmesgCheck]):
         logger: tmt.log.Logger,
     ) -> list[CheckResult]:
         if not invocation.guest.facts.has_capability(GuestCapability.SYSLOG_ACTION_READ_ALL):
-            return [CheckResult(name='dmesg', result=ResultOutcome.SKIP)]
+            return [
+                CheckResult(
+                    name='dmesg',
+                    result=ResultOutcome.SKIP,
+                    note=hints_as_notes('test-checks/dmesg/fetching-skipped'),
+                )
+            ]
 
-        outcome, path = check._save_dmesg(invocation, CheckEvent.BEFORE_TEST, logger)
-
-        return [CheckResult(name='dmesg', result=outcome, log=[path])]
+        outcome, paths = check._save_dmesg(invocation, CheckEvent.BEFORE_TEST, logger)
+        return [CheckResult(name='dmesg', result=outcome, log=paths)]
 
     @classmethod
     def after_test(
@@ -208,11 +234,22 @@ class Dmesg(CheckPlugin[DmesgCheck]):
         logger: tmt.log.Logger,
     ) -> list[CheckResult]:
         if not invocation.guest.facts.has_capability(GuestCapability.SYSLOG_ACTION_READ_ALL):
-            return [CheckResult(name='dmesg', result=ResultOutcome.SKIP)]
+            return [
+                CheckResult(
+                    name='dmesg',
+                    result=ResultOutcome.SKIP,
+                    note=hints_as_notes('test-checks/dmesg/fetching-skipped'),
+                )
+            ]
 
         if not invocation.is_guest_healthy:
-            return [CheckResult(name='dmesg', result=ResultOutcome.SKIP)]
+            return [
+                CheckResult(
+                    name='dmesg',
+                    result=ResultOutcome.SKIP,
+                    note=hints_as_notes('guest-not-healthy'),
+                )
+            ]
 
-        outcome, path = check._save_dmesg(invocation, CheckEvent.AFTER_TEST, logger)
-
-        return [CheckResult(name='dmesg', result=outcome, log=[path])]
+        outcome, paths = check._save_dmesg(invocation, CheckEvent.AFTER_TEST, logger)
+        return [CheckResult(name='dmesg', result=outcome, log=paths)]
