@@ -17,28 +17,27 @@ import fmf.utils
 from click import echo
 
 import tmt
+import tmt._bootstrap
 import tmt.base
-import tmt.cli
 import tmt.config
 import tmt.convert
-import tmt.export
 import tmt.identifier
 import tmt.log
 import tmt.options
 import tmt.plugins
 import tmt.plugins.plan_shapers
+import tmt.policy
 import tmt.steps
 import tmt.templates
-import tmt.trying
 import tmt.utils
 import tmt.utils.jira
-import tmt.utils.rest
 from tmt.cli import CliInvocation, Context, ContextObject, CustomGroup, pass_context
 from tmt.options import Deprecated, create_options_decorator, option
-from tmt.utils import Command, Path, effective_workdir_root
+from tmt.utils import Command, GeneralError, Path, effective_workdir_root
 from tmt.utils.themes import style
 
 if TYPE_CHECKING:
+    import tmt.steps.cleanup
     import tmt.steps.discover
     import tmt.steps.execute
     import tmt.steps.finish
@@ -65,6 +64,52 @@ story_flags_filter_options = create_options_decorator(tmt.options.STORY_FLAGS_FI
 remote_plan_options = create_options_decorator(tmt.options.REMOTE_PLAN_OPTIONS)
 lint_options = create_options_decorator(tmt.options.LINT_OPTIONS)
 environment_options = create_options_decorator(tmt.options.ENVIRONMENT_OPTIONS)
+policy_options = create_options_decorator(tmt.options.POLICY_OPTIONS)
+
+
+def _load_policies(
+    policy_name: Optional[str],
+    policy_path: Optional[Path],
+    policy_root: Optional[Path],
+) -> list[tmt.policy.Policy]:
+    """
+    A helper for loading policies.
+
+    :param policy_name: policy name to look up under the policy root.
+        If set, it is preferred over ``policy_path``, and it requires
+        ``policy_root`` to be set as well.
+    :param policy_path: path to the policy file.
+    :param policy_root: if set, policy files must be located under this
+        directory.
+    """
+
+    if policy_name is not None:
+        if policy_root is None:
+            raise GeneralError(
+                "Policy can be loaded by its name only when '--policy-root' is specified."
+            )
+
+        if policy_path is not None:
+            raise GeneralError(
+                "Options '--policy-name' and '--policy-file' are mutually exclusive."
+            )
+
+        return [
+            tmt.policy.Policy.load_by_name(
+                name=policy_name,
+                root=policy_root,
+            )
+        ]
+
+    if policy_path is not None:
+        return [
+            tmt.policy.Policy.load_by_filepath(
+                path=policy_path,
+                root=policy_root,
+            )
+        ]
+
+    return []
 
 
 # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
@@ -163,7 +208,7 @@ def main(
     #
     # ignore[unused-ignore]: mypy does not report this issue, and the
     # ignore fools mypy into reporting the waiver as unused.
-    tmt.cli.EXCEPTION_LOGGER = logger  # type: ignore[reportConstantRedefinition,unused-ignore]
+    tmt._bootstrap.EXCEPTION_LOGGER = logger  # type: ignore[reportConstantRedefinition,unused-ignore]
 
     # Save click context and fmf context for future use
     tmt.utils.Common.store_cli_invocation(click_contex)
@@ -191,6 +236,7 @@ def main(
         fmf_context=tmt.utils.FmfContext.from_spec('cli', context, logger),
         steps=set(),
         tree=tree,
+        config=tmt.config.Config(logger),
     )
 
     # Show overview of available tests, plans and stories
@@ -297,7 +343,16 @@ def main(
 @verbosity_options
 @force_dry_options
 @again_option
-def run(context: Context, id_: Optional[str], workdir_root: Optional[Path], **kwargs: Any) -> None:
+@policy_options
+def run(
+    context: Context,
+    id_: Optional[str],
+    workdir_root: Optional[Path],
+    policy_file: Optional[Path],
+    policy_name: Optional[str],
+    policy_root: Optional[Path],
+    **kwargs: Any,
+) -> None:
     """
     Run test steps.
     """
@@ -306,14 +361,16 @@ def run(context: Context, id_: Optional[str], workdir_root: Optional[Path], **kw
     logger = context.obj.logger.descend(logger_name='run', extra_shift=0)
     logger.apply_verbosity_options(**kwargs)
 
-    run = tmt.Run(
+    policies = _load_policies(policy_name, policy_file, policy_root)
+
+    context.obj.run = tmt.Run(
         id_=Path(id_) if id_ is not None else None,
         tree=context.obj.tree,
         cli_invocation=CliInvocation.from_context(context),
         workdir_root=effective_workdir_root(workdir_root),
+        policies=policies,
         logger=logger,
     )
-    context.obj.run = run
 
 
 for plugin_class in tmt.plugins.plan_shapers._PLAN_SHAPER_PLUGIN_REGISTRY.iter_plugins():
@@ -327,6 +384,7 @@ run.add_command(tmt.steps.prepare.PreparePlugin.command())
 run.add_command(tmt.steps.execute.ExecutePlugin.command())
 run.add_command(tmt.steps.report.ReportPlugin.command())
 run.add_command(tmt.steps.finish.FinishPlugin.command())
+run.add_command(tmt.steps.cleanup.CleanupPlugin.command())
 run.add_command(tmt.steps.Login.command())
 run.add_command(tmt.steps.Reboot.command())
 
@@ -889,6 +947,7 @@ _test_export_default = 'yaml'
     metavar='PATH',
     help="Path to a template to use for rendering the export. Used with '--how=template' only.",
 )
+@policy_options
 def tests_export(
     context: Context,
     format: str,
@@ -896,6 +955,9 @@ def tests_export(
     nitrate: bool,
     bugzilla: bool,
     template: Optional[str],
+    policy_file: Optional[Path],
+    policy_name: Optional[str],
+    policy_root: Optional[Path],
     **kwargs: Any,
 ) -> None:
     """
@@ -934,9 +996,16 @@ def tests_export(
         )
 
     else:
+        tests = context.obj.tree.tests()
+
+        policies = _load_policies(policy_name, policy_file, policy_root)
+
+        for policy in policies:
+            policy.apply_to_tests(tests=tests, logger=context.obj.logger)
+
         echo(
             tmt.Test.export_collection(
-                collection=context.obj.tree.tests(),
+                collection=tests,
                 format=how,
                 template=Path(template) if template else None,
             )
