@@ -11,7 +11,8 @@ from typing import TYPE_CHECKING, Any, Callable, Generic, Optional, TypeVar, Uni
 
 import fmf
 
-from tmt._compat.pydantic import BaseModel, Extra, ValidationError
+from tmt._compat.pydantic import PYDANTIC_V1, BaseModel, ConfigDict, Field, ValidationError
+from tmt._compat.typing import Self
 
 if TYPE_CHECKING:
     from _typeshed import DataclassInstance
@@ -98,7 +99,7 @@ class FieldMetadata(Generic[T]):
 
     #: Specific values that should be shown in the documentation as
     #: interesting examples of the field usage.
-    help_example_values: list[str] = simple_field(default_factory=list)
+    help_example_values: list[str] = simple_field(default_factory=list[str])
 
     #: If field accepts a value, this string would represent it in documentation.
     #: This stores the metavar provided when field was created - it may be unset.
@@ -153,7 +154,7 @@ class FieldMetadata(Generic[T]):
 
     #: CLI option parameters, for lazy option creation.
     _option_args: Optional['FieldCLIOption'] = None
-    _option_kwargs: dict[str, Any] = simple_field(default_factory=dict)
+    _option_kwargs: dict[str, Any] = simple_field(default_factory=dict[str, Any])
 
     #: A :py:func:`click.option` decorator defining a corresponding CLI option.
     _option: Optional['tmt.options.ClickOptionDecoratorType'] = None
@@ -165,7 +166,7 @@ class FieldMetadata(Generic[T]):
         """
 
         if isinstance(self._choices, (list, tuple)):
-            return list(self._choices)
+            return list(cast(Sequence[str], self._choices))
 
         if callable(self._choices):
             return self._choices()
@@ -448,7 +449,7 @@ SpecOutT = TypeVar('SpecOutT')
 @container
 class SpecBasedContainer(Generic[SpecInT, SpecOutT], DataContainer):
     @classmethod
-    def from_spec(cls: type[SpecBasedContainerT], spec: SpecInT) -> SpecBasedContainerT:
+    def from_spec(cls, spec: SpecInT) -> Self:
         """
         Convert from a specification file or from a CLI option
 
@@ -513,9 +514,7 @@ class SerializableContainer(DataContainer):
             setattr(obj, name, value)
 
     @classmethod
-    def extract_from(
-        cls: type[SerializableContainerDerivedType], obj: Any
-    ) -> SerializableContainerDerivedType:
+    def extract_from(cls, obj: Any) -> Self:
         """
         Extract keys from given object, and save them in a container
         """
@@ -566,9 +565,7 @@ class SerializableContainer(DataContainer):
         return serialized
 
     @classmethod
-    def from_serialized(
-        cls: type[SerializableContainerDerivedType], serialized: dict[str, Any]
-    ) -> SerializableContainerDerivedType:
+    def from_serialized(cls, serialized: dict[str, Any]) -> Self:
         """
         Convert from a serialized form loaded from a file.
 
@@ -601,6 +598,18 @@ class SerializableContainer(DataContainer):
 
         return cls(**dict(_produce_unserialized()))
 
+    @classmethod
+    def unserialize_class(cls) -> Any:
+        """
+        Provide the actual class to unserialize.
+
+        In some situations, the class recorded in the serialized state
+        cannot be safely unserialized. Such classes may reimplement this
+        method, and return the right class.
+        """
+
+        return cls
+
     # ignore[misc,type-var]: mypy is correct here, method does return a
     # TypeVar, but there is no way to deduce the actual type, because
     # the method is static. That's on purpose, method tries to find the
@@ -653,6 +662,10 @@ class SerializableContainer(DataContainer):
         # honor promise given by return value annotation.
         assert issubclass(klass, SerializableContainer)
 
+        klass = klass.unserialize_class()
+
+        assert issubclass(klass, SerializableContainer)
+
         # Apparently, the issubclass() check above is not good enough for mypy.
         return cast(SerializableContainerDerivedType, klass.from_serialized(serialized))
 
@@ -865,7 +878,9 @@ def field(
             'tmt': FieldMetadata(
                 internal=internal,
                 help=textwrap.dedent(help).strip() if help else None,
-                help_example_values=help_example_values or [],
+                help_example_values=help_example_values
+                if help_example_values is not None
+                else list[str](),
                 _metavar=metavar,
                 default=default,
                 default_factory=default_factory,
@@ -904,25 +919,49 @@ MetadataContainerT = TypeVar(
     bound='MetadataContainer',
 )
 
+metadata_field = Field
+
 
 class MetadataContainer(BaseModel):
     """
     A base class of containers backed by fmf nodes.
     """
 
-    class Config:
-        # Accept only keys with dashes instead of underscores
-        alias_generator = key_to_option
-        extra = Extra.forbid
-        validate_all = True
-        validate_assignment = True
+    if PYDANTIC_V1:
+
+        class Config:
+            # Accept only keys with dashes instead of underscores
+            alias_generator = key_to_option
+            extra = "forbid"
+            validate_all = True
+            validate_assignment = True
+
+    else:
+        model_config = ConfigDict(
+            alias_generator=key_to_option,
+            extra="forbid",
+            validate_default=True,
+            validate_assignment=True,
+        )
 
     @classmethod
-    def from_fmf(cls: type[MetadataContainerT], tree: fmf.Tree) -> MetadataContainerT:
+    def from_fmf(cls, tree: fmf.Tree) -> Self:
         try:
-            return cls.parse_obj(tree.data)
+            return cls.model_validate(tree.data)
 
         except ValidationError as error:
             import tmt.utils
 
             raise tmt.utils.SpecificationError(f"Invalid metadata in '{tree.name}'.") from error
+
+    @classmethod
+    def from_yaml(cls, yaml: str) -> Self:
+        import tmt.utils
+
+        try:
+            return cls.model_validate(tmt.utils.yaml_to_dict(yaml))
+
+        except ValidationError as error:
+            import tmt.utils
+
+            raise tmt.utils.SpecificationError("Invalid metadata in YAML data.") from error
