@@ -27,7 +27,6 @@ managing handlers themselves, which would be very messy given the propagation of
 import enum
 import itertools
 import logging
-import logging.handlers
 import os
 import sys
 from typing import (
@@ -66,6 +65,7 @@ class Topic(enum.Enum):
     COMMAND_EVENTS = 'command-events'
     ADJUST_DECISIONS = 'adjust-decisions'
     HELP_RENDERING = 'help-rendering'
+    POLICY = 'policy'
 
 
 DEFAULT_TOPICS: set[Topic] = set()
@@ -271,7 +271,7 @@ class LogRecordDetails:
     color: 'tmt.utils.themes.Style' = None
     shift: int = 0
 
-    logger_labels: list[str] = simple_field(default_factory=list)
+    logger_labels: list[str] = simple_field(default_factory=list[str])
     logger_labels_padding: int = 0
 
     logger_verbosity_level: int = 0
@@ -283,7 +283,7 @@ class LogRecordDetails:
     logger_quiet: bool = False
     ignore_quietness: bool = False
 
-    logger_topics: set[Topic] = simple_field(default_factory=set)
+    logger_topics: set[Topic] = simple_field(default_factory=set[Topic])
     message_topic: Optional[Topic] = None
 
 
@@ -769,7 +769,6 @@ class Logger:
         self,
         text: str,
         color: 'tmt.utils.themes.Style' = None,
-        shift: int = 0,
     ) -> str:
         """
         Format the given text in a way suitable for :py:meth:`print`
@@ -779,7 +778,7 @@ class Logger:
             text,
             # Always apply colors - message can be decolorized later.
             color=color,
-            level=shift + self._base_shift,
+            level=0,
             labels=self.labels,
             labels_padding=self.labels_padding,
         )
@@ -790,12 +789,11 @@ class Logger:
         self,
         text: str,
         color: 'tmt.utils.themes.Style' = None,
-        shift: int = 0,
         file: Optional[TextIO] = None,
     ) -> None:
         file = file or sys.stdout
 
-        print(self.print_format(text, color=color, shift=shift), file=file)
+        print(self.print_format(text, color=color), file=file)
 
     def info(
         self,
@@ -803,8 +801,12 @@ class Logger:
         value: Optional[LoggableValue] = None,
         color: 'tmt.utils.themes.Style' = None,
         shift: int = 0,
+        topic: Optional[Topic] = None,
     ) -> None:
-        self._log(logging.INFO, LogRecordDetails(key=key, value=value, color=color, shift=shift))
+        self._log(
+            logging.INFO,
+            LogRecordDetails(key=key, value=value, color=color, shift=shift, message_topic=topic),
+        )
 
     def verbose(
         self,
