@@ -54,7 +54,7 @@ class Deprecated:
 class Path(click.ParamType):
     name = 'path'
 
-    def convert(
+    def convert(  # noqa: RET503
         self,
         value: Any,
         param: Optional[click.Parameter],
@@ -71,7 +71,7 @@ class Path(click.ParamType):
         The ``param`` and ``ctx`` arguments may be ``None`` in certain
         situations, such as when converting prompt input.
 
-        If the value cannot be converted, call :meth:`fail` with a
+        If the value cannot be converted, call :py:meth:`fail` with a
         descriptive message.
 
         :param value: The value to convert.
@@ -100,8 +100,7 @@ class Path(click.ParamType):
                 ctx=ctx,
             )
 
-        # RET503: ruff does not recognize NoReturn annotation of `self.fail`.
-        self.fail(  # noqa: RET503
+        self.fail(
             f"Field '{param.name}' must be a path-like string, '{type(value).__name__}' found.",
             param=param,
             ctx=ctx,
@@ -524,6 +523,45 @@ ENVIRONMENT_OPTIONS: list[ClickOptionDecoratorType] = [
     ),
 ]
 
+POLICY_OPTIONS: list[ClickOptionDecoratorType] = [
+    option(
+        '--policy-file',
+        metavar='PATH',
+        type=Path(),
+        envvar='TMT_POLICY_FILE',
+        help="""
+             Location of a file with policy rules for modification of
+             test metadata keys. Both absolute and relative paths are
+             accepted; a relative path is interpreted either against the
+             current working directory, or against the policy root
+             directory if it was specified.
+             """,
+    ),
+    option(
+        '--policy-name',
+        metavar='NAME',
+        envvar='TMT_POLICY_NAME',
+        help="""
+             Name of the file with policy rules for modification of test
+             metadata keys. The name would be extended with ``.yaml``
+             suffix, and tmt will try to locate it under the policy root
+             directory.
+             """,
+    ),
+    option(
+        '--policy-root',
+        metavar='PATH',
+        type=Path(),
+        envvar='TMT_POLICY_ROOT',
+        help="""
+             If set, policy files must be located under this directory.
+             Policy specified by its name is searched under this
+             directory only, and policy specified by its filepath will
+             be allowed only if the filepath is under this directory.
+             """,
+    ),
+]
+
 
 def create_options_decorator(options: list[ClickOptionDecoratorType]) -> Callable[[FC], FC]:
     def common_decorator(fn: FC) -> FC:
@@ -663,16 +701,16 @@ def create_method_class(methods: MethodDictType) -> type[click.Command]:
                         break
 
             if how and self._method is None:
-                from tmt.utils.hints import print_hint
+                from tmt.utils.hints import print_hints
 
                 # Use run for logging, steps may not be initialized yet
                 assert context.obj.run is not None  # narrow type
                 assert self.name is not None  # narrow type
 
-                print_hint(
-                    id_=f'{self.name}/{how}', ignore_missing=True, logger=context.obj.run._logger
+                print_hints(
+                    f'{self.name}/{how}', ignore_missing=True, logger=context.obj.run._logger
                 )
-                print_hint(id_=self.name, ignore_missing=True, logger=context.obj.run._logger)
+                print_hints(self.name, ignore_missing=True, logger=context.obj.run._logger)
 
                 raise tmt.utils.SpecificationError(f"Unsupported {self.name} method '{how}'.")
 
