@@ -14,7 +14,7 @@ from typing import Any, Generic, Optional, TypeVar, cast
 import tmt
 import tmt.utils
 from tmt.log import Logger
-from tmt.utils import Path
+from tmt.utils import GeneralError, Path
 
 ModuleT = TypeVar('ModuleT', bound=ModuleType)
 
@@ -75,6 +75,7 @@ def _discover_packages() -> list[tuple[str, Path]]:
         ('tmt.export', Path('export')),
         ('tmt.frameworks', Path('frameworks')),
         ('tmt.checks', Path('checks')),
+        ('tmt.checks.internal', Path('checks/internal')),
         ('tmt.package_managers', Path('package_managers')),
         ('tmt.steps.prepare.feature', Path('steps/prepare/feature')),
         ('tmt.plugins.plan_shapers', Path('plugins/plan_shapers')),
@@ -271,9 +272,9 @@ def _import_or_raise(
 
     except tmt.utils.GeneralError as exc:
         if hint_id is not None:
-            from tmt.utils.hints import print_hint
+            from tmt.utils.hints import print_hints
 
-            print_hint(id_=hint_id, logger=logger)
+            print_hints(hint_id, logger=logger)
 
         if exc_message is not None:
             raise exc_class(exc_message) from exc
@@ -305,7 +306,7 @@ def import_module(
 
     return _import_or_raise(
         module=module,
-        exc_class=SystemExit,
+        exc_class=GeneralError,
         exc_message=f"Failed to import the '{module}' module from '{path}'.",
         logger=logger,
     )
@@ -343,7 +344,9 @@ class PluginRegistry(Generic[RegisterableT]):
 
     _plugins: dict[str, RegisterableT]
 
-    def __init__(self) -> None:
+    def __init__(self, name: str) -> None:
+        self.name = name
+
         self._plugins = {}
 
     def register_plugin(
@@ -408,6 +411,12 @@ class PluginRegistry(Generic[RegisterableT]):
     def items(self) -> Iterator[tuple[str, RegisterableT]]:
         yield from self._plugins.items()
 
+    def __len__(self) -> int:
+        return len(self._plugins)
+
+    def __bool__(self) -> bool:
+        return bool(self._plugins)
+
 
 class ModuleImporter(Generic[ModuleT]):
     """
@@ -437,3 +446,42 @@ class ModuleImporter(Generic[ModuleT]):
 
         assert self._module  # narrow type
         return self._module
+
+
+def iter_plugin_registries() -> Iterator[PluginRegistry[Any]]:
+    # TODO: maybe there is a better way, but as of now, registries do
+    # not report their existence, there is no method to iterate over
+    # them. Using a static list for now.
+    from tmt.base import Plan, Story, Test
+    from tmt.checks import _CHECK_PLUGIN_REGISTRY
+    from tmt.frameworks import _FRAMEWORK_PLUGIN_REGISTRY
+    from tmt.package_managers import _PACKAGE_MANAGER_PLUGIN_REGISTRY
+    from tmt.plugins.plan_shapers import _PLAN_SHAPER_PLUGIN_REGISTRY
+    from tmt.steps.discover import DiscoverPlugin
+    from tmt.steps.execute import ExecutePlugin
+    from tmt.steps.finish import FinishPlugin
+    from tmt.steps.prepare import PreparePlugin
+    from tmt.steps.prepare.feature import _FEATURE_PLUGIN_REGISTRY
+    from tmt.steps.provision import ProvisionPlugin
+    from tmt.steps.report import ReportPlugin
+
+    yield Story._export_plugin_registry
+    yield Plan._export_plugin_registry
+    yield Test._export_plugin_registry
+    yield _CHECK_PLUGIN_REGISTRY
+    yield _FRAMEWORK_PLUGIN_REGISTRY
+    yield _PLAN_SHAPER_PLUGIN_REGISTRY
+    yield _PACKAGE_MANAGER_PLUGIN_REGISTRY
+    yield _FEATURE_PLUGIN_REGISTRY
+    yield DiscoverPlugin._supported_methods
+    yield ProvisionPlugin._supported_methods
+    yield PreparePlugin._supported_methods
+    yield ExecutePlugin._supported_methods
+    yield FinishPlugin._supported_methods
+    yield ReportPlugin._supported_methods
+
+
+def iter_plugins() -> Iterator[tuple[PluginRegistry[RegisterableT], str, RegisterableT]]:
+    for registry in iter_plugin_registries():
+        for plugin_id, plugin_class in registry.items():
+            yield registry, plugin_id, plugin_class
