@@ -1,5 +1,4 @@
 import enum
-import re
 from typing import TYPE_CHECKING, Any, Callable, Optional, cast
 
 import fmf
@@ -17,7 +16,6 @@ from tmt.utils.themes import style
 if TYPE_CHECKING:
     import tmt.base
     import tmt.steps.execute
-    import tmt.steps.provision
 
 # Extra keys used for identification in Result class
 EXTRA_RESULT_IDENTIFICATION_KEYS = ['extra-nitrate', 'extra-task']
@@ -224,6 +222,19 @@ class BaseResult(SerializableContainer):
     def printable_note(self) -> str:
         return ', '.join(self.note)
 
+    @property
+    def failure_logs(self) -> list[Path]:
+        """
+        Return paths to all failure logs from the result
+        """
+
+        if self.result not in (ResultOutcome.FAIL, ResultOutcome.ERROR, ResultOutcome.WARN):
+            return []
+
+        return list(
+            {path for path in self.log if path.name == tmt.steps.execute.TEST_FAILURES_FILENAME}
+        )
+
 
 @container
 class CheckResult(BaseResult):
@@ -270,12 +281,16 @@ class SubResult(BaseResult):
         ],
     )
 
-    @staticmethod
-    def failures(log: Optional[str], msg_type: str = 'FAIL') -> str:
+    @property
+    def failure_logs(self) -> list[Path]:
         """
-        Filter stdout and get only messages with certain type
+        Return paths to all failure logs from the result
         """
-        return Result.failures(log, msg_type)
+
+        failure_logs = super().failure_logs
+        for check in self.check:
+            failure_logs += check.failure_logs
+        return list(set(failure_logs))
 
 
 @container
@@ -366,7 +381,7 @@ class Result(BaseResult):
         default_ids: ResultIds = {tmt.identifier.ID_KEY: invocation.test.id}
 
         for key in EXTRA_RESULT_IDENTIFICATION_KEYS:
-            value: Any = invocation.test.node.get(key)
+            value: Any = cast(Any, invocation.test.node.get(key))
 
             default_ids[key] = None if value is None else str(value)
 
@@ -377,7 +392,7 @@ class Result(BaseResult):
             name=invocation.test.name,
             serial_number=invocation.test.serial_number,
             fmf_id=invocation.test.fmf_id,
-            context=invocation.phase.step.plan._fmf_context,
+            context=invocation.phase.step.plan.fmf_context,
             result=result,
             note=note or [],
             start_time=invocation.start_time,
@@ -461,9 +476,10 @@ class Result(BaseResult):
             return self
 
         # Interpret check results (aggregated by the check name)
-        check_outcomes: list[ResultOutcome] = []
-        for check_name in tmt.utils.uniq([check.name for check in self.check]):
-            check_outcomes.append(self.interpret_check_result(check_name, interpret_checks))
+        check_outcomes: list[ResultOutcome] = [
+            self.interpret_check_result(check_name, interpret_checks)
+            for check_name in tmt.utils.uniq([check.name for check in self.check])
+        ]
 
         # Aggregate check results with the main test result
         self.result = ResultOutcome.reduce([self.result, *check_outcomes])
@@ -513,7 +529,7 @@ class Result(BaseResult):
         Return dictionary with total stats for given results
         """
 
-        stats = {result: 0 for result in RESULT_OUTCOME_COLORS}
+        stats = dict.fromkeys(RESULT_OUTCOME_COLORS, 0)
 
         for result in results:
             stats[result.result] += 1
@@ -575,57 +591,16 @@ class Result(BaseResult):
 
         return ' '.join(components)
 
-    @staticmethod
-    def failures(log: Optional[str], msg_type: str = 'FAIL') -> str:
+    @property
+    def failure_logs(self) -> list[Path]:
         """
-        Filter stdout and get only messages with certain type
+        Return paths to all failure logs from the result
         """
 
-        if not log:
-            return ''
-        filtered = ''
-
-        # Filter beakerlib style logs in the following way:
-        # 1. Reverse the log string by lines
-        # 2. Search for each FAIL and extract every associated line.
-        # 3. For failed phases also extract phase name so the log is easier to understand
-        # 4. Reverse extracted lines back into correct order.
-        if re.search(':: \\[   FAIL   \\] ::', log):  # dumb check for a beakerlib log
-            copy_line = False
-            copy_phase_name = False
-            failure_log: list[str] = []
-            # we will be processing log lines in a reversed order
-            iterator = iter(reversed(log.split("\n")))
-            for line in iterator:
-                # found FAIL enables log extraction
-                if re.search(':: \\[   FAIL   \\] ::', line):
-                    copy_line = True
-                    copy_phase_name = True
-                # BEGIN of rlRun block or previous command or beginning of a test section
-                # disables extraction
-                elif re.search('(:: \\[.{10}\\] ::|[:]{80})', line):
-                    copy_line = False
-                # extract line from the log
-                if copy_line:
-                    failure_log.append(line)
-                # Add beakerlib phase name to a failure log, in order to properly match the phase
-                # name we need to do this in two steps.
-                if copy_phase_name and re.search('[:]{80}', line):
-                    # read the next line containing phase name
-                    line = next(iterator)
-                    failure_log.append(f'\n{line}')
-                    copy_phase_name = False
-            # reverse extracted lines to restore previous order
-            failure_log.reverse()
-            return '\n'.join(failure_log).strip()
-
-        # Check for other failures and errors when not using beakerlib
-        for m in re.findall(
-            rf'.*\b(?=error|fail|{msg_type})\b.*', log, re.IGNORECASE | re.MULTILINE
-        ):
-            filtered += m + '\n'
-
-        return filtered or log
+        failure_logs = super().failure_logs
+        for check in self.check:
+            failure_logs += check.failure_logs
+        return list(set(failure_logs))
 
 
 def results_to_exit_code(results: list[Result], execute_enabled: bool = True) -> int:
@@ -670,3 +645,28 @@ def results_to_exit_code(results: list[Result], execute_enabled: bool = True) ->
         return TmtExitCode.SUCCESS
 
     raise GeneralError("Unhandled combination of test result.")
+
+
+def save_failures(
+    invocation: 'tmt.steps.execute.TestInvocation', directory: Path, failures: list[str]
+) -> Path:
+    """
+    Save test failures to a file.
+
+    :param invocation: test invocation.
+    :param directory: directory to save the file in.
+    :param failures: list of failures to save.
+    """
+
+    path = directory / tmt.steps.execute.TEST_FAILURES_FILENAME
+
+    try:
+        existing_failures = tmt.utils.yaml_to_list(invocation.phase.read(path))
+    except tmt.utils.FileError:
+        existing_failures = []
+
+    existing_failures += failures
+
+    invocation.phase.write(path, tmt.utils.dict_to_yaml(existing_failures))
+    assert invocation.phase.step.workdir is not None  # narrow type
+    return path.relative_to(invocation.phase.step.workdir)
