@@ -37,6 +37,7 @@ import tmt.options
 import tmt.queue
 import tmt.utils
 import tmt.utils.rest
+from tmt._compat.typing import Self
 from tmt.container import (
     SerializableContainer,
     SpecBasedContainer,
@@ -46,6 +47,7 @@ from tmt.container import (
     field,
     key_to_option,
     option_to_key,
+    simple_field,
 )
 from tmt.options import option
 from tmt.utils import (
@@ -65,7 +67,7 @@ if TYPE_CHECKING:
     import tmt.steps.discover
     import tmt.steps.execute
     from tmt.base import Plan
-    from tmt.result import BaseResult
+    from tmt.result import BaseResult, PhaseResult
     from tmt.steps.provision import Guest
 
 
@@ -97,7 +99,7 @@ PHASE_ORDER_PREPARE_INSTALL_REQUIRES = 70
 PHASE_ORDER_PREPARE_INSTALL_RECOMMENDS = 75
 
 # Supported steps and actions
-STEPS: list[str] = ['discover', 'provision', 'prepare', 'execute', 'report', 'finish']
+STEPS: list[str] = ['discover', 'provision', 'prepare', 'execute', 'report', 'finish', 'cleanup']
 ACTIONS: list[str] = ['login', 'reboot']
 DEFAULT_LOGIN_COMMAND = 'bash'
 
@@ -194,7 +196,11 @@ def prune_directory(path: Path, preserved_members: set[str], logger: tmt.log.Log
                 shutil.rmtree(member)
 
         except OSError as error:
-            logger.warning(f"Unable to remove '{member}': {error}")
+            tmt.utils.show_exception_as_warning(
+                exception=error,
+                message=f"Unable to remove '{member}'.",
+                logger=logger,
+            )
 
     return actual_preserved_members
 
@@ -408,10 +414,10 @@ class StepData(
     # ignore[override]: expected, we need to accept one extra parameter, `logger`.
     @classmethod
     def from_spec(  # type: ignore[override]
-        cls: type[StepDataT],
+        cls,
         raw_data: _RawStepData,
         logger: tmt.log.Logger,
-    ) -> StepDataT:
+    ) -> Self:
         """
         Convert from a specification file or from a CLI option
         """
@@ -906,6 +912,16 @@ class Step(tmt.utils.MultiInvokableCommon, tmt.export.Exportable['Step']):
 
         self._raw_data = raw_data
 
+    def suspend(self) -> None:
+        """
+        Suspend the step.
+
+        Perform any actions necessary before quitting the step and tmt.
+        The step may be revisited by future tmt invocations.
+        """
+
+        self.debug(f"Suspending step '{self.name}'.")
+
     def _apply_cli_invocations(self, raw_data: list[_RawStepData]) -> list[_RawStepData]:
         # Override step data with command line options
         #
@@ -1410,6 +1426,19 @@ def provides_method(
     return _method
 
 
+@container
+class PluginOutcome:
+    """
+    Bundles together results produced by a plugin, and possible error.
+    """
+
+    #: Results produced by the plugin invocation.
+    results: list['PhaseResult'] = simple_field(default_factory=list['PhaseResult'])
+
+    #: Exceptions the plugin would like to raise and report.
+    exceptions: list[Exception] = simple_field(default_factory=list[Exception])
+
+
 class BasePlugin(Phase, Generic[StepDataT, PluginReturnValueT]):
     """
     Common parent of all step plugins
@@ -1698,10 +1727,10 @@ class BasePlugin(Phase, Generic[StepDataT, PluginReturnValueT]):
                 assert isinstance(plugin, BasePlugin)
                 return plugin
 
-        from tmt.utils.hints import print_hint
+        from tmt.utils.hints import print_hints
 
-        print_hint(id_=f'{step.name}/{how}', ignore_missing=True, logger=step._logger)
-        print_hint(id_=step.name, ignore_missing=True, logger=step._logger)
+        print_hints(f'{step.name}/{how}', ignore_missing=True, logger=step._logger)
+        print_hints(step.name, ignore_missing=True, logger=step._logger)
 
         # Report invalid method
         if step.plan is None:
@@ -1841,7 +1870,7 @@ class BasePlugin(Phase, Generic[StepDataT, PluginReturnValueT]):
         Check if the plugin is enabled by 'when' keyword
         """
 
-        fmf_context = fmf.context.Context(**self.step.plan._fmf_context)
+        fmf_context = fmf.context.Context(**self.step.plan.fmf_context)
         when_rules = self.get('when', [])
         if not when_rules:
             # No 'when' -> enabled everywhere
@@ -1959,7 +1988,11 @@ class BasePlugin(Phase, Generic[StepDataT, PluginReturnValueT]):
             try:
                 shutil.rmtree(self.workdir)
             except OSError as error:
-                logger.warning(f"Unable to remove '{self.workdir}': {error}")
+                tmt.utils.show_exception_as_warning(
+                    exception=error,
+                    message=f"Unable to remove '{self.workdir}'.",
+                    logger=logger,
+                )
 
 
 class GuestlessPlugin(BasePlugin[StepDataT, PluginReturnValueT]):
@@ -2038,8 +2071,9 @@ class Action(Phase, tmt.utils.MultiInvokableCommon):
                 steps: list[Step] = [s for s in step.plan.steps() if s.status() == 'done']
                 login_during = steps[-1] if steps else None
             # Default to the last enabled step if no completed step found
+            # (skip the cleanup step as no actions are expected there)
             if login_during is None:
-                login_during = list(step.plan.steps())[-1]
+                login_during = list(step.plan.steps(skip=["cleanup"]))[-1]
             # Only login if the error occurred after provision
             if login_during != step.plan.discover:
                 phases[login_during.name] = [PHASE_END]
@@ -2114,8 +2148,11 @@ class Reboot(Action):
         @option(
             '--command',
             type=str,
-            default=str(DEFAULT_REBOOT_COMMAND),
-            help='A command to run on the guest to trigger the reboot.',
+            default=None,
+            help=f"""
+            A command to run on the guest to trigger the reboot.
+            Default is ``{DEFAULT_REBOOT_COMMAND}``.
+            """,
         )
         def reboot(context: 'tmt.cli.Context', **kwargs: Any) -> None:
             """
@@ -2150,7 +2187,12 @@ class Reboot(Action):
         assert hasattr(self.parent, 'plan')
         assert self.parent.plan is not None
         for guest in self.parent.plan.provision.ready_guests:
-            guest.reboot(hard=self.opt('hard'), command=tmt.utils.ShellScript(self.opt('command')))
+            guest.reboot(
+                hard=self.opt('hard'),
+                command=tmt.utils.ShellScript(self.opt('command'))
+                if self.opt('command')
+                else None,
+            )
         self.info('reboot', 'Reboot finished', color='yellow')
 
 
@@ -2304,6 +2346,11 @@ class Login(Action):
         Run the interactive command
         """
 
+        # Nothing to do if there are no guests ready for login
+        if not self.parent.plan.provision.ready_guests:
+            self.info('login', 'No guests ready for login', color='yellow')
+            return
+
         scripts = [
             tmt.utils.ShellScript(script)
             for script in self.opt('command', (DEFAULT_LOGIN_COMMAND,))
@@ -2349,7 +2396,7 @@ class Login(Action):
 
     def after_test(
         self,
-        result: 'tmt.base.Result',
+        results: list['tmt.base.Result'],
         cwd: Optional[Path] = None,
         env: Optional[tmt.utils.Environment] = None,
     ) -> None:
@@ -2357,7 +2404,7 @@ class Login(Action):
         Check and login after test execution
         """
 
-        if self._enabled_by_results([result]):
+        if self._enabled_by_results(results):
             self._login(cwd, env)
 
 
@@ -2530,6 +2577,9 @@ class Topology(SerializableContainer):
         Save and push topology to a given guest
         """
 
+        # Avoid circular imports
+        from tmt.steps.provision import TransferOptions
+
         topology_filepaths = self.save(dirpath=dirpath, filename_base=filename_base)
 
         environment = Environment()
@@ -2540,7 +2590,7 @@ class Topology(SerializableContainer):
             guest.push(
                 source=filepath,
                 destination=filepath,
-                options=["-s", "-p", "--chmod=755"],
+                options=TransferOptions(protect_args=True, preserve_perms=True, chmod=0o755),
             )
 
             if filepath.suffix == '.sh':
@@ -2555,7 +2605,6 @@ class Topology(SerializableContainer):
         return environment
 
 
-@container
 class ActionTask(tmt.queue.GuestlessTask[None]):
     """
     A task to run an action
@@ -2563,9 +2612,8 @@ class ActionTask(tmt.queue.GuestlessTask[None]):
 
     phase: Action
 
-    # Custom yet trivial `__init__` is necessary, see note in `tmt.queue.Task`.
-    def __init__(self, logger: tmt.log.Logger, phase: Action, **kwargs: Any) -> None:
-        super().__init__(logger, **kwargs)
+    def __init__(self, phase: Action, logger: tmt.log.Logger) -> None:
+        super().__init__(logger)
 
         self.phase = phase
 
@@ -2577,7 +2625,6 @@ class ActionTask(tmt.queue.GuestlessTask[None]):
         self.phase.go()
 
 
-@container
 class PluginTask(
     tmt.queue.MultiGuestTask[PluginReturnValueT],
     Generic[StepDataT, PluginReturnValueT],
@@ -2588,15 +2635,13 @@ class PluginTask(
 
     phase: Plugin[StepDataT, PluginReturnValueT]
 
-    # Custom yet trivial `__init__` is necessary, see note in `tmt.queue.Task`.
     def __init__(
         self,
-        logger: tmt.log.Logger,
-        guests: list['Guest'],
         phase: Plugin[StepDataT, PluginReturnValueT],
-        **kwargs: Any,
+        guests: list['Guest'],
+        logger: tmt.log.Logger,
     ) -> None:
-        super().__init__(logger, guests, **kwargs)
+        super().__init__(guests, logger)
 
         self.phase = phase
 
@@ -2621,8 +2666,8 @@ class PluginTask(
     def name(self) -> str:
         return f'{self.phase_name} on {fmf.utils.listed(self.guest_ids)}'
 
-    def run_on_guest(self, guest: 'Guest', logger: tmt.log.Logger) -> None:
-        self.phase.go(guest=guest, logger=logger)
+    def run_on_guest(self, guest: 'Guest', logger: tmt.log.Logger) -> PluginReturnValueT:
+        return self.phase.go(guest=guest, logger=logger)
 
 
 class PhaseQueue(tmt.queue.Queue[Union[ActionTask, PluginTask[StepDataT, PluginReturnValueT]]]):
@@ -2631,7 +2676,7 @@ class PhaseQueue(tmt.queue.Queue[Union[ActionTask, PluginTask[StepDataT, PluginR
     """
 
     def enqueue_action(self, *, phase: Action) -> None:
-        self.enqueue_task(ActionTask(logger=phase._logger, phase=phase))
+        self.enqueue_task(ActionTask(phase, phase._logger))
 
     def enqueue_plugin(
         self, *, phase: Plugin[StepDataT, PluginReturnValueT], guests: list['Guest']
@@ -2641,19 +2686,14 @@ class PhaseQueue(tmt.queue.Queue[Union[ActionTask, PluginTask[StepDataT, PluginR
                 f'No guests queued for phase "{phase}". A typo in "where" key?'
             )
 
-        self.enqueue_task(PluginTask(logger=phase._logger, guests=guests, phase=phase))
+        self.enqueue_task(PluginTask(phase, guests, phase._logger))
 
 
-@container
 class PushTask(tmt.queue.MultiGuestTask[None]):
     """
     Task performing a workdir push to a guest
     """
 
-    # Custom yet trivial `__init__` is necessary, see note in `tmt.queue.Task`.
-    def __init__(self, logger: tmt.log.Logger, guests: list['Guest'], **kwargs: Any) -> None:
-        super().__init__(logger, guests, **kwargs)
-
     @property
     def name(self) -> str:
         return f'push to {fmf.utils.listed(self.guest_ids)}'
@@ -2662,7 +2702,6 @@ class PushTask(tmt.queue.MultiGuestTask[None]):
         guest.push()
 
 
-@container
 class PullTask(tmt.queue.MultiGuestTask[None]):
     """
     Task performing a workdir pull from a guest
@@ -2670,15 +2709,10 @@ class PullTask(tmt.queue.MultiGuestTask[None]):
 
     source: Optional[Path]
 
-    # Custom yet trivial `__init__` is necessary, see note in `tmt.queue.Task`.
     def __init__(
-        self,
-        logger: tmt.log.Logger,
-        guests: list['Guest'],
-        source: Optional[Path] = None,
-        **kwargs: Any,
+        self, guests: list['Guest'], source: Optional[Path], logger: tmt.log.Logger
     ) -> None:
-        super().__init__(logger, guests, **kwargs)
+        super().__init__(guests, logger)
 
         self.source = source
 
