@@ -90,6 +90,19 @@ class DiscoverStepData(tmt.steps.WhereableStepData, tmt.steps.StepData):
         ],
     )
 
+    require_test: list[str] = field(
+        default_factory=list,
+        option=('--require-test'),
+        metavar='NAMES',
+        multiple=True,
+        help="""
+            A list of test names that must be discovered during the run. If an execute
+            step is present, these tests must also be executed. If any of the
+            specified tests are not discovered or executed, an exception is raised.
+            """,
+        normalize=tmt.utils.normalize_string_list,
+    )
+
 
 DiscoverStepDataT = TypeVar('DiscoverStepDataT', bound=DiscoverStepData)
 
@@ -104,7 +117,7 @@ class DiscoverPlugin(tmt.steps.GuestlessPlugin[DiscoverStepDataT, None]):
     _data_class = DiscoverStepData  # type: ignore[assignment]
 
     # Methods ("how: ..." implementations) registered for the same step.
-    _supported_methods: PluginRegistry[tmt.steps.Method] = PluginRegistry()
+    _supported_methods: PluginRegistry[tmt.steps.Method] = PluginRegistry('step.discover')
 
     @classmethod
     def base_command(
@@ -193,18 +206,18 @@ class DiscoverPlugin(tmt.steps.GuestlessPlugin[DiscoverStepDataT, None]):
         Log details about the imported plan
         """
 
-        parent = cast(Optional[tmt.steps.discover.Discover], self.parent)
+        parent = cast(Optional[Discover], self.parent)
         if (
             parent
             and parent.plan._original_plan
-            and parent.plan._original_plan._imported_plan_fmf_id
+            and parent.plan._original_plan._imported_plan_references
         ):
-            remote_plan_id = parent.plan._original_plan._imported_plan_fmf_id
-            # FIXME: cast() - https://github.com/python/mypy/issues/7981
-            # Note the missing Optional for values - to_minimal_dict() would
-            # not include unset keys, therefore all values should be valid.
-            for key, value in cast(dict[str, str], remote_plan_id.to_minimal_spec()).items():
-                self.verbose(f'import {key}', value, 'green')
+            for remote_plan_id in parent.plan._original_plan._imported_plan_references:
+                # FIXME: cast() - https://github.com/python/mypy/issues/7981
+                # Note the missing Optional for values - to_minimal_dict() would
+                # not include unset keys, therefore all values should be valid.
+                for key, value in cast(dict[str, str], remote_plan_id.to_minimal_spec()).items():
+                    self.verbose(f'import {key}', value, 'green')
 
     def post_dist_git(self, created_content: list[Path]) -> None:
         """
@@ -236,6 +249,9 @@ class Discover(tmt.steps.Step):
         self._tests: dict[str, list[tmt.Test]] = {}
         self._failed_tests: dict[str, list[tmt.Test]] = {}
 
+        # Collection of required tests per discover step phase
+        self._required_test_names: dict[str, list[str]] = {}
+
         # Test will be (re)discovered in other phases/steps
         self.extract_tests_later: bool = False
 
@@ -247,6 +263,21 @@ class Discover(tmt.steps.Step):
 
         return {*super()._preserved_workdir_members, 'tests.yaml'}
 
+    @property
+    def required_tests(self) -> list[TestOrigin]:
+        """
+        The list of required tests gathered from all phases
+        """
+
+        tests = []
+        for phase_name, required_test_names in self._required_test_names.items():
+            tests += [
+                test_origin
+                for test_origin in self.tests(phase_name=phase_name)
+                if test_origin.test.name in required_test_names
+            ]
+        return tests
+
     def load(self) -> None:
         """
         Load step data from the workdir
@@ -438,6 +469,12 @@ class Discover(tmt.steps.Step):
 
                 # Prefix test name only if multiple plugins configured
                 prefix = f'/{phase.name}' if len(self.phases()) > 1 else ''
+
+                self._required_test_names[phase.name] = [
+                    f"{prefix}{test_name}"
+                    for test_name in cast(DiscoverStepData, phase.data).require_test
+                ]
+
                 # Check discovered tests, modify test name/path
                 for test_origin in phase.tests(enabled=True):
                     test = test_origin.test
@@ -464,7 +501,10 @@ class Discover(tmt.steps.Step):
                     fmf_id = test_origin.test.fmf_id
 
                     if not fmf_id.url:
-                        continue
+                        raise tmt.utils.DiscoverError(
+                            f"`tmt run discover --fmf-id` without `url` option "
+                            f"in plan `{self.plan}` can be used only within git repo."
+                        )
 
                     exported = test_origin.test.fmf_id.to_minimal_spec()
 
@@ -482,12 +522,14 @@ class Discover(tmt.steps.Step):
             assert isinstance(self.parent, tmt.base.Plan)  # narrow type
 
             # Get failed results from previous run execute
-            for result in self.parent.execute._results:
+            failed_results = [
+                result
+                for result in self.parent.execute._results
                 if (
                     result.result is not tmt.result.ResultOutcome.PASS
                     and result.result is not tmt.result.ResultOutcome.INFO
-                ):
-                    failed_results.append(result)
+                )
+            ]
 
             # Filter existing tests into another variable which is then used by tests() method
             for test_phase in self._tests:
@@ -497,6 +539,14 @@ class Discover(tmt.steps.Step):
                         if test.name == result.name and test.serial_number == result.serial_number:
                             self._failed_tests[test_phase].append(test)
 
+        # Assert that all required tests were discovered
+        for phase_name, required_test_names in self._required_test_names.items():
+            for required_test in required_test_names:
+                if not any(required_test == test.name for test in self._tests.get(phase_name, [])):
+                    raise tmt.utils.DiscoverError(
+                        f"Required test '{required_test}' not discovered in phase '{phase_name}'."
+                    )
+
         # Give a summary, update status and save
         self.summary()
         self.status('done')
