@@ -1,9 +1,7 @@
 import contextlib
 import glob
-import os
 import re
 import shutil
-import subprocess
 from typing import Any, Optional, cast
 
 import fmf
@@ -16,6 +14,7 @@ import tmt.options
 import tmt.steps
 import tmt.steps.discover
 import tmt.utils
+import tmt.utils.filesystem
 import tmt.utils.git
 from tmt.base import _RawAdjustRule
 from tmt.container import container, field
@@ -93,7 +92,9 @@ class DiscoverFmfStepData(tmt.steps.discover.DiscoverStepData):
             List of test names or regular expressions used to
             select tests by name. Duplicate test names are allowed
             to enable repetitive test execution, preserving the
-            listed test order.
+            listed test order. The search mode is used for pattern
+            matching. See the :ref:`regular-expressions` section for
+            details.
             """,
         normalize=tmt.utils.normalize_string_list,
     )
@@ -124,12 +125,31 @@ class DiscoverFmfStepData(tmt.steps.discover.DiscoverStepData):
             """,
         normalize=tmt.utils.normalize_string_list,
     )
+
+    include: list[str] = field(
+        default_factory=list,
+        option=('-i', '--include'),
+        metavar='REGEXP',
+        multiple=True,
+        help="""
+            Include only tests matching given regular expression.
+            Respect the :ref:`/spec/core/order` defined in test.
+            The search mode is used for pattern matching. See the
+            :ref:`regular-expressions` section for details.
+            """,
+        normalize=tmt.utils.normalize_string_list,
+    )
+
     exclude: list[str] = field(
         default_factory=list,
         option=('-x', '--exclude'),
         metavar='REGEXP',
         multiple=True,
-        help="Exclude tests matching given regular expression.",
+        help="""
+            Exclude tests matching given regular expression.
+            The search mode is used for pattern matching. See the
+            :ref:`regular-expressions` section for details.
+            """,
         normalize=tmt.utils.normalize_string_list,
     )
 
@@ -290,18 +310,9 @@ class DiscoverFmf(tmt.steps.discover.DiscoverPlugin[DiscoverFmfStepData]):
 
     If no ``ref`` is provided, the default branch from the origin is used.
 
-    The following keys are used to limit the test discovery:
-
-    ``test`` - list of test names or regular expressions used to select tests
-
-    ``link`` - select tests using the link keys
-
-    ``filter`` - apply advanced filter based on test metadata attributes
-
-    ``exclude`` - exclude tests which match a regular expression
-
-    ``prune`` - copy only immediate directories of executed tests and their required files
 
+    Dist Git
+    ^^^^^^^^
 
     For DistGit repo one can download sources and use code from them in
     the tests. Sources are extracted into ``$TMT_SOURCE_DIR`` path,
@@ -319,6 +330,63 @@ class DiscoverFmf(tmt.steps.discover.DiscoverPlugin[DiscoverFmfStepData]):
             how: fmf
             dist-git-source: true
 
+    Name Filter
+    ^^^^^^^^^^^
+
+    Use the ``test`` key to limit which tests should be executed by
+    providing regular expression matching the test name:
+
+    .. code-block:: yaml
+
+        discover:
+            how: fmf
+            test: ^/tests/area/.*
+
+    .. code-block:: shell
+
+        tmt run discover --how fmf --verbose --test "^/tests/core.*"
+
+    When several regular expressions are provided, tests matching each
+    regular expression are concatenated. In this way it is possible to
+    execute a single test multiple times:
+
+    .. code-block:: yaml
+
+        discover:
+            how: fmf
+            test:
+              - ^/test/one$
+              - ^/test/two$
+              - ^/special/setup$
+              - ^/test/one$
+              - ^/test/two$
+
+    .. code-block:: shell
+
+        tmt run discover -h fmf -v -t '^/test/one$' -t '^/special/setup$' -t '^/test/two$'
+
+    The ``include`` key also allows to select tests by name, with two
+    important distinctions from the ``test`` key:
+
+    * The original test :ref:`/spec/core/order` is preserved so it does
+      not matter in which order tests are listed under the ``include``
+      key.
+
+    * Test duplication is not allowed, so even if a test name is
+      repeated several times, test will be executed only once.
+
+    Finally, the ``exclude`` key can be used to specify regular
+    expressions matching tests which should be skipped during the
+    discovery.
+
+    The ``test``, ``include`` and ``exclude`` keys use search mode for
+    matching patterns. See the :ref:`regular-expressions` section for
+    detailed information about how exactly the regular expressions are
+    handled.
+
+    Link Filter
+    ^^^^^^^^^^^
+
     Selecting tests containing specified link is possible using ``link``
     key accepting ``RELATION:TARGET`` format of values. Regular
     expressions are supported for both relation and target part of the
@@ -330,6 +398,30 @@ class DiscoverFmf(tmt.steps.discover.DiscoverPlugin[DiscoverFmfStepData]):
             how: fmf
             link: verifies:.*issue/850$
 
+    Advanced Filter
+    ^^^^^^^^^^^^^^^
+
+    The ``filter`` key can be used to apply an advanced filter based on
+    test metadata attributes. These can be especially useful when tests
+    are grouped by the :ref:`/spec/core/tag` or :ref:`/spec/core/tier`
+    keys:
+
+    .. code-block:: yaml
+
+        discover:
+            how: fmf
+            filter: tier:3 & tag:provision
+
+    .. code-block:: shell
+
+        tmt run discover --how fmf --filter "tier:3 & tag:provision"
+
+    See the ``pydoc fmf.filter`` documentation for more details about
+    the supported syntax and available operators.
+
+    Modified Tests
+    ^^^^^^^^^^^^^^
+
     It is also possible to limit tests only to those that have changed
     in git since a given revision. This can be particularly useful when
     testing changes to tests themselves (e.g. in a pull request CI).
@@ -350,6 +442,9 @@ class DiscoverFmf(tmt.steps.discover.DiscoverPlugin[DiscoverFmfStepData]):
     specified via ``test``, so those tests will also be selected even if
     not modified.
 
+    Adjust Tests
+    ^^^^^^^^^^^^
+
     Use the ``adjust-tests`` key to modify the discovered tests'
     metadata directly from the plan. For example, extend the test
     duration for slow hardware or modify the list of required packages
@@ -379,9 +474,10 @@ class DiscoverFmf(tmt.steps.discover.DiscoverPlugin[DiscoverFmfStepData]):
     # Options which require .git to be present for their functionality
     _REQUIRES_GIT = (
         "ref",
-        "modified-url",
-        "modified-only",
-        "fmf-id",
+        "modified_url",
+        "modified_only",
+        "fmf_id",
+        "sync_repo",
     )
 
     @property
@@ -390,7 +486,7 @@ class DiscoverFmf(tmt.steps.discover.DiscoverPlugin[DiscoverFmfStepData]):
         Enable standalone mode when listing fmf ids
         """
 
-        if self.opt('fmf_id'):
+        if self.data.fmf_id:
             return True
         return super().is_in_standalone_mode
 
@@ -435,48 +531,6 @@ class DiscoverFmf(tmt.steps.discover.DiscoverPlugin[DiscoverFmfStepData]):
                 "Cannot manipulate with dist-git without the `--dist-git-merge` option."
             )
 
-        # Raise an exception if --fmf-id uses w/o url and git root
-        # doesn't exist for discovered plan
-        if self.opt('fmf_id'):
-
-            def assert_git_url(plan_name: Optional[str] = None) -> None:
-                try:
-                    subprocess.run(
-                        'git rev-parse --show-toplevel'.split(),
-                        stdout=subprocess.PIPE,
-                        stderr=subprocess.DEVNULL,
-                        check=True,
-                    )
-                except subprocess.CalledProcessError:
-                    raise tmt.utils.DiscoverError(
-                        f"`tmt run discover --fmf-id` without `url` option in "
-                        f"plan `{plan_name}` can be used only within"
-                        f" git repo."
-                    )
-
-            # It covers only one case, when there is:
-            # 1) no --url on CLI
-            # 2) plan w/o url exists in test run
-            if not self.opt('url'):
-                try:
-                    fmf_tree = fmf.Tree(os.getcwd())
-                except fmf.utils.RootError:
-                    raise tmt.utils.DiscoverError(
-                        "No metadata found in the current directory. "
-                        "Use 'tmt init' to get started."
-                    )
-                for attr in fmf_tree.climb():
-                    try:
-                        plan_url = attr.data.get('discover').get('url')
-                        plan_name = attr.name
-                        if not plan_url:
-                            assert_git_url(plan_name)
-                    except AttributeError:
-                        pass
-            # All other cases are covered by this condition
-            if not url:
-                assert_git_url(self.step.plan.name)
-
         self.log_import_plan_details()
 
         # Clone provided git repository (if url given) with disabled
@@ -499,10 +553,8 @@ class DiscoverFmf(tmt.steps.discover.DiscoverPlugin[DiscoverFmfStepData]):
             if path is not None:
                 fmf_root: Optional[Path] = path
             else:
-                fmf_root = Path(self.step.plan.node.root)
-            requires_git = self.opt('sync-repo') or any(
-                self.get(opt) for opt in self._REQUIRES_GIT
-            )
+                fmf_root = Path(self.step.plan.fmf_root) if self.step.plan.fmf_root else None
+            requires_git = any(getattr(self.data, key) for key in self._REQUIRES_GIT)
             # Path for distgit sources cannot be checked until the
             # they are extracted
             if path and not path.is_dir() and not dist_git_source:
@@ -540,7 +592,7 @@ class DiscoverFmf(tmt.steps.discover.DiscoverPlugin[DiscoverFmfStepData]):
             if not dist_git_source or dist_git_merge:
                 self.debug(f"Copy '{directory}' to '{self.testdir}'.")
                 if not self.is_dry_run:
-                    shutil.copytree(directory, self.testdir, symlinks=True)
+                    tmt.utils.filesystem.copy_tree(directory, self.testdir, self._logger)
 
         # Prepare path of the dynamic reference
         try:
@@ -563,7 +615,7 @@ class DiscoverFmf(tmt.steps.discover.DiscoverPlugin[DiscoverFmfStepData]):
         if self.testdir.is_dir():
             with contextlib.suppress(tmt.utils.RunError, AttributeError):
                 self.verbose(
-                    'hash',
+                    'commit-hash',
                     tmt.utils.git.git_hash(directory=self.testdir, logger=self._logger),
                     'green',
                 )
@@ -571,38 +623,54 @@ class DiscoverFmf(tmt.steps.discover.DiscoverPlugin[DiscoverFmfStepData]):
         # Dist-git source processing during discover step
         if dist_git_source:
             try:
-                # 'ref' is checked out in self.testdir
-                self.download_distgit_source(
-                    distgit_dir=self.testdir if ref else git_root,
-                    target_dir=sourcedir,
-                    handler_name=self.get('dist-git-type'),
-                )
-                # Copy rest of files so TMT_SOURCE_DIR has patches, sources and spec file
-                # FIXME 'worktree' could be used as sourcedir when 'url' is not set
-                shutil.copytree(
-                    self.testdir if ref else git_root,
-                    sourcedir,
-                    symlinks=True,
-                    dirs_exist_ok=True,
-                )
-                # patch & rediscover will happen later in the prepare step
-                if not self.get('dist-git-download-only'):
-                    # Check if prepare is enabled, warn user if not
-                    if not self.step.plan.prepare.enabled:
-                        self.warn("Sources will not be extracted, prepare step is not enabled.")
-                    insert_to_prepare_step(
-                        discover_plugin=self,
-                        sourcedir=sourcedir,
-                    )
-                # merge or not, detect later
-                self.step.plan.discover.extract_tests_later = True
-                self.info("Tests will be discovered after dist-git patching in prepare.")
+                distgit_dir = self.testdir if ref else git_root
+                self.process_distgit_source(distgit_dir, sourcedir)
                 return
             except Exception as error:
                 raise tmt.utils.DiscoverError("Failed to process 'dist-git-source'.") from error
+
         # Discover tests
         self.do_the_discovery(path)
 
+        # Apply tmt run policy
+        if self.step.plan.my_run is not None:
+            for policy in self.step.plan.my_run.policies:
+                policy.apply_to_tests(tests=self._tests, logger=self._logger)
+
+    def process_distgit_source(self, distgit_dir: Path, sourcedir: Path) -> None:
+        """
+        Process dist-git source during the discover step.
+        """
+
+        self.download_distgit_source(
+            distgit_dir=distgit_dir,
+            target_dir=sourcedir,
+            handler_name=self.get('dist-git-type'),
+        )
+
+        # Copy rest of files so TMT_SOURCE_DIR has patches, sources and spec file
+        # FIXME 'worktree' could be used as sourcedir when 'url' is not set
+        tmt.utils.filesystem.copy_tree(
+            distgit_dir,
+            sourcedir,
+            self._logger,
+        )
+
+        # patch & rediscover will happen later in the prepare step
+        if not self.get('dist-git-download-only'):
+            # Check if prepare is enabled, warn user if not
+            if not self.step.plan.prepare.enabled:
+                self.warn("Sources will not be extracted, prepare step is not enabled.")
+
+            insert_to_prepare_step(
+                discover_plugin=self,
+                sourcedir=sourcedir,
+            )
+
+        # merge or not, detect later
+        self.step.plan.discover.extract_tests_later = True
+        self.info("Tests will be discovered after dist-git patching in prepare.")
+
     def do_the_discovery(self, path: Optional[Path] = None) -> None:
         """
         Discover the tests
@@ -643,7 +711,8 @@ class DiscoverFmf(tmt.steps.discover.DiscoverPlugin[DiscoverFmfStepData]):
         for link_needle in link_needles:
             self.info('link', str(link_needle), 'green')
 
-        excludes = list(tmt.base.Test._opt('exclude') or self.get('exclude', []))
+        excludes = list(tmt.base.Test._opt('exclude') or self.data.exclude)
+        includes = list(tmt.base.Test._opt('include') or self.data.include)
 
         # Filter only modified tests if requested
         modified_only = self.get('modified-only')
@@ -703,7 +772,7 @@ class DiscoverFmf(tmt.steps.discover.DiscoverPlugin[DiscoverFmfStepData]):
         tree = tmt.Tree(
             logger=self._logger,
             path=tree_path,
-            fmf_context=self.step.plan._fmf_context,
+            fmf_context=self.step.plan.fmf_context,
             additional_rules=self.data.adjust_tests,
         )
         self._tests = tree.tests(
@@ -712,6 +781,7 @@ class DiscoverFmf(tmt.steps.discover.DiscoverPlugin[DiscoverFmfStepData]):
             conditions=["manual is False"],
             unique=False,
             links=link_needles,
+            includes=includes,
             excludes=excludes,
         )
 
@@ -720,17 +790,17 @@ class DiscoverFmf(tmt.steps.discover.DiscoverPlugin[DiscoverFmfStepData]):
             clonedir = self.clone_dirpath / 'tests'
             clone_tree_path = clonedir / path.unrooted()
             for file_path in tmt.utils.filter_paths(tree_path, [r'\.fmf']):
-                shutil.copytree(
+                tmt.utils.filesystem.copy_tree(
                     file_path,
                     clone_tree_path / file_path.relative_to(tree_path),
-                    dirs_exist_ok=True,
+                    self._logger,
                 )
 
             # Save upgrade plan
             upgrade_path = self.get('upgrade_path')
             if upgrade_path:
                 upgrade_path = f"{upgrade_path.lstrip('/')}.fmf"
-                (clone_tree_path / upgrade_path).parent.mkdir()
+                (clone_tree_path / upgrade_path).parent.mkdir(parents=True, exist_ok=True)
                 shutil.copyfile(tree_path / upgrade_path, clone_tree_path / upgrade_path)
                 shutil.copymode(tree_path / upgrade_path, clone_tree_path / upgrade_path)
 
@@ -743,10 +813,10 @@ class DiscoverFmf(tmt.steps.discover.DiscoverPlugin[DiscoverFmfStepData]):
                 # Save only current test data
                 assert test.path is not None  # narrow type
                 relative_test_path = test.path.unrooted()
-                shutil.copytree(
+                tmt.utils.filesystem.copy_tree(
                     tree_path / relative_test_path,
                     clone_tree_path / relative_test_path,
-                    dirs_exist_ok=True,
+                    self._logger,
                 )
 
                 # Copy all parent main.fmf files
@@ -754,6 +824,8 @@ class DiscoverFmf(tmt.steps.discover.DiscoverPlugin[DiscoverFmfStepData]):
                 while parent_dir.resolve() != Path.cwd().resolve():
                     parent_dir = parent_dir.parent
                     if (tree_path / parent_dir / 'main.fmf').exists():
+                        # Ensure parent directory exists
+                        (clone_tree_path / parent_dir).mkdir(parents=True, exist_ok=True)
                         shutil.copyfile(
                             tree_path / parent_dir / 'main.fmf',
                             clone_tree_path / parent_dir / 'main.fmf',
@@ -777,7 +849,7 @@ class DiscoverFmf(tmt.steps.discover.DiscoverPlugin[DiscoverFmfStepData]):
             # Clean self.testdir and copy back only required tests and files from clonedir
             # This is to have correct paths in tests
             shutil.rmtree(self.testdir, ignore_errors=True)
-            shutil.copytree(clonedir, self.testdir)
+            tmt.utils.filesystem.copy_tree(clonedir, self.testdir, self._logger)
 
         # Cleanup clone directories
         if self.clone_dirpath.exists():
@@ -842,7 +914,7 @@ class DiscoverFmf(tmt.steps.discover.DiscoverPlugin[DiscoverFmfStepData]):
                     git_root = self.get_git_root(Path(self.step.plan.node.root))
                     self.debug(f"Copy '{git_root}' to '{self.testdir}'.")
                     if not self.is_dry_run:
-                        shutil.copytree(git_root, self.testdir, symlinks=True, dirs_exist_ok=True)
+                        tmt.utils.filesystem.copy_tree(git_root, self.testdir, self._logger)
 
         # Copy extracted sources into testdir
         if not self.is_dry_run:
@@ -857,11 +929,10 @@ class DiscoverFmf(tmt.steps.discover.DiscoverPlugin[DiscoverFmfStepData]):
             for to_copy in copy_these:
                 src = sourcedir / to_copy
                 if src.is_dir():
-                    shutil.copytree(
+                    tmt.utils.filesystem.copy_tree(
                         sourcedir / to_copy,
                         self.testdir if flatten else self.testdir / to_copy,
-                        symlinks=True,
-                        dirs_exist_ok=True,
+                        self._logger,
                     )
                 else:
                     shutil.copyfile(src, self.testdir / to_copy)
@@ -873,6 +944,11 @@ class DiscoverFmf(tmt.steps.discover.DiscoverPlugin[DiscoverFmfStepData]):
         for test in self._tests:
             test.environment['TMT_SOURCE_DIR'] = EnvVarValue(sourcedir)
 
+        # Apply tmt run policy
+        if self.step.plan.my_run is not None:
+            for policy in self.step.plan.my_run.policies:
+                policy.apply_to_tests(tests=self._tests, logger=self._logger)
+
         # Inject newly found tests into parent discover at the right position
         # FIXME
         # Prefix test name only if multiple plugins configured
