@@ -13,6 +13,7 @@ import tmt.steps
 import tmt.steps.discover
 import tmt.utils
 import tmt.utils.git
+from tmt._compat.typing import Self
 from tmt.container import SerializableContainer, SpecBasedContainer, container, field
 from tmt.steps.prepare.distgit import insert_to_prepare_step
 from tmt.utils import (
@@ -153,8 +154,8 @@ class TestDescription(
     # type than the one declared in superclass.
     @classmethod
     def from_spec(  # type: ignore[override]
-        cls: type[T], raw_data: dict[str, Any], logger: tmt.log.Logger
-    ) -> T:
+        cls, raw_data: dict[str, Any], logger: tmt.log.Logger
+    ) -> Self:
         """
         Convert from a specification file or from a CLI option
         """
@@ -349,7 +350,9 @@ class DiscoverShell(tmt.steps.discover.DiscoverPlugin[DiscoverShellData]):
             self.run(Command('git', 'checkout', '-f', ref), cwd=testdir)
 
         # Log where HEAD leads to
-        self.debug('hash', tmt.utils.git.git_hash(directory=testdir, logger=self._logger))
+        self.verbose(
+            'commit-hash', tmt.utils.git.git_hash(directory=testdir, logger=self._logger), 'green'
+        )
 
         # Remove .git so that it's not copied to the SUT
         # if 'keep-git-metadata' option is not specified
@@ -489,6 +492,11 @@ class DiscoverShell(tmt.steps.discover.DiscoverPlugin[DiscoverShellData]):
             if self.data.dist_git_source:
                 test.environment['TMT_SOURCE_DIR'] = EnvVarValue(sourcedir)
 
+        # Apply tmt run policy
+        if self.step.plan.my_run is not None:
+            for policy in self.step.plan.my_run.policies:
+                policy.apply_to_tests(tests=self._tests, logger=self._logger)
+
     def tests(
         self, *, phase_name: Optional[str] = None, enabled: Optional[bool] = None
     ) -> list[tmt.steps.discover.TestOrigin]:
