@@ -5,9 +5,9 @@ import os
 import signal as _signal
 import subprocess
 import threading
+from collections.abc import Sequence
 from contextlib import suppress
-from tempfile import NamedTemporaryFile
-from typing import TYPE_CHECKING, Any, Callable, Optional, TypeVar, Union, cast
+from typing import TYPE_CHECKING, Any, Optional, TypeVar, Union, cast
 
 import click
 import fmf
@@ -17,17 +17,22 @@ import tmt
 import tmt.base
 import tmt.log
 import tmt.steps
+import tmt.steps.scripts
 import tmt.utils
-from tmt.checks import CheckEvent
+import tmt.utils.signals
+import tmt.utils.wait
+from tmt.checks import Check, CheckEvent, CheckPlugin
 from tmt.container import container, field, simple_field
 from tmt.options import option
 from tmt.plugins import PluginRegistry
 from tmt.result import CheckResult, Result, ResultGuestData, ResultInterpret, ResultOutcome
 from tmt.steps import Action, ActionTask, PhaseQueue, PluginTask, Step
+from tmt.steps.abort import AbortContext, AbortStep
 from tmt.steps.discover import Discover, DiscoverPlugin, DiscoverStepData
 from tmt.steps.provision import Guest
 from tmt.utils import (
     Command,
+    CommandOutput,
     Path,
     ShellScript,
     Stopwatch,
@@ -35,7 +40,7 @@ from tmt.utils import (
     format_duration,
     format_timestamp,
 )
-from tmt.utils.templates import render_template_file
+from tmt.utils.wait import Deadline, Waiting
 
 if TYPE_CHECKING:
     import tmt.cli
@@ -57,201 +62,11 @@ TEST_OUTPUT_FILENAME = 'output.txt'
 # Metadata file with details about the current test
 TEST_METADATA_FILENAME = 'metadata.yaml'
 
-#: Scripts source directory
-SCRIPTS_SRC_DIR = tmt.utils.resource_files('steps/execute/scripts')
+# File containing test failures
+TEST_FAILURES_FILENAME = 'failures.yaml'
 
-#: The default scripts destination directory
-DEFAULT_SCRIPTS_DEST_DIR = Path("/usr/local/bin")
-
-#: The default scripts destination directory for rpm-ostree based distributions, https://github.com/teemtee/tmt/discussions/3260
-DEFAULT_SCRIPTS_DEST_DIR_OSTREE = Path("/var/lib/tmt/scripts")
-
-#: The tmt environment variable name for forcing ``SCRIPTS_DEST_DIR``
-SCRIPTS_DEST_DIR_VARIABLE = 'TMT_SCRIPTS_DIR'
-
-
-@container
-class Script:
-    """
-    Represents a script provided by the internal executor.
-
-    Must be used as a context manager. The context manager returns
-    the source filename.
-
-    The source file is defined by the ``source_filename`` attribute and its
-    location is relative to the directory specified via the :py:data:`SCRIPTS_SRC_DIR`
-    variable. All scripts must be located in this directory.
-
-    The default destination directory of the scripts is :py:data:`DEFAULT_SCRIPTS_DEST_DIR`.
-    On ``rpm-ostree`` distributions like Fedora CoreOS, the default destination
-    directory is :py:data:``DEFAULT_SCRIPTS_DEST_DIR_OSTREE``. The destination directory
-    of the scripts can be forced by the script using ``destination_path`` attribute.
-
-    The destination directory can be overridden using the environment variable defined
-    by the :py:data:`DEFAULT_SCRIPTS_DEST_DIR_VARIABLE` variable.
-
-    The ``enabled`` attribute can specify a function which is called with :py:class:`Guest`
-    instance to evaluate if the script is enabled. This can be useful to optionally disable
-    a script for specific guests.
-    """
-
-    source_filename: str
-    destination_path: Optional[Path]
-    aliases: list[str]
-    related_variables: list[str]
-    enabled: Callable[[Guest], bool]
-
-    def __enter__(self) -> Path:
-        return SCRIPTS_SRC_DIR / self.source_filename
-
-    def __exit__(self, *args: object) -> None:
-        pass
-
-
-@container
-class ScriptCreatingFile(Script):
-    """
-    Represents a script which creates a file.
-
-    See :py:class:`Script` for more details.
-    """
-
-    created_file: str
-
-
-@container
-class ScriptTemplate(Script):
-    """
-    Represents a Jinja2 templated script.
-
-    The source filename is constructed from the name of the file specified
-    via the ``source_filename`` attribute, with the ``.j2`` suffix appended.
-    The template file must be located in the directory specified
-    via :py:data:`SCRIPTS_SRC_DIR` variable.
-    """
-
-    context: dict[str, str]
-
-    _rendered_script_path: Optional[Path] = None
-
-    def __enter__(self) -> Path:
-        with NamedTemporaryFile(mode='w', delete=False) as rendered_script:
-            rendered_script.write(
-                render_template_file(
-                    SCRIPTS_SRC_DIR / f"{self.source_filename}.j2", None, **self.context
-                )
-            )
-
-        self._rendered_script_path = Path(rendered_script.name)
-
-        return self._rendered_script_path
-
-    def __exit__(self, *args: object) -> None:
-        assert self._rendered_script_path
-        os.unlink(self._rendered_script_path)
-
-
-def effective_scripts_dest_dir(default: Path = DEFAULT_SCRIPTS_DEST_DIR) -> Path:
-    """
-    Find out what the actual scripts destination directory is.
-
-    If the ``TMT_SCRIPTS_DIR`` environment variable is set, it is used
-    as the scripts destination directory. Otherwise, the ``default``
-    parameter path is returned.
-    """
-
-    return Path(os.environ.get(SCRIPTS_DEST_DIR_VARIABLE, default))
-
-
-# Script handling reboots, in restraint compatible fashion
-TMT_REBOOT_SCRIPT = ScriptCreatingFile(
-    source_filename='tmt-reboot',
-    destination_path=None,
-    aliases=[
-        'rstrnt-reboot',
-        'rhts-reboot',
-    ],
-    related_variables=[
-        "TMT_REBOOT_COUNT",
-        "REBOOTCOUNT",
-        "RSTRNT_REBOOTCOUNT",
-    ],
-    created_file="reboot-request",
-    enabled=lambda _: True,
-)
-
-TMT_REBOOT_CORE_SCRIPT = Script(
-    source_filename='tmt-reboot-core',
-    destination_path=None,
-    aliases=[],
-    related_variables=[],
-    enabled=lambda _: True,
-)
-
-# Script handling result reporting, in restraint compatible fashion
-TMT_REPORT_RESULT_SCRIPT = ScriptCreatingFile(
-    source_filename='tmt-report-result',
-    destination_path=None,
-    aliases=[
-        'rstrnt-report-result',
-        'rhts-report-result',
-    ],
-    related_variables=[],
-    created_file="tmt-report-results.yaml",
-    enabled=lambda _: True,
-)
-
-# Script for archiving a file, usable for BEAKERLIB_COMMAND_SUBMIT_LOG
-TMT_FILE_SUBMIT_SCRIPT = Script(
-    source_filename='tmt-file-submit',
-    destination_path=None,
-    aliases=[
-        'rstrnt-report-log',
-        'rhts-submit-log',
-        'rhts_submit_log',
-    ],
-    related_variables=[],
-    enabled=lambda _: True,
-)
-
-# Script handling text execution abortion, in restraint compatible fashion
-TMT_ABORT_SCRIPT = ScriptCreatingFile(
-    source_filename='tmt-abort',
-    destination_path=None,
-    aliases=[
-        'rstrnt-abort',
-        'rhts-abort',
-    ],
-    related_variables=[],
-    created_file="abort",
-    enabled=lambda _: True,
-)
-
-# Profile script for adding SCRIPTS_DEST_DIR to executable paths system-wide.
-# Used only for distributions using rpm-ostree.
-TMT_ETC_PROFILE_D = ScriptTemplate(
-    source_filename='tmt.sh',
-    destination_path=Path("/etc/profile.d/tmt.sh"),
-    aliases=[],
-    related_variables=[],
-    context={
-        'SCRIPTS_DEST_DIR': str(
-            effective_scripts_dest_dir(default=DEFAULT_SCRIPTS_DEST_DIR_OSTREE)
-        )
-    },
-    enabled=lambda guest: guest.facts.is_ostree is True,
-)
-
-
-# List of all available scripts
-SCRIPTS = (
-    TMT_ABORT_SCRIPT,
-    TMT_ETC_PROFILE_D,
-    TMT_FILE_SUBMIT_SCRIPT,
-    TMT_REBOOT_SCRIPT,
-    TMT_REBOOT_CORE_SCRIPT,
-    TMT_REPORT_RESULT_SCRIPT,
-)
+# File containing paths of submitted files
+SUBMITTED_FILES_FILENAME = "submitted-files.log"
 
 
 @container
@@ -288,14 +103,6 @@ class ExecuteStepData(tmt.steps.WhereableStepData, tmt.steps.StepData):
 ExecuteStepDataT = TypeVar('ExecuteStepDataT', bound=ExecuteStepData)
 
 
-class AbortExecute(tmt.utils.GeneralError):
-    """
-    Raised by ``execute`` phases when the entire step should abort.
-    """
-
-    pass
-
-
 @container
 class TestInvocation:
     """
@@ -317,6 +124,11 @@ class TestInvocation:
     process: Optional[subprocess.Popen[bytes]] = None
     process_lock: threading.Lock = simple_field(default_factory=threading.Lock)
 
+    #: If set, there is a callback registered through
+    #: :py:func:`tmt.utils.signals.add_callback` which would be called when
+    #: tmt gets terminated.
+    on_interrupt_callback_token: Optional[int] = None
+
     results: list[Result] = simple_field(default_factory=list)
     check_results: list[CheckResult] = simple_field(default_factory=list)
 
@@ -327,11 +139,24 @@ class TestInvocation:
     end_time: Optional[str] = None
     real_duration: Optional[str] = None
 
+    #: List of exceptions encountered by the invocation.
+    exceptions: list[Exception] = simple_field(default_factory=list)
+
     #: Number of times the test has been restarted.
     _restart_count: int = 0
     #: Number of times the guest has been rebooted.
     _reboot_count: int = 0
 
+    @functools.cached_property
+    def step_workdir(self) -> Path:
+        """
+        Absolute path to which relative paths are relative to.
+        """
+
+        assert self.phase.step.workdir is not None  # narrow type
+
+        return self.phase.step.workdir
+
     @functools.cached_property
     def path(self) -> Path:
         """
@@ -363,9 +188,7 @@ class TestInvocation:
         Invocation directory path relative to step workdir
         """
 
-        assert self.phase.step.workdir is not None  # narrow type
-
-        return self.path.relative_to(self.phase.step.workdir)
+        return self.path.relative_to(self.step_workdir)
 
     @functools.cached_property
     def test_data_path(self) -> Path:
@@ -391,21 +214,21 @@ class TestInvocation:
 
         return self.path / CHECK_DATA
 
+    @functools.cached_property
+    def submission_log_path(self) -> Path:
+        """
+        A path to log containing submitted files paths
+        """
+
+        return self.path / SUBMITTED_FILES_FILENAME
+
     @functools.cached_property
     def reboot_request_path(self) -> Path:
         """
         A path to the reboot request file
         """
 
-        return self.test_data_path / TMT_REBOOT_SCRIPT.created_file
-
-    @functools.cached_property
-    def abort_request_path(self) -> Path:
-        """
-        A path to the abort request file
-        """
-
-        return self.test_data_path / TMT_ABORT_SCRIPT.created_file
+        return self.test_data_path / tmt.steps.scripts.TMT_REBOOT_SCRIPT.created_file
 
     @property
     def soft_reboot_requested(self) -> bool:
@@ -435,14 +258,6 @@ class TestInvocation:
 
         return self.return_code in self.test.restart_on_exit_code
 
-    @property
-    def abort_requested(self) -> bool:
-        """
-        Whether a testing abort was requested
-        """
-
-        return self.abort_request_path.exists()
-
     @property
     def is_guest_healthy(self) -> bool:
         """
@@ -463,6 +278,38 @@ class TestInvocation:
 
         return True
 
+    @property
+    def submitted_files(self) -> list[Path]:
+        """
+        Paths of all files submitted during test
+        """
+
+        if not self.submission_log_path.exists():
+            return []
+
+        # * `self.test_data_path / line` -> absolute log path (`line`
+        #   shall be relative to test data)
+        # * .resolve() -> absolute log path without any `../`,
+        # * .relative_to(...) -> *relative* log path, relative to the
+        #    step workdir, as expected from log paths attached to results
+        #
+        # Note `step_workdir.resolve()`: `resolve()` resolves symlinks
+        # as well, and `relative_to()` might consider `step_workdir`
+        # a different tree if it contains symlinks. Compare resolved
+        # paths then.
+        return [
+            (self.test_data_path / line).resolve().relative_to(self.step_workdir.resolve())
+            for line in self.submission_log_path.read_text().splitlines()
+        ]
+
+    @functools.cached_property
+    def abort(self) -> AbortContext:
+        """
+        Abort context for this invocation.
+        """
+
+        return AbortContext(path=self.test_data_path, logger=self.logger)
+
     def handle_restart(self) -> bool:
         """
         "Restart" the test if the test requested it.
@@ -494,7 +341,7 @@ class TestInvocation:
                 f" and test restart count {self._restart_count}."
             )
 
-            return False
+            raise tmt.utils.RestartMaxAttemptsError("Maximum test restart attempts exceeded.")
 
         if self.test.restart_with_reboot:
             self.hard_reboot_requested = True
@@ -512,7 +359,7 @@ class TestInvocation:
             # test, but the guest may be still booting. Make sure it's
             # alive.
             if not self.guest.reconnect():
-                return False
+                raise tmt.utils.ReconnectTimeoutError("Reconnect timed out.")
 
         self.logger.debug(
             f"Test restart during test '{self.test}'"
@@ -558,22 +405,24 @@ class TestInvocation:
             reboot_data = json.loads(self.reboot_request_path.read_text())
 
             reboot_command: Optional[ShellScript] = None
-            timeout: Optional[int] = None
 
             if reboot_data.get('command'):
                 with suppress(TypeError):
                     reboot_command = ShellScript(reboot_data.get('command'))
 
-            try:
-                timeout = int(reboot_data.get('timeout'))
-            except ValueError:
-                timeout = None
+            if reboot_data.get('timeout'):
+                deadline = Deadline.from_seconds(int(reboot_data.get('timeout')))
+
+            else:
+                deadline = Deadline.from_seconds(tmt.steps.provision.REBOOT_TIMEOUT)
+
+            waiting = Waiting(deadline=deadline)
 
             os.remove(self.reboot_request_path)
             self.guest.push(self.test_data_path)
 
             try:
-                rebooted = self.guest.reboot(hard=False, command=reboot_command, timeout=timeout)
+                rebooted = self.guest.reboot(hard=False, command=reboot_command, waiting=waiting)
 
             except tmt.utils.RunError:
                 if reboot_command is not None:
@@ -586,7 +435,7 @@ class TestInvocation:
             except tmt.steps.provision.RebootModeNotSupportedError:
                 self.logger.warning("Guest does not support soft reboot, trying hard reboot.")
 
-                rebooted = self.guest.reboot(hard=True, timeout=timeout)
+                rebooted = self.guest.reboot(hard=True, waiting=waiting)
 
         if not rebooted:
             raise tmt.utils.RebootTimeoutError("Reboot timed out.")
@@ -595,6 +444,113 @@ class TestInvocation:
 
         return True
 
+    def invoke_test(
+        self,
+        command: ShellScript,
+        *,
+        cwd: Path,
+        env: tmt.utils.Environment,
+        log: tmt.log.LoggingFunction,
+        interactive: bool,
+        timeout: Optional[int],
+    ) -> tmt.utils.CommandOutput:
+        """
+        Start the command which represents the test in this invocation.
+
+        :param cwd: if set, command would be executed in the given directory,
+            otherwise the current working directory is used.
+        :param env: environment variables to combine with the current environment
+            before running the command.
+        :param interactive: if set, the command would be executed in an interactive
+            manner, i.e. with stdout and stdout connected to terminal for live
+            interaction with user.
+        :param timeout: if set, command would be interrupted, if still running,
+            after this many seconds.
+        :param log: a logging function to use for logging of command output. By
+            default, ``logger.debug`` is used.
+        :returns: command output.
+        """
+
+        # Write down process and let tmt kill it if tmt gets interrupted.
+        def _save_process(
+            command: Command, process: subprocess.Popen[bytes], logger: tmt.log.Logger
+        ) -> None:
+            """
+            Record process info in the invocation.
+
+            Called by :py:class:`Command` as the ``on_process_start``
+            callback.
+            """
+
+            with self.process_lock:
+                self.process = process
+
+                self.on_interrupt_callback_token = tmt.utils.signals.add_callback(
+                    self.terminate_process, logger=self.logger
+                )
+
+                logger.debug(
+                    f'Test invocation process spawned with pid {self.process.pid}.', level=3
+                )
+
+        def _reset_process(
+            command: Command,
+            process: subprocess.Popen[bytes],
+            output: CommandOutput,
+            logger: tmt.log.Logger,
+        ) -> None:
+            """
+            Reset process info in the invocation.
+
+            Called by :py:class:`Command` as the ``on_process_end``
+            callback.
+            """
+
+            with self.process_lock:
+                self.process = None
+
+                if self.on_interrupt_callback_token is not None:
+                    tmt.utils.signals.remove_callback(self.on_interrupt_callback_token)
+
+        with Stopwatch() as timer:
+            self.start_time = format_timestamp(timer.start_time)
+
+            try:
+                output = self.guest.execute(
+                    command,
+                    cwd=cwd,
+                    env=env,
+                    join=True,
+                    interactive=interactive,
+                    tty=self.test.tty,
+                    log=log,
+                    timeout=timeout,
+                    on_process_start=_save_process,
+                    on_process_end=_reset_process,
+                    test_session=True,
+                    friendly_command=str(self.test.test),
+                )
+
+                self.return_code = tmt.utils.ProcessExitCodes.SUCCESS
+
+            except tmt.utils.RunError as error:
+                self.exceptions.append(error)
+
+                output = error.output
+
+                self.return_code = error.returncode
+
+                if self.return_code == tmt.utils.ProcessExitCodes.TIMEOUT:
+                    self.logger.debug(f"Test duration '{self.test.duration}' exceeded.")
+
+                elif tmt.utils.ProcessExitCodes.is_pidfile(self.return_code):
+                    self.logger.warning('Test failed to manage its pidfile.')
+
+        self.end_time = format_timestamp(timer.end_time)
+        self.real_duration = format_duration(timer.duration)
+
+        return output
+
     def terminate_process(
         self,
         signal: _signal.Signals = _signal.SIGTERM,
@@ -676,13 +632,11 @@ class ExecutePlugin(tmt.steps.Plugin[ExecuteStepDataT, None]):
     _data_class = ExecuteStepData  # type: ignore[assignment]
 
     # Methods ("how: ..." implementations) registered for the same step.
-    _supported_methods: PluginRegistry[tmt.steps.Method] = PluginRegistry()
+    _supported_methods: PluginRegistry[tmt.steps.Method] = PluginRegistry('step.execute')
 
     # Internal executor is the default implementation
     how = 'tmt'
 
-    scripts: tuple['Script', ...] = ()
-
     _login_after_test: Optional[tmt.steps.Login] = None
 
     #: If set, plugin should run tests only from this discover phase.
@@ -768,7 +722,7 @@ class ExecutePlugin(tmt.steps.Plugin[ExecuteStepDataT, None]):
 
             # Exported metadata is the test's metadata along with other variables like the context
             test_metadata = test._metadata.copy()
-            test_metadata["context"] = self.step.plan._fmf_context.to_spec()
+            test_metadata["context"] = self.step.plan.fmf_context.to_spec()
             self.write(
                 invocation.path / TEST_METADATA_FILENAME,
                 tmt.utils.dict_to_yaml(test_metadata),
@@ -795,37 +749,12 @@ class ExecutePlugin(tmt.steps.Plugin[ExecuteStepDataT, None]):
 
         return invocations
 
-    def prepare_scripts(self, guest: "tmt.steps.provision.Guest") -> None:
-        """
-        Prepare additional scripts for testing
-        """
-
-        # Make sure scripts directory exists
-        command = Command("mkdir", "-p", f"{guest.scripts_path}")
-
-        if not guest.facts.is_superuser:
-            command = Command("sudo") + command
-
-        guest.execute(command)
-
-        # Install all scripts on guest
-        for script in self.scripts:
-            with script as source:
-                for filename in [script.source_filename, *script.aliases]:
-                    if script.enabled(guest):
-                        guest.push(
-                            source=source,
-                            destination=script.destination_path or guest.scripts_path / filename,
-                            options=["-p", "--chmod=755"],
-                            superuser=guest.facts.is_superuser is not True,
-                        )
-
     def _tmt_report_results_filepath(self, invocation: TestInvocation) -> Path:
         """
         Create path to test's ``tmt-report-result`` file
         """
 
-        return invocation.test_data_path / TMT_REPORT_RESULT_SCRIPT.created_file
+        return invocation.test_data_path / tmt.steps.scripts.TMT_REPORT_RESULT_SCRIPT.created_file
 
     def _load_custom_results_file(self, invocation: TestInvocation) -> ResultCollection:
         """
@@ -946,7 +875,7 @@ class ExecutePlugin(tmt.steps.Plugin[ExecuteStepDataT, None]):
                 partial_result.start_time = invocation.start_time
                 partial_result.end_time = invocation.end_time
                 partial_result.duration = invocation.real_duration
-                partial_result.context = self.step.plan._fmf_context
+                partial_result.context = self.step.plan.fmf_context
 
             custom_results.append(partial_result)
 
@@ -1097,12 +1026,13 @@ class ExecutePlugin(tmt.steps.Plugin[ExecuteStepDataT, None]):
         *,
         event: CheckEvent,
         invocation: TestInvocation,
+        checks: Sequence[Check],
         environment: Optional[tmt.utils.Environment] = None,
         logger: tmt.log.Logger,
     ) -> list[CheckResult]:
         results: list[CheckResult] = []
 
-        for check in invocation.test.check:
+        for check in checks:
             with Stopwatch() as timer:
                 check_results = check.go(
                     event=event, invocation=invocation, environment=environment, logger=logger
@@ -1129,6 +1059,7 @@ class ExecutePlugin(tmt.steps.Plugin[ExecuteStepDataT, None]):
         return self._run_checks_for_test(
             event=CheckEvent.BEFORE_TEST,
             invocation=invocation,
+            checks=invocation.test.check,
             environment=environment,
             logger=logger,
         )
@@ -1143,6 +1074,22 @@ class ExecutePlugin(tmt.steps.Plugin[ExecuteStepDataT, None]):
         return self._run_checks_for_test(
             event=CheckEvent.AFTER_TEST,
             invocation=invocation,
+            checks=invocation.test.check,
+            environment=environment,
+            logger=logger,
+        )
+
+    def run_internal_checks(
+        self,
+        *,
+        invocation: TestInvocation,
+        environment: Optional[tmt.utils.Environment] = None,
+        logger: tmt.log.Logger,
+    ) -> list[CheckResult]:
+        return self._run_checks_for_test(
+            event=CheckEvent.AFTER_TEST,
+            invocation=invocation,
+            checks=CheckPlugin.internal_checks(logger),
             environment=environment,
             logger=logger,
         )
@@ -1306,8 +1253,8 @@ class Execute(tmt.steps.Step):
             if not test.where:
                 result_guests = guests
 
-            for guest in result_guests:
-                results.append(
+            results.extend(
+                [
                     Result(
                         name=test.name,
                         serial_number=test.serial_number,
@@ -1318,7 +1265,9 @@ class Execute(tmt.steps.Step):
                             role=guest[1],
                         ),
                     )
-                )
+                    for guest in result_guests
+                ]
+            )
         return results
 
     def go(self, force: bool = False) -> None:
@@ -1393,10 +1342,10 @@ class Execute(tmt.steps.Step):
             if outcome.exc:
                 outcome.logger.fail(str(outcome.exc))
 
-                # Special exception serving as a signal to not run any more phases.
-                # Not necessarily a failed task calling for the final exception
-                # to be raised, crashing the whole run.
-                if isinstance(outcome.exc, AbortExecute):
+                # Special exceptions serving as signals to not run any more
+                # phases. Not necessarily a failed task calling for the final
+                # exception to be raised, crashing the whole run.
+                if isinstance(outcome.exc, (AbortStep, tmt.utils.signals.Interrupted)):
                     break
 
                 failed_tasks.append(outcome)
@@ -1423,6 +1372,8 @@ class Execute(tmt.steps.Step):
                 causes=[outcome.exc for outcome in failed_tasks if outcome.exc is not None],
             )
 
+        self._assert_required_tests_executed()
+
     def results(self) -> list["tmt.result.Result"]:
         """
         Results from executed tests
@@ -1460,3 +1411,26 @@ class Execute(tmt.steps.Step):
             for test_origin in tests
             if test_origin.test.serial_number not in referenced_serial_numbers
         ]
+
+    def _assert_required_tests_executed(self) -> None:
+        """
+        Assert all required tests were executed.
+        """
+
+        for result, test_origin in self.results_for_tests(self.plan.discover.required_tests):
+            if test_origin is None:
+                continue
+            test = test_origin.test
+
+            # This one should never happen, but just in case
+            if result is None:
+                raise tmt.utils.ExecuteError(f"Required test '{test.name}' was not executed.")
+            if result.result == ResultOutcome.PENDING:
+                raise tmt.utils.ExecuteError(
+                    f"Required test '{result.name}' on guest '{result.guest.name}' "
+                    f"is still pending."
+                )
+            if result.result == ResultOutcome.SKIP:
+                raise tmt.utils.ExecuteError(
+                    f"Required test '{result.name}' on guest '{result.guest.name}' was skipped."
+                )
