@@ -1,5 +1,4 @@
 import os
-import subprocess
 import textwrap
 from typing import Any, Optional, cast
 
@@ -11,28 +10,25 @@ import tmt.log
 import tmt.options
 import tmt.steps
 import tmt.steps.execute
+import tmt.steps.scripts
 import tmt.utils
+import tmt.utils.signals
 import tmt.utils.themes
 from tmt.container import container, field
-from tmt.result import BaseResult, Result, ResultOutcome
+from tmt.result import Result, ResultOutcome
 from tmt.steps import safe_filename
+from tmt.steps.abort import AbortStep
 from tmt.steps.execute import (
-    SCRIPTS,
     TEST_OUTPUT_FILENAME,
-    TMT_REBOOT_SCRIPT,
-    AbortExecute,
     TestInvocation,
 )
-from tmt.steps.provision import Guest
+from tmt.steps.provision import DEFAULT_PULL_OPTIONS, Guest, TransferOptions
+from tmt.steps.report.display import ResultRenderer
 from tmt.utils import (
-    Command,
     Environment,
     EnvVarValue,
     Path,
     ShellScript,
-    Stopwatch,
-    format_duration,
-    format_timestamp,
 )
 from tmt.utils.themes import style
 
@@ -396,7 +392,6 @@ class ExecuteInternal(tmt.steps.execute.ExecutePlugin[ExecuteInternalData]):
     def __init__(self, **kwargs: Any):
         super().__init__(**kwargs)
         self._previous_progress_message = ""
-        self.scripts = SCRIPTS
 
     def _test_environment(
         self,
@@ -412,7 +407,12 @@ class ExecuteInternal(tmt.steps.execute.ExecutePlugin[ExecuteInternalData]):
         extra_environment = extra_environment or Environment()
 
         environment = extra_environment.copy()
-        environment.update(invocation.test.environment)
+        environment.update(
+            {
+                **invocation.guest.environment,
+                **invocation.test.environment,
+            }
+        )
         assert self.parent is not None
         assert isinstance(self.parent, tmt.steps.execute.Execute)
         assert self.parent.plan.my_run is not None
@@ -424,7 +424,9 @@ class ExecuteInternal(tmt.steps.execute.ExecutePlugin[ExecuteInternalData]):
             effective_pidfile_root() / TEST_PIDFILE_LOCK_FILENAME
         )
         environment["TMT_TEST_NAME"] = EnvVarValue(invocation.test.name)
+        environment["TMT_TEST_INVOCATION_PATH"] = EnvVarValue(invocation.path)
         environment["TMT_TEST_DATA"] = EnvVarValue(invocation.test_data_path)
+        environment["TMT_TEST_SUBMITTED_FILES"] = EnvVarValue(invocation.submission_log_path)
         environment['TMT_TEST_SERIAL_NUMBER'] = EnvVarValue(str(invocation.test.serial_number))
         environment['TMT_TEST_ITERATION_ID'] = EnvVarValue(
             f"{self.parent.plan.my_run.unique_id}-{invocation.test.serial_number}"
@@ -433,7 +435,7 @@ class ExecuteInternal(tmt.steps.execute.ExecutePlugin[ExecuteInternalData]):
             invocation.path / tmt.steps.execute.TEST_METADATA_FILENAME
         )
         environment["TMT_REBOOT_REQUEST"] = EnvVarValue(
-            invocation.test_data_path / TMT_REBOOT_SCRIPT.created_file
+            invocation.test_data_path / tmt.steps.scripts.TMT_REBOOT_SCRIPT.created_file
         )
 
         # Set the restraint-compatible mode if enabled
@@ -442,7 +444,7 @@ class ExecuteInternal(tmt.steps.execute.ExecutePlugin[ExecuteInternalData]):
         )
 
         # Set all supported reboot variables
-        for reboot_variable in TMT_REBOOT_SCRIPT.related_variables:
+        for reboot_variable in tmt.steps.scripts.TMT_REBOOT_SCRIPT.related_variables:
             environment[reboot_variable] = EnvVarValue(str(invocation._reboot_count))
         environment['TMT_TEST_RESTART_COUNT'] = EnvVarValue(str(invocation._restart_count))
 
@@ -518,7 +520,7 @@ class ExecuteInternal(tmt.steps.execute.ExecutePlugin[ExecuteInternalData]):
             guest.push(
                 source=test_wrapper_filepath,
                 destination=test_wrapper_filepath,
-                options=["-s", "-p", "--chmod=755"],
+                options=TransferOptions(protect_args=True, preserve_perms=True, chmod=0o755),
             )
 
             return test_wrapper_filepath
@@ -561,91 +563,86 @@ class ExecuteInternal(tmt.steps.execute.ExecutePlugin[ExecuteInternalData]):
                 key=key, value=value, color=color, shift=shift, level=level, topic=topic
             )
 
-        def _save_process(
-            command: Command, process: subprocess.Popen[bytes], logger: tmt.log.Logger
-        ) -> None:
-            with invocation.process_lock:
-                invocation.process = process
-
         # TODO: do we want timestamps? Yes, we do, leaving that for refactoring later,
         # to use some reusable decorator.
         invocation.check_results = self.run_checks_before_test(
             invocation=invocation, environment=environment, logger=logger
         )
 
-        # Execute the test, save the output and return code
-        with Stopwatch() as timer:
-            invocation.start_time = format_timestamp(timer.start_time)
+        # Pick the proper timeout for the test
+        timeout: Optional[int]
 
-            timeout: Optional[int]
+        if self.data.interactive:
+            if test.duration:
+                logger.warning('Ignoring requested duration, not supported in interactive mode.')
 
-            if self.data.interactive:
-                if test.duration:
-                    logger.warning(
-                        'Ignoring requested duration, not supported in interactive mode.'
-                    )
+            timeout = None
 
-                timeout = None
+        elif self.data.ignore_duration:
+            logger.debug("Test duration is not effective due to ignore-duration option.")
+            timeout = None
 
-            elif self.data.ignore_duration:
-                logger.debug("Test duration is not effective due ignore-duration option.")
-                timeout = None
+        else:
+            timeout = tmt.utils.duration_to_seconds(
+                test.duration, tmt.base.DEFAULT_TEST_DURATION_L1
+            )
 
-            else:
-                timeout = tmt.utils.duration_to_seconds(
-                    test.duration, tmt.base.DEFAULT_TEST_DURATION_L1
-                )
+        if logger.verbosity_level >= 1:
+            shift = 1 if self.verbosity_level < 2 else 2
+            logger.verbose(
+                'duration limit',
+                f"{timeout}{' seconds' if timeout is not None else ''}",
+                color="yellow",
+                shift=shift,
+                level=1,
+            )
 
-            try:
-                output = guest.execute(
-                    remote_command,
-                    cwd=workdir,
-                    env=environment,
-                    join=True,
-                    interactive=self.data.interactive,
-                    tty=test.tty,
-                    log=_test_output_logger,
-                    timeout=timeout,
-                    on_process_start=_save_process,
-                    test_session=True,
-                    friendly_command=str(test.test),
-                )
-                invocation.return_code = 0
-                stdout = output.stdout
-            except tmt.utils.RunError as error:
-                stdout = error.stdout
-
-                invocation.return_code = error.returncode
-                if invocation.return_code == tmt.utils.ProcessExitCodes.TIMEOUT:
-                    logger.debug(f"Test duration '{test.duration}' exceeded.")
-
-                elif tmt.utils.ProcessExitCodes.is_pidfile(invocation.return_code):
-                    logger.warning('Test failed to manage its pidfile.')
-
-        with invocation.process_lock:
-            invocation.process = None
-
-        invocation.end_time = format_timestamp(timer.end_time)
-        invocation.real_duration = format_duration(timer.duration)
+        # And invoke the test process.
+        output = invocation.invoke_test(
+            remote_command,
+            cwd=workdir,
+            env=environment,
+            interactive=self.data.interactive,
+            log=_test_output_logger,
+            timeout=timeout,
+        )
 
         # Save the captured output. Do not let the follow-up pulls
         # overwrite it.
-        self.write(invocation.path / TEST_OUTPUT_FILENAME, stdout or '', mode='a', level=3)
+        self.write(invocation.path / TEST_OUTPUT_FILENAME, output.stdout or '', mode='a', level=3)
+
+        # Reset `has-rsync` fact: tmt is expected to install rsync if it
+        # is missing after a test. To achieve that, pretend we don't
+        # know whether rsync is installed, and let any attempt to use
+        # rsync answer and react before calling the command.
+        guest.facts.has_rsync = None
+
+        def pull_from_guest() -> None:
+            if not invocation.is_guest_healthy:
+                return
+
+            options = test.test_framework.get_pull_options(
+                invocation, DEFAULT_PULL_OPTIONS, logger
+            )
+            # Do not overwrite the captured output
+            options.exclude.append(str(invocation.path / TEST_OUTPUT_FILENAME))
+
+            try:
+                guest.pull(source=invocation.path, options=options)
+
+                # Fetch plan data content as well in order to prevent
+                # losing logs if the guest becomes later unresponsive.
+                guest.pull(source=self.step.plan.data_directory)
+
+            # Handle failing to pull test artifacts after guest becoming
+            # unresponsive. If not handled test would stay in 'pending' state.
+            # See issue https://github.com/teemtee/tmt/issues/3647.
+            except Exception as exc:
+                invocation.exceptions.append(exc)
 
         # Fetch #1: we need logs and everything the test produced so we could
         # collect its results.
-        if invocation.is_guest_healthy:
-            guest.pull(
-                source=invocation.path,
-                extend_options=[
-                    *test.test_framework.get_pull_options(invocation, logger),
-                    '--exclude',
-                    str(invocation.path / TEST_OUTPUT_FILENAME),
-                ],
-            )
-            # Fetch plan data content as well in order to prevent
-            # losing logs if the guest becomes later unresponsive.
-            guest.pull(source=self.step.plan.data_directory)
+        pull_from_guest()
 
         # Run after-test checks before extracting results
         invocation.check_results += self.run_checks_after_test(
@@ -657,26 +654,17 @@ class ExecuteInternal(tmt.steps.execute.ExecutePlugin[ExecuteInternalData]):
         # results after a successful reboot in the middle of a test.
         invocation.results = self.extract_results(invocation, logger)
 
-        if invocation.is_guest_healthy:
-            # Fetch #2: after-test checks might have produced remote files as well,
-            # we need to fetch them too.
-            guest.pull(
-                source=invocation.path,
-                extend_options=[
-                    *test.test_framework.get_pull_options(invocation, logger),
-                    '--exclude',
-                    str(invocation.path / TEST_OUTPUT_FILENAME),
-                ],
-            )
-            # Fetch plan data content as well in order to prevent
-            # losing logs if the guest becomes later unresponsive.
-            guest.pull(source=self.step.plan.data_directory)
+        # Fetch #2: after-test checks might have produced remote files as well,
+        # we need to fetch them too.
+        pull_from_guest()
 
         # Attach check results to every test result. There might be more than one,
         # and it's hard to pick the main one, who knows what custom results might
         # cover, so let's make sure every single result can lead to check results
         # related to its lifetime.
         for result in invocation.results:
+            if result.name == invocation.test.name:
+                result.log.extend(invocation.submitted_files)
             result.check = invocation.check_results
 
         return invocation.results
@@ -712,12 +700,10 @@ class ExecuteInternal(tmt.steps.execute.ExecutePlugin[ExecuteInternalData]):
         Execute tests on provided guest
         """
 
-        # Prepare tests and helper scripts, check options
-        test_invocations = self.prepare_tests(guest, logger)
+        assert self.workdir is not None  # narrow type
 
-        # Prepare scripts, except localhost guest
-        if not guest.localhost:
-            self.prepare_scripts(guest)
+        # Prepare tests, check options
+        test_invocations = self.prepare_tests(guest, logger)
 
         # Push workdir to guest and execute tests
         guest.push()
@@ -725,10 +711,12 @@ class ExecuteInternal(tmt.steps.execute.ExecutePlugin[ExecuteInternalData]):
         index = 0
 
         # TODO: plugin does not return any value. Results are exchanged
-        # via `self.results`, to signal abort we need a bigger gun. Once
-        # we get back to refactoring the plugin, this would turn into a
-        # better way of transporting "plugin outcome" back to the step.
-        abort_execute_exception: Optional[AbortExecute] = None
+        # via `self.results`, to signal abort or interruption we need a
+        # bigger gun. Once we get back to refactoring the plugin, this
+        # would turn into a better way of transporting "plugin outcome"
+        # back to the step.
+        abort_execute_exception: Optional[AbortStep] = None
+        interrupt_exception: Optional[tmt.utils.signals.Interrupted] = None
 
         with UpdatableMessage(self) as progress_bar:
             while index < len(test_invocations):
@@ -757,18 +745,14 @@ class ExecuteInternal(tmt.steps.execute.ExecutePlugin[ExecuteInternalData]):
                         if invocation.handle_restart():
                             continue
 
-                    except tmt.utils.RebootTimeoutError:
+                    except (
+                        tmt.utils.RebootTimeoutError,
+                        tmt.utils.ReconnectTimeoutError,
+                        tmt.utils.RestartMaxAttemptsError,
+                    ) as error:
+                        invocation.exceptions.append(error)
                         for result in invocation.results:
                             result.result = ResultOutcome.ERROR
-                            result.note.append('reboot timeout')
-
-                    else:
-                        for result in invocation.results:
-                            result.result = ResultOutcome.ERROR
-                            result.note.append(
-                                'crashed too many times, '
-                                'you may want to set restart-max-count larger'
-                            )
 
                 # Handle reboot
                 if invocation.reboot_requested:
@@ -777,57 +761,54 @@ class ExecuteInternal(tmt.steps.execute.ExecutePlugin[ExecuteInternalData]):
                     try:
                         if invocation.handle_reboot():
                             continue
-                    except tmt.utils.RebootTimeoutError:
+                    except tmt.utils.RebootTimeoutError as error:
+                        invocation.exceptions.append(error)
                         for result in invocation.results:
                             result.result = ResultOutcome.ERROR
-                            result.note.append('reboot timeout')
 
-                if invocation.abort_requested:
-                    for result in invocation.results:
-                        # In case of aborted all results in list will be aborted
-                        result.note.append('aborted')
-
-                self._results.extend(invocation.results)
-                self.step.plan.execute.update_results(self.results())
-                self.step.plan.execute.save()
-
-                # If test duration information is missing, print 8 spaces to keep indentation
-                def _format_duration(result: BaseResult) -> str:
-                    return style(result.duration, fg='cyan') if result.duration else 8 * ' '
-
-                for result in invocation.results:
-                    logger.verbose(
-                        f"{_format_duration(result)} {result.show()} [{progress}]", shift=shift
-                    )
-
-                    for check_result in result.check:
-                        # Indent the check one extra level, to make it clear it belongs to
-                        # a parent test.
-                        logger.verbose(
-                            f'{_format_duration(check_result)} '
-                            f'{" " * tmt.utils.INDENT}'
-                            f'{check_result.show()} '
-                            f'({check_result.event.value} check)',
-                            shift=shift,
-                        )
-
-                abort_execute = invocation.abort_requested or (
+                # Handle abort signs
+                if invocation.abort.requested or (
                     self.data.exit_first
                     and any(
                         result.result in (ResultOutcome.FAIL, ResultOutcome.ERROR)
                         for result in invocation.results
                     )
-                )
-
-                if abort_execute:
-                    if invocation.abort_requested:
+                ):
+                    if invocation.abort.requested:
                         abort_message = f'Test {test.name} aborted, stopping execution.'
 
                     else:
                         abort_message = f'Test {test.name} failed, stopping execution.'
 
-                    abort_execute_exception = AbortExecute(abort_message)
+                    abort_execute_exception = AbortStep(abort_message)
 
+                # Handle interrupt
+                if tmt.utils.signals.INTERRUPT_PENDING.is_set():
+                    interrupt_exception = tmt.utils.signals.Interrupted()
+
+                    invocation.exceptions.append(interrupt_exception)
+
+                # Execute internal checks
+                invocation.check_results += self.run_internal_checks(
+                    invocation=invocation,
+                    environment=self._test_environment(
+                        invocation=invocation, extra_environment=extra_environment, logger=logger
+                    ),
+                    logger=logger,
+                )
+
+                self._results.extend(invocation.results)
+                self.step.plan.execute.update_results(self.results())
+                self.step.plan.execute.save()
+
+                ResultRenderer(
+                    basepath=self.workdir,
+                    logger=logger,
+                    shift=shift,
+                    variables={'PROGRESS': f'[{progress}]'},
+                ).print_results(invocation.results)
+
+                if abort_execute_exception is not None or interrupt_exception is not None:
                     progress_bar.clear()
 
                     break
@@ -844,7 +825,7 @@ class ExecuteInternal(tmt.steps.execute.ExecutePlugin[ExecuteInternalData]):
                     else:
                         cwd = self.discover.workdir / test.path.unrooted()
                     self._login_after_test.after_test(
-                        result,
+                        invocation.results,
                         cwd=cwd,
                         env=self._test_environment(
                             invocation=invocation,
@@ -860,6 +841,9 @@ class ExecuteInternal(tmt.steps.execute.ExecutePlugin[ExecuteInternalData]):
         if abort_execute_exception:
             raise abort_execute_exception
 
+        if interrupt_exception:
+            raise interrupt_exception
+
     def results(self) -> list[Result]:
         """
         Return test results
