@@ -6,13 +6,13 @@ import requests
 import tmt
 import tmt.base
 import tmt.log
-import tmt.options
+import tmt.result
 import tmt.steps
 import tmt.steps.prepare
 import tmt.steps.provision
 import tmt.utils
 from tmt.container import container, field
-from tmt.result import PhaseResult
+from tmt.result import ResultOutcome
 from tmt.steps.provision import (
     ANSIBLE_COLLECTION_PLAYBOOK_PATTERN,
     AnsibleApplicable,
@@ -24,6 +24,7 @@ from tmt.utils import (
     ENVFILE_RETRY_SESSION_RETRIES,
     Path,
     PrepareError,
+    RunError,
     normalize_string_list,
     retry_session,
 )
@@ -40,10 +41,13 @@ class PrepareAnsibleData(tmt.steps.prepare.PrepareStepData):
         default_factory=list,
         option=('-p', '--playbook'),
         multiple=True,
-        metavar='PATH|URL',
+        metavar='PATH|URL|NAMESPACE.COLLECTION.PLAYBOOK',
         help="""
-             Path or URL of an ansible playbook to run.
-             The playbook path must be relative to the metadata tree root.
+             Path or URL of an Ansible playbook, or a playbook
+             bundled within a collection, to run on a guest.
+             Playbook ``PATH`` must be relative to the metadata tree
+             root, if the metadata tree exists, or to the current
+             working directory.
              """,
         normalize=tmt.utils.normalize_string_list,
     )
@@ -166,30 +170,43 @@ class PrepareAnsible(tmt.steps.prepare.PreparePlugin[PrepareAnsibleData]):
 
     _data_class = PrepareAnsibleData
 
+    @property
+    def _preserved_workdir_members(self) -> set[str]:
+        return {
+            *super()._preserved_workdir_members,
+            # Include directories storing individual playbook logs.
+            *{f'playbook-{i}' for i in range(len(self.data.playbook))},
+        }
+
     def go(
         self,
         *,
         guest: 'Guest',
         environment: Optional[tmt.utils.Environment] = None,
         logger: tmt.log.Logger,
-    ) -> list[PhaseResult]:
+    ) -> tmt.steps.PluginOutcome:
         """
         Prepare the guests
         """
 
-        results = super().go(guest=guest, environment=environment, logger=logger)
+        assert self.step.workdir is not None
+        assert self.workdir is not None
+
+        outcome = super().go(guest=guest, environment=environment, logger=logger)
 
         # Apply each playbook on the guest
-        for _playbook in self.data.playbook:
+        for playbook_index, _playbook in enumerate(self.data.playbook):
             logger.info('playbook', _playbook, 'green')
 
+            playbook_name = f'{self.name} / {_playbook}'
             lowercased_playbook = _playbook.lower()
 
-            def normalize_remote_playbook(raw_playbook: str) -> Path:
-                assert self.step.plan.my_run is not None  # narrow type
-                assert self.step.plan.my_run.tree is not None  # narrow type
-                assert self.step.plan.my_run.tree.root is not None  # narrow type
-                root_path = self.step.plan.my_run.tree.root
+            playbook_record_dirpath = self.workdir / f'playbook-{playbook_index}' / guest.safe_name
+            playbook_log_filepath = playbook_record_dirpath / 'output.txt'
+
+            def normalize_remote_playbook(raw_playbook: str) -> tuple[Path, AnsibleApplicable]:
+                assert self.step.workdir is not None  # narrow type
+                root_path = self.step.workdir
 
                 try:
                     with retry_session(
@@ -216,38 +233,87 @@ class PrepareAnsible(tmt.steps.prepare.PreparePlugin[PrepareAnsibleData]):
                     file.write(response.content)
                     file.flush()
 
-                    return Path(file.name).relative_to(root_path)
+                    return root_path, Path(file.name).relative_to(root_path)
 
-            def normalize_local_playbook(raw_playbook: str) -> Path:
+            def normalize_local_playbook(raw_playbook: str) -> tuple[Path, AnsibleApplicable]:
                 if raw_playbook.startswith('file://'):
-                    return Path(raw_playbook[7:])
+                    return self.step.plan.anchor_path, Path(raw_playbook[7:])
 
-                return Path(raw_playbook)
+                return self.step.plan.anchor_path, Path(raw_playbook)
 
-            def normalize_collection_playbook(raw_playbook: str) -> AnsibleCollectionPlaybook:
-                return AnsibleCollectionPlaybook(raw_playbook)
+            def normalize_collection_playbook(raw_playbook: str) -> tuple[Path, AnsibleApplicable]:
+                return self.step.plan.anchor_path, AnsibleCollectionPlaybook(raw_playbook)
 
-            playbook: AnsibleApplicable
+            try:
+                playbook_record_dirpath.mkdir(parents=True, exist_ok=True)
 
-            if lowercased_playbook.startswith(('http://', 'https://')):
-                playbook = normalize_remote_playbook(lowercased_playbook)
+                if lowercased_playbook.startswith(('http://', 'https://')):
+                    playbook_root, playbook = normalize_remote_playbook(lowercased_playbook)
 
-            elif lowercased_playbook.startswith('file://'):
-                playbook = normalize_local_playbook(lowercased_playbook)
+                elif lowercased_playbook.startswith('file://'):
+                    playbook_root, playbook = normalize_local_playbook(lowercased_playbook)
 
-            elif ANSIBLE_COLLECTION_PLAYBOOK_PATTERN.match(lowercased_playbook):
-                playbook = normalize_collection_playbook(lowercased_playbook)
+                elif ANSIBLE_COLLECTION_PLAYBOOK_PATTERN.match(lowercased_playbook):
+                    playbook_root, playbook = normalize_collection_playbook(lowercased_playbook)
+
+                else:
+                    playbook_root, playbook = normalize_local_playbook(lowercased_playbook)
+
+                output = guest.ansible(
+                    playbook,
+                    playbook_root=playbook_root,
+                    extra_args=self.data.extra_args,
+                )
+
+            except RunError as exc:
+                self.write(
+                    playbook_log_filepath,
+                    '\n'.join(
+                        tmt.utils.render_command_report(label=playbook_name, output=exc.output)
+                    ),
+                )
+
+                outcome.results.append(
+                    tmt.result.PhaseResult(
+                        name=playbook_name,
+                        result=ResultOutcome.FAIL,
+                        note=tmt.utils.render_exception_as_notes(exc),
+                        log=[playbook_log_filepath.relative_to(self.step.workdir)],
+                    )
+                )
+
+                outcome.exceptions.append(exc)
+
+                return outcome
+
+            except Exception as exc:
+                outcome.results.append(
+                    tmt.result.PhaseResult(
+                        name=playbook_name,
+                        result=ResultOutcome.ERROR,
+                        note=tmt.utils.render_exception_as_notes(exc),
+                    )
+                )
+
+                outcome.exceptions.append(exc)
+
+                return outcome
 
             else:
-                playbook = normalize_local_playbook(lowercased_playbook)
+                self.write(
+                    playbook_log_filepath,
+                    '\n'.join(tmt.utils.render_command_report(label=playbook_name, output=output)),
+                )
 
-            guest.ansible(
-                playbook,
-                playbook_root=self.step.plan.anchor_path,
-                extra_args=self.data.extra_args,
-            )
+                outcome.results.append(
+                    tmt.result.PhaseResult(
+                        name=playbook_name,
+                        result=ResultOutcome.PASS,
+                        log=[playbook_log_filepath.relative_to(self.step.workdir)],
+                    )
+                )
 
-        return results
+        return outcome
 
     def essential_requires(self) -> list[tmt.base.Dependency]:
         """
