@@ -24,7 +24,6 @@ from tmt.package_managers import (
     PackageUrl,
 )
 from tmt.package_managers.bootc import Bootc, BootcEngine
-from tmt.result import PhaseResult
 from tmt.steps.provision import Guest
 from tmt.utils import Command, Path, ShellScript
 
@@ -157,7 +156,7 @@ class InstallBase(tmt.utils.Common):
         workdir = cast(PrepareInstall, self.parent).step.workdir
         if not workdir:
             raise tmt.utils.GeneralError('workdir should not be empty')
-        self.package_directory = workdir / 'packages'
+        self.package_directory = workdir / self.guest.safe_name / 'packages'
         self.package_directory.mkdir(parents=True)
 
         # Copy local packages into workdir, push to guests
@@ -200,6 +199,8 @@ class InstallBase(tmt.utils.Common):
             # Wrapping them with try/except gives us a chance to attach the original exception
             # to whatever the code may raise, and therefore preserve the information attached
             # to the original exception.
+            self.warn('Installation failed, trying again after metadata refresh.')
+
             try:
                 # Refresh cache in case of recent but not updated change do repodata
                 self.guest.package_manager.refresh_metadata()
@@ -586,8 +587,14 @@ class InstallApt(InstallBase):
 class InstallBootc(InstallBase):
     """Install packages using bootc container image mode"""
 
+    @property
+    def _engine(self) -> BootcEngine:
+        assert isinstance(self.guest.package_manager.engine, BootcEngine)
+
+        return self.guest.package_manager.engine
+
     def install_from_repository(self) -> None:
-        self.guest.package_manager.engine.install(
+        self._engine.install(
             *self.list_installables("package", *self.packages),
             options=Options(
                 excluded_packages=self.exclude,
@@ -596,7 +603,7 @@ class InstallBootc(InstallBase):
         )
 
     def install_from_url(self) -> None:
-        self.guest.package_manager.engine.install(
+        self._engine.install(
             *self.list_installables("remote package", *self.remote_packages),
             options=Options(
                 excluded_packages=self.exclude,
@@ -605,19 +612,29 @@ class InstallBootc(InstallBase):
         )
 
     def install_local(self) -> None:
+        # Make sure the containerfile session has been initialized. The
+        # engine would do it for us, but we need to prepend some
+        # directives first before using the engine.
+        self._engine.open_containerfile_directives()
+
         # Filelist for packages on the guest
         filelist = [
             PackagePath(self.package_directory / filename.name) for filename in self.local_packages
         ]
 
-        cast(BootcEngine, self.guest.package_manager.engine).containerfile_directives.append(
-            f'RUN mkdir -p {self.package_directory}'
-        )
-        cast(BootcEngine, self.guest.package_manager.engine).containerfile_directives.append(
-            f'COPY {" ".join(str(file) for file in filelist)} {self.package_directory}'
-        )
+        self._engine.containerfile_directives.append(f'RUN mkdir -p {self.package_directory}')
 
-        self.guest.package_manager.engine.install(
+        assert self.guest.parent is not None
+        assert hasattr(self.guest.parent, 'workdir')
+
+        workdir = self.guest.parent.workdir
+        assert isinstance(workdir, Path)
+
+        files = " ".join(str(file.relative_to(workdir)) for file in filelist)
+
+        self._engine.containerfile_directives.append(f'COPY {files} {self.package_directory}')
+
+        self._engine.install(
             *filelist,
             options=Options(
                 excluded_packages=self.exclude,
@@ -626,7 +643,7 @@ class InstallBootc(InstallBase):
             ),
         )
 
-        self.guest.package_manager.engine.reinstall(
+        self._engine.reinstall(
             *filelist,
             options=Options(
                 excluded_packages=self.exclude,
@@ -638,7 +655,7 @@ class InstallBootc(InstallBase):
     def install_debuginfo(self) -> None:
         packages = self.list_installables("debuginfo", *self.debuginfo_packages)
 
-        self.guest.package_manager.engine.install_debuginfo(
+        self._engine.install_debuginfo(
             *packages,
             options=Options(
                 excluded_packages=self.exclude,
@@ -649,7 +666,7 @@ class InstallBootc(InstallBase):
         # Check the packages are installed on the guest because 'debuginfo-install'
         # returns 0 even though it didn't manage to install the required packages
         if not self.skip_missing:
-            self.guest.package_manager.engine.check_presence(
+            self._engine.check_presence(
                 *[Package(f'{package}-debuginfo') for package in self.debuginfo_packages]
             )
 
@@ -872,16 +889,16 @@ class PrepareInstall(tmt.steps.prepare.PreparePlugin[PrepareInstallData]):
         guest: 'Guest',
         environment: Optional[tmt.utils.Environment] = None,
         logger: tmt.log.Logger,
-    ) -> list[PhaseResult]:
+    ) -> tmt.steps.PluginOutcome:
         """
         Perform preparation for the guests
         """
 
-        results = super().go(guest=guest, environment=environment, logger=logger)
+        outcome = super().go(guest=guest, environment=environment, logger=logger)
 
         # Nothing to do in dry mode
         if self.is_dry_run:
-            return results
+            return outcome
 
         # Pick the right implementation
         # TODO: it'd be nice to use a "plugin registry" and make the
@@ -975,4 +992,4 @@ class PrepareInstall(tmt.steps.prepare.PreparePlugin[PrepareInstallData]):
         # ... and install packages.
         installer.install()
 
-        return results
+        return outcome
