@@ -1,3 +1,4 @@
+import threading
 from typing import Any, Optional, cast
 
 import fmf
@@ -8,11 +9,11 @@ import tmt.log
 import tmt.steps
 import tmt.steps.prepare
 import tmt.utils
+import tmt.utils.git
 from tmt.container import container, field
-from tmt.result import PhaseResult
 from tmt.steps import safe_filename
-from tmt.steps.provision import Guest
-from tmt.utils import ShellScript
+from tmt.steps.provision import Guest, TransferOptions
+from tmt.utils import Command, EnvVarValue, ShellScript
 
 PREPARE_WRAPPER_FILENAME = 'tmt-prepare-wrapper.sh'
 
@@ -30,6 +31,27 @@ class PrepareShellData(tmt.steps.prepare.PrepareStepData):
         unserialize=lambda serialized: [ShellScript(script) for script in serialized],
     )
 
+    url: Optional[str] = field(
+        default=None,
+        option='--url',
+        metavar='REPOSITORY',
+        help="""
+            URL of a repository to clone. It will be pushed to guests before
+            running any scripts, the path on the guest will be stored in
+            a step variable.
+            """,
+    )
+
+    ref: Optional[str] = field(
+        default=None,
+        option='--ref',
+        metavar='REVISION',
+        help="""
+            Branch, tag or commit to checkout in the git repository
+            cloned when ``url`` is specified.
+            """,
+    )
+
     # ignore[override] & cast: two base classes define to_spec(), with conflicting
     # formal types.
     def to_spec(self) -> dict[str, Any]:  # type: ignore[override]
@@ -56,9 +78,25 @@ class PrepareShell(tmt.steps.prepare.PreparePlugin[PrepareShellData]):
               - sudo dnf install -y 'dnf-command(copr)'
               - sudo dnf copr enable -y psss/tmt
               - sudo dnf install -y tmt
+
+    Scripts can also be fetched from a remote git repository.
+    Specify the ``url`` for the repository and optionally ``ref``
+    to checkout a specific branch, tag or commit.
+    ``TMT_PREPARE_SHELL_URL_REPOSITORY`` will hold the value of the
+    repository path.
+
+    .. code-block:: yaml
+
+        prepare:
+            how: shell
+            url: https://github.com/teemtee/tmt.git
+            ref: main
+            script: cd $TMT_PREPARE_SHELL_URL_REPOSITORY && make docs
     """
 
     _data_class = PrepareShellData
+    _url_clone_lock = threading.Lock()
+    _cloned_repo_path_envvar_name = 'TMT_PREPARE_SHELL_URL_REPOSITORY'
 
     def go(
         self,
@@ -66,21 +104,54 @@ class PrepareShell(tmt.steps.prepare.PreparePlugin[PrepareShellData]):
         guest: 'Guest',
         environment: Optional[tmt.utils.Environment] = None,
         logger: tmt.log.Logger,
-    ) -> list[PhaseResult]:
+    ) -> tmt.steps.PluginOutcome:
         """
         Prepare the guests
         """
 
-        results = super().go(guest=guest, environment=environment, logger=logger)
+        outcome = super().go(guest=guest, environment=environment, logger=logger)
 
         environment = environment or tmt.utils.Environment()
 
+        environment.update(guest.environment)
+
         # Give a short summary
         overview = fmf.utils.listed(self.data.script, 'script')
         logger.info('overview', f'{overview} found', 'green')
 
-        workdir = self.step.plan.worktree
-        assert workdir is not None  # narrow type
+        worktree = self.step.plan.worktree
+        assert worktree is not None  # narrow type
+
+        if self.data.url:
+            assert self.workdir is not None  # narrow type
+
+            repo_path = self.workdir / "repository"
+
+            environment[self._cloned_repo_path_envvar_name] = EnvVarValue(repo_path.resolve())
+
+            if not self.is_dry_run:
+                with self._url_clone_lock:
+                    if not repo_path.exists():
+                        repo_path.parent.mkdir(parents=True, exist_ok=True)
+                        tmt.utils.git.git_clone(
+                            url=self.data.url,
+                            destination=repo_path,
+                            shallow=False,
+                            env=environment,
+                            logger=self._logger,
+                        )
+
+                        if self.data.ref:
+                            self.info('ref', self.data.ref, 'green')
+                            self.run(
+                                Command('git', 'checkout', '-f', self.data.ref), cwd=repo_path
+                            )
+
+                guest.push(
+                    source=repo_path,
+                    destination=repo_path,
+                    options=TransferOptions(protect_args=True, preserve_perms=True, chmod=0o755),
+                )
 
         if not self.is_dry_run:
             topology = tmt.steps.Topology(self.step.plan.provision.ready_guests)
@@ -88,7 +159,7 @@ class PrepareShell(tmt.steps.prepare.PreparePlugin[PrepareShellData]):
 
             environment.update(
                 topology.push(
-                    dirpath=workdir,
+                    dirpath=worktree,
                     guest=guest,
                     logger=logger,
                     filename_base=safe_filename(
@@ -98,7 +169,7 @@ class PrepareShell(tmt.steps.prepare.PreparePlugin[PrepareShellData]):
             )
 
         prepare_wrapper_filename = safe_filename(PREPARE_WRAPPER_FILENAME, self, guest)
-        prepare_wrapper_path = workdir / prepare_wrapper_filename
+        prepare_wrapper_path = worktree / prepare_wrapper_filename
 
         logger.debug('prepare wrapper', prepare_wrapper_path, level=3)
 
@@ -112,13 +183,13 @@ class PrepareShell(tmt.steps.prepare.PreparePlugin[PrepareShellData]):
             guest.push(
                 source=prepare_wrapper_path,
                 destination=prepare_wrapper_path,
-                options=["-s", "-p", "--chmod=755"],
+                options=TransferOptions(protect_args=True, preserve_perms=True, chmod=0o755),
             )
             command: ShellScript
             if guest.become and not guest.facts.is_superuser:
                 command = tmt.utils.ShellScript(f'sudo -E {prepare_wrapper_path}')
             else:
                 command = tmt.utils.ShellScript(f'{prepare_wrapper_path}')
-            guest.execute(command=command, cwd=workdir, env=environment)
+            guest.execute(command=command, cwd=worktree, env=environment)
 
-        return results
+        return outcome
