@@ -1,4 +1,3 @@
-import datetime
 import functools
 from typing import Any, Optional, TypedDict, Union, cast
 
@@ -12,6 +11,7 @@ import tmt.steps
 import tmt.steps.provision
 import tmt.utils
 import tmt.utils.signals
+import tmt.utils.wait
 from tmt.container import container, field
 from tmt.utils import (
     Command,
@@ -22,6 +22,7 @@ from tmt.utils import (
     normalize_string_dict,
     retry_session,
 )
+from tmt.utils.wait import Deadline, Waiting
 
 # List of Artemis API versions supported and understood by this plugin.
 # Since API gains support for new features over time, it is important to
@@ -79,10 +80,10 @@ SUPPORTED_LOG_TYPES = [
 DEFAULT_API_VERSION = SUPPORTED_API_VERSIONS[0]
 
 DEFAULT_API_URL = 'http://127.0.0.1:8001'
-DEFAULT_USER = 'root'
 DEFAULT_ARCH = 'x86_64'
 DEFAULT_PRIORITY_GROUP = 'default-priority'
-DEFAULT_KEYNAME = 'default'
+# Align the value with Artemis project
+DEFAULT_KEYNAME = 'master-key'
 DEFAULT_PROVISION_TIMEOUT = 600
 DEFAULT_PROVISION_TICK = 60
 DEFAULT_API_TIMEOUT = 10
@@ -103,13 +104,6 @@ def _normalize_log_type(key_address: str, raw_value: Any, logger: tmt.log.Logger
 
 @container
 class ArtemisGuestData(tmt.steps.provision.GuestSshData):
-    user: str = field(
-        default=DEFAULT_USER,
-        option=('-u', '--user'),
-        metavar='USERNAME',
-        help='Username to use for all guest operations.',
-    )
-
     # API
     api_url: str = field(
         default=DEFAULT_API_URL,
@@ -605,17 +599,14 @@ class GuestArtemis(tmt.GuestSsh):
                 if state == 'ready':
                     return current
 
-                raise tmt.utils.WaitingIncompleteError
+                raise tmt.utils.wait.WaitingIncompleteError
 
             try:
-                guest_info = tmt.utils.wait(
-                    self,
-                    get_new_state,
-                    datetime.timedelta(seconds=self.provision_timeout),
-                    tick=self.provision_tick,
-                )
+                guest_info = Waiting(
+                    Deadline.from_seconds(self.provision_timeout), tick=self.provision_tick
+                ).wait(get_new_state, self._logger)
 
-            except tmt.utils.WaitingTimedOutError:
+            except tmt.utils.wait.WaitingTimedOutError:
                 # The provisioning chain has been already started, make sure we
                 # remove the guest.
                 self.remove()
@@ -642,6 +633,8 @@ class GuestArtemis(tmt.GuestSsh):
         self.verbose('primary address', self.primary_address, 'green')
         self.verbose('topology address', self.topology_address, 'green')
 
+        self.assert_reachable()
+
     def remove(self) -> None:
         """
         Remove the guest
@@ -670,9 +663,7 @@ class GuestArtemis(tmt.GuestSsh):
         self,
         hard: bool = False,
         command: Optional[Union[Command, ShellScript]] = None,
-        timeout: Optional[int] = None,
-        tick: float = tmt.utils.DEFAULT_WAIT_TICK,
-        tick_increase: float = tmt.utils.DEFAULT_WAIT_TICK_INCREASE,
+        waiting: Optional[Waiting] = None,
     ) -> bool:
         """
         Reboot the guest, and wait for the guest to recover.
@@ -689,6 +680,8 @@ class GuestArtemis(tmt.GuestSsh):
         :returns: ``True`` if the reboot succeeded, ``False`` otherwise.
         """
 
+        waiting = waiting or tmt.steps.provision.default_reboot_waiting()
+
         if hard:
             if self.guestname is None:
                 raise ArtemisProvisionError("Cannot reboot - guest does not exist")
@@ -704,21 +697,14 @@ class GuestArtemis(tmt.GuestSsh):
 
             return self.perform_reboot(
                 trigger_reboot,
-                timeout=timeout,
-                tick=tick,
-                tick_increase=tick_increase,
+                waiting,
                 fetch_boot_time=False,
             )
 
-        actual_command = command or tmt.steps.DEFAULT_REBOOT_COMMAND
-        self.debug(f"Reboot using the command '{actual_command}'.")
-
         return super().reboot(
             hard=False,
-            command=actual_command,
-            timeout=timeout,
-            tick=tick,
-            tick_increase=tick_increase,
+            command=command,
+            waiting=waiting,
         )
 
 
@@ -753,9 +739,9 @@ class ProvisionArtemis(tmt.steps.provision.ProvisionPlugin[ProvisionArtemisData]
 
     .. note::
 
-        When used together with the Testing Farm infrastructure
-        some of the options from the first example below
-        will be filled for you by the Testing Farm service.
+        When used together with the :ref:`testing-farm` infrastructure
+        some of the options from the first example below will be filled
+        for you by the Testing Farm service.
 
     .. note::
 
