@@ -6,8 +6,7 @@ import tmt.steps.provision
 import tmt.utils
 from tmt.container import container, field
 from tmt.utils import Command, ShellScript
-
-DEFAULT_USER = "root"
+from tmt.utils.wait import Waiting
 
 
 @container
@@ -21,15 +20,8 @@ class ConnectGuestData(tmt.steps.provision.GuestSshData):
     guest: Optional[str] = field(
         default=None,
         option=('-g', '--guest'),
-        metavar='GUEST',
-        help='Select remote host to connect to (hostname or ip).',
-    )
-
-    user: str = field(
-        default=DEFAULT_USER,
-        option=('-u', '--user'),
-        metavar='USERNAME',
-        help='Username to use for all guest operations.',
+        metavar='HOSTNAME|IP',
+        help='A preexisting machine to connect to.',
     )
 
     soft_reboot: Optional[ShellScript] = field(
@@ -89,9 +81,7 @@ class GuestConnect(tmt.steps.provision.GuestSsh):
         self,
         hard: bool = False,
         command: Optional[Union[Command, ShellScript]] = None,
-        timeout: Optional[int] = None,
-        tick: float = tmt.utils.DEFAULT_WAIT_TICK,
-        tick_increase: float = tmt.utils.DEFAULT_WAIT_TICK_INCREASE,
+        waiting: Optional[Waiting] = None,
     ) -> bool:
         """
         Reboot the guest, and wait for the guest to recover.
@@ -125,6 +115,8 @@ class GuestConnect(tmt.steps.provision.GuestSsh):
         :returns: ``True`` if the reboot succeeded, ``False`` otherwise.
         """
 
+        waiting = waiting or tmt.steps.provision.default_reboot_waiting()
+
         if hard:
             if self.hard_reboot is None:
                 raise tmt.steps.provision.RebootModeNotSupportedError(guest=self, hard=True)
@@ -135,9 +127,7 @@ class GuestConnect(tmt.steps.provision.GuestSsh):
             # being `None`, missing the explicit check above.
             return self.perform_reboot(
                 lambda: self._run_guest_command(self.hard_reboot.to_shell_command()),  # type: ignore[union-attr]
-                timeout=timeout,
-                tick=tick,
-                tick_increase=tick_increase,
+                waiting,
                 fetch_boot_time=False,
             )
 
@@ -145,9 +135,7 @@ class GuestConnect(tmt.steps.provision.GuestSsh):
             return super().reboot(
                 hard=False,
                 command=command,
-                timeout=timeout,
-                tick=tick,
-                tick_increase=tick_increase,
+                waiting=waiting,
             )
 
         if self.soft_reboot is not None:
@@ -157,12 +145,10 @@ class GuestConnect(tmt.steps.provision.GuestSsh):
             # being `None`, missing the explicit check above.
             return self.perform_reboot(
                 lambda: self._run_guest_command(self.soft_reboot.to_shell_command()),  # type: ignore[union-attr]
-                timeout=timeout,
-                tick=tick,
-                tick_increase=tick_increase,
+                waiting,
             )
 
-        return super().reboot(hard=False, timeout=timeout, tick=tick, tick_increase=tick_increase)
+        return super().reboot(hard=False, waiting=waiting)
 
     def start(self) -> None:
         """
@@ -174,14 +160,26 @@ class GuestConnect(tmt.steps.provision.GuestSsh):
         self.verbose('primary address', self.primary_address, 'green')
         self.verbose('topology address', self.topology_address, 'green')
 
+        self.assert_reachable()
+
 
 @tmt.steps.provides_method('connect')
 class ProvisionConnect(tmt.steps.provision.ProvisionPlugin[ProvisionConnectData]):
+    #
+    # This plugin docstring has been reviewed and updated to follow
+    # our documentation best practices. When changing it, please make
+    # sure new changes are following them as well.
+    #
+    # https://tmt.readthedocs.io/en/stable/contribute.html#docs
+    #
     """
     Connect to a provisioned guest using SSH.
 
-    Do not provision a new system. Instead, use provided
-    authentication data to connect to a running machine.
+    Do not provision any system, tests will be executed directly on the
+    machine that has been already provisioned. Use provided
+    authentication information to connect to it over SSH.
+
+
 
     Private key authentication (using ``sudo`` to run scripts):
 
@@ -213,11 +211,13 @@ class ProvisionConnect(tmt.steps.provision.ProvisionPlugin[ProvisionConnectData]
             how: connect
             guest: host.example.org
 
-    To trigger a hard reboot of a guest, ``hard-reboot`` must be set to
+
+
+    To support hard reboot of a guest, ``hard-reboot`` must be set to
     an executable command or script. Without this key set, hard reboot
     will remain unsupported and result in an error. In comparison,
-    ``soft-reboot`` is optional, it will be preferred over the default
-    soft reboot command, ``reboot``:
+    ``soft-reboot`` is optional, but if set, the given command will be
+    preferred over the default soft reboot command, ``reboot``:
 
     .. code-block:: yaml
 
@@ -226,6 +226,12 @@ class ProvisionConnect(tmt.steps.provision.ProvisionPlugin[ProvisionConnectData]
           hard-reboot: virsh reboot my-example-vm
           soft-reboot: ssh root@my-example-vm 'shutdown -r now'
 
+    .. code-block:: shell
+
+        provision --how connect \\
+                  --hard-reboot="virsh reboot my-example-vm" \\
+                  --soft-reboot="ssh root@my-example-vm 'shutdown -r now'"
+
     .. warning::
 
         Both ``hard-reboot`` and ``soft-reboot`` commands are executed
@@ -274,4 +280,5 @@ class ProvisionConnect(tmt.steps.provision.ProvisionPlugin[ProvisionConnectData]
         self._guest = GuestConnect(
             logger=self._logger, data=data, name=self.name, parent=self.step
         )
+        self._guest.start()
         self._guest.setup()
