@@ -1,3 +1,4 @@
+from collections.abc import Sequence
 from typing import Any, Optional, Union
 
 import tmt
@@ -5,9 +6,13 @@ import tmt.base
 import tmt.log
 import tmt.steps
 import tmt.steps.provision
+import tmt.steps.scripts
 import tmt.utils
 from tmt.container import container
-from tmt.utils import Command, OnProcessStartCallback, Path, ShellScript
+from tmt.steps.provision import TransferOptions
+from tmt.utils import Command, OnProcessEndCallback, OnProcessStartCallback, Path, ShellScript
+from tmt.utils.hints import get_hint
+from tmt.utils.wait import Waiting
 
 
 @container
@@ -80,10 +85,11 @@ class GuestLocal(tmt.Guest):
             )
             # fmt: on
         except tmt.utils.RunError as exc:
-            if exc.stderr and 'ansible-playbook: command not found' in exc.stderr:
-                from tmt.utils.hints import print_hint
+            hint = get_hint('ansible-not-available', ignore_missing=False)
+
+            if hint.search_cli_patterns(exc.stderr, exc.stdout, exc.message):
+                hint.print(self._logger)
 
-                print_hint(id_='ansible-not-available', logger=self._logger)
             raise exc
 
     def execute(
@@ -98,6 +104,7 @@ class GuestLocal(tmt.Guest):
         log: Optional[tmt.log.LoggingFunction] = None,
         interactive: bool = False,
         on_process_start: Optional[OnProcessStartCallback] = None,
+        on_process_end: Optional[OnProcessEndCallback] = None,
         **kwargs: Any,
     ) -> tmt.utils.CommandOutput:
         """
@@ -125,9 +132,13 @@ class GuestLocal(tmt.Guest):
             cwd=cwd,
             interactive=interactive,
             on_process_start=on_process_start,
+            on_process_end=on_process_end,
             **kwargs,
         )
 
+    def install_scripts(self, scripts: Sequence[tmt.steps.scripts.Script]) -> None:
+        self.debug("No installation of tmt scripts is needed on localhost.")
+
     def start(self) -> None:
         """
         Start the guest
@@ -138,6 +149,8 @@ class GuestLocal(tmt.Guest):
         self.verbose('primary address', self.primary_address, 'green')
         self.verbose('topology address', self.topology_address, 'green')
 
+        self.assert_reachable()
+
     def stop(self) -> None:
         """
         Stop the guest
@@ -149,9 +162,7 @@ class GuestLocal(tmt.Guest):
         self,
         hard: bool = False,
         command: Optional[Union[Command, ShellScript]] = None,
-        timeout: Optional[int] = None,
-        tick: float = tmt.utils.DEFAULT_WAIT_TICK,
-        tick_increase: float = tmt.utils.DEFAULT_WAIT_TICK_INCREASE,
+        waiting: Optional[Waiting] = None,
     ) -> bool:
         # No localhost reboot allowed!
         self.debug(f"Doing nothing to reboot guest '{self.primary_address}'.")
@@ -162,7 +173,7 @@ class GuestLocal(tmt.Guest):
         self,
         source: Optional[Path] = None,
         destination: Optional[Path] = None,
-        options: Optional[list[str]] = None,
+        options: Optional[TransferOptions] = None,
         superuser: bool = False,
     ) -> None:
         """
@@ -173,8 +184,7 @@ class GuestLocal(tmt.Guest):
         self,
         source: Optional[Path] = None,
         destination: Optional[Path] = None,
-        options: Optional[list[str]] = None,
-        extend_options: Optional[list[str]] = None,
+        options: Optional[TransferOptions] = None,
     ) -> None:
         """
         Nothing to be done to pull workdir
@@ -183,34 +193,49 @@ class GuestLocal(tmt.Guest):
 
 @tmt.steps.provides_method('local')
 class ProvisionLocal(tmt.steps.provision.ProvisionPlugin[ProvisionLocalData]):
+    #
+    # This plugin docstring has been reviewed and updated to follow
+    # our documentation best practices. When changing it, please make
+    # sure new changes are following them as well.
+    #
+    # https://tmt.readthedocs.io/en/stable/contribute.html#docs
+    #
     """
-    Use local host for test execution.
+    Use the localhost for the test execution.
 
-    Do not provision any system. Tests will be executed
-    directly on the localhost. Note that for some actions like
-    installing additional packages you need root permission or
-    enabled sudo.
+    Do not provision any system, tests will be executed directly on the
+    localhost.
 
     .. warning::
 
-        In general, it is not recommended to run tests on your local machine
-        as there might be security risks. Run only those tests which you
-        know are safe so that you don't destroy your laptop ;-)
+        In general, it is not recommended to run tests on your local
+        machine as there might be security risks. Run only those tests
+        which you know are safe so that you don't destroy your
+        workstation ;-)
 
         From tmt version 1.38, the ``--feeling-safe`` option or
         the ``TMT_FEELING_SAFE=1`` environment variable is
-        required in order to use the local provision plugin.
+        required in order to use the ``local`` provision plugin.
 
-    Example config:
+    Using the plugin:
 
     .. code-block:: yaml
 
         provision:
             how: local
 
-    Note that ``tmt run`` is expected to be executed under a regular user.
-    If there are admin rights required (for example in the prepare step)
-    you might be asked for a ``sudo`` password.
+    .. code-block:: shell
+
+        provision --how local
+
+    .. note::
+
+        ``tmt run`` is expected to be executed under a non-privileged
+        user account. For some actions on the localhost, e.g.
+        installation of test requirements, ``local`` will require
+        elevated privileges, either by running under ``root``
+        account, or by using ``sudo`` to run the sensitive commands. You
+        may be asked for a password in such cases.
 
     .. note::
 
@@ -244,4 +269,5 @@ class ProvisionLocal(tmt.steps.provision.ProvisionPlugin[ProvisionLocalData]):
             self.warn("The 'local' provision plugin does not support hardware requirements.")
 
         self._guest = GuestLocal(logger=self._logger, data=data, name=self.name, parent=self.step)
+        self._guest.start()
         self._guest.setup()
