@@ -1,14 +1,15 @@
 import asyncio
 import dataclasses
 import datetime
+import functools
 import importlib.metadata
+import json
 import logging
-import os
 import re
-from collections.abc import Mapping
+import threading
 from contextlib import suppress
 from functools import wraps
-from typing import Any, Callable, Optional, TypedDict, Union, cast
+from typing import Any, Callable, Optional, TypedDict, TypeVar, Union, cast
 
 import packaging.version
 
@@ -21,6 +22,7 @@ import tmt.steps
 import tmt.steps.provision
 import tmt.utils
 import tmt.utils.signals
+import tmt.utils.wait
 from tmt.config.models.hardware import MrackTranslation
 from tmt.container import container, field, simple_field
 from tmt.utils import (
@@ -31,6 +33,7 @@ from tmt.utils import (
     UpdatableMessage,
 )
 from tmt.utils.templates import render_template
+from tmt.utils.wait import Deadline, Waiting
 
 MRACK_VERSION: Optional[str] = None
 
@@ -44,8 +47,8 @@ BeakerTransformer: Any
 TmtBeakerTransformer: Any
 
 _MRACK_IMPORTED: bool = False
+_MRACK_CONTEXT_INITIALIZED: bool = False
 
-DEFAULT_USER = 'root'
 DEFAULT_ARCH = 'x86_64'
 DEFAULT_IMAGE = 'fedora'
 DEFAULT_PROVISION_TIMEOUT = 3600  # 1 hour timeout at least
@@ -252,14 +255,64 @@ class MrackHWNotGroup(MrackHWGroup):
     name: str = 'not'
 
 
-def _transform_unsupported(constraint: tmt.hardware.Constraint[Any]) -> dict[str, Any]:
+#
+# Constraint transformations
+#
+
+#: A type var representing actual constraint type in transformers and
+#: their type annotations.
+ConstraintT = TypeVar('ConstraintT', bound=tmt.hardware.Constraint)
+
+
+#: A type of constraint transformers.
+ConstraintTransformer = Callable[[ConstraintT, tmt.log.Logger], BeakerizedConstraint]
+
+
+#: A mapping between constraint names and corresponding transformers.
+_CONSTRAINT_TRANSFORMERS: dict[str, ConstraintTransformer[Any]] = {}
+
+
+def transforms(fn: ConstraintTransformer[ConstraintT]) -> ConstraintTransformer[ConstraintT]:
+    """
+    A decorator marking a function as a constraint transformer.
+
+    Function name is expected to provide the constraint name it
+    transforms: decorator strips away the initial ``_transform_``
+    prefix, and replaces the first underscore, ``_`` with a dot, ``.``:
+
+    .. code-block::
+
+        _transform_beaker_pool => beaker.pool
+        _transform_disk_physical_sector_size => disk.physical_sector_size
+
+    :param fn: function to decorate.
+    """
+
+    global _CONSTRAINT_TRANSFORMERS
+
+    _CONSTRAINT_TRANSFORMERS[fn.__name__.replace('_transform_', '').replace('_', '.', 1)] = fn
+
+    @functools.wraps(fn)
+    def _transforms(constraint: ConstraintT, logger: tmt.log.Logger) -> BeakerizedConstraint:
+        return fn(constraint, logger)
+
+    return _transforms
+
+
+def _transform_unsupported(constraint: tmt.hardware.Constraint) -> dict[str, Any]:
     # We have to return something, return an {}- no harm done, composable with other elements.
 
     return {}
 
 
+def _get_registry_from_url(bootc_image_url: str) -> str:
+    """Extract registry from image URL"""
+    # Handle quay.io/repo/image:tag -> quay.io
+    return bootc_image_url.split('/')[0] if '/' in bootc_image_url else bootc_image_url
+
+
 def _translate_constraint_by_config(
-    constraint: tmt.hardware.Constraint[Any],
+    constraint: tmt.hardware.Constraint,
     logger: tmt.log.Logger,
 ) -> dict[str, Any]:
     """
@@ -281,7 +334,7 @@ def _translate_constraint_by_config(
         return _transform_unsupported(constraint)
 
     beaker_operator, actual_value, negate = operator_to_beaker_op(
-        constraint.operator, constraint.value
+        constraint.operator, str(constraint.value)
     )
 
     return tmt.utils.yaml_to_dict(
@@ -296,7 +349,7 @@ def _translate_constraint_by_config(
 
 
 def _translate_constraint_by_transformer(
-    constraint: tmt.hardware.Constraint[Any],
+    constraint: tmt.hardware.Constraint,
     logger: tmt.log.Logger,
 ) -> BeakerizedConstraint:
     """
@@ -315,16 +368,16 @@ def _translate_constraint_by_transformer(
     return transformer(constraint, logger)
 
 
+@transforms
 def _transform_beaker_pool(
     constraint: tmt.hardware.TextConstraint, logger: tmt.log.Logger
 ) -> MrackBaseHWElement:
-    beaker_operator, actual_value, negate = operator_to_beaker_op(
-        constraint.operator, constraint.value
-    )
+    beaker_operator, actual_value, _ = operator_to_beaker_op(constraint.operator, constraint.value)
 
     return MrackHWBinOp('pool', beaker_operator, actual_value)
 
 
+@transforms
 def _transform_cpu_family(
     constraint: tmt.hardware.IntegerConstraint, logger: tmt.log.Logger
 ) -> MrackBaseHWElement:
@@ -335,6 +388,7 @@ def _transform_cpu_family(
     return MrackHWGroup('cpu', children=[MrackHWBinOp('family', beaker_operator, actual_value)])
 
 
+@transforms
 def _transform_cpu_flag(
     constraint: tmt.hardware.TextConstraint, logger: tmt.log.Logger
 ) -> MrackBaseHWElement:
@@ -348,6 +402,7 @@ def _transform_cpu_flag(
     return MrackHWGroup('cpu', children=[MrackHWBinOp('flag', beaker_operator, actual_value)])
 
 
+@transforms
 def _transform_cpu_model(
     constraint: tmt.hardware.IntegerConstraint, logger: tmt.log.Logger
 ) -> MrackBaseHWElement:
@@ -358,6 +413,7 @@ def _transform_cpu_model(
     return MrackHWGroup('cpu', children=[MrackHWBinOp('model', beaker_operator, actual_value)])
 
 
+@transforms
 def _transform_cpu_processors(
     constraint: tmt.hardware.IntegerConstraint, logger: tmt.log.Logger
 ) -> MrackBaseHWElement:
@@ -370,6 +426,7 @@ def _transform_cpu_processors(
     )
 
 
+@transforms
 def _transform_cpu_cores(
     constraint: tmt.hardware.IntegerConstraint, logger: tmt.log.Logger
 ) -> MrackBaseHWElement:
@@ -380,6 +437,7 @@ def _transform_cpu_cores(
     return MrackHWGroup('cpu', children=[MrackHWBinOp('cores', beaker_operator, actual_value)])
 
 
+@transforms
 def _transform_cpu_model_name(
     constraint: tmt.hardware.TextConstraint, logger: tmt.log.Logger
 ) -> MrackBaseHWElement:
@@ -401,6 +459,7 @@ def _transform_cpu_model_name(
     )
 
 
+@transforms
 def _transform_cpu_frequency(
     constraint: tmt.hardware.NumberConstraint, logger: tmt.log.Logger
 ) -> MrackBaseHWElement:
@@ -411,6 +470,7 @@ def _transform_cpu_frequency(
     return MrackHWGroup('cpu', children=[MrackHWBinOp('speed', beaker_operator, actual_value)])
 
 
+@transforms
 def _transform_cpu_stepping(
     constraint: tmt.hardware.IntegerConstraint, logger: tmt.log.Logger
 ) -> MrackBaseHWElement:
@@ -421,6 +481,7 @@ def _transform_cpu_stepping(
     return MrackHWGroup('cpu', children=[MrackHWBinOp('stepping', beaker_operator, actual_value)])
 
 
+@transforms
 def _transform_cpu_vendor_name(
     constraint: tmt.hardware.TextConstraint, logger: tmt.log.Logger
 ) -> MrackBaseHWElement:
@@ -440,6 +501,7 @@ def _transform_cpu_vendor_name(
     return MrackHWGroup('cpu', children=[MrackHWBinOp('vendor', beaker_operator, actual_value)])
 
 
+@transforms
 def _transform_cpu_hyper_threading(
     constraint: tmt.hardware.FlagConstraint, logger: tmt.log.Logger
 ) -> MrackBaseHWElement:
@@ -450,21 +512,42 @@ def _transform_cpu_hyper_threading(
     return MrackHWGroup('cpu', children=[MrackHWBinOp('hyper', beaker_operator, actual_value)])
 
 
-def _transform_disk_driver(
-    constraint: tmt.hardware.TextConstraint, logger: tmt.log.Logger
+def _transform_driver_helper(
+    beaker_key: str,
+    constraint: tmt.hardware.TextConstraint,
+    logger: tmt.log.Logger,
 ) -> MrackBaseHWElement:
+    """
+    A helper to transform a hardware driver constraint into its Beaker key-value filter.
+    """
+
     beaker_operator, actual_value, negate = operator_to_beaker_op(
         constraint.operator, constraint.value
     )
 
     if negate:
         return MrackHWNotGroup(
-            children=[MrackHWKeyValue('BOOTDISK', beaker_operator, actual_value)]
+            children=[MrackHWKeyValue(beaker_key, beaker_operator, actual_value)]
         )
 
-    return MrackHWKeyValue('BOOTDISK', beaker_operator, actual_value)
+    return MrackHWKeyValue(beaker_key, beaker_operator, actual_value)
 
 
+@transforms
+def _transform_disk_driver(
+    constraint: tmt.hardware.TextConstraint, logger: tmt.log.Logger
+) -> MrackBaseHWElement:
+    return _transform_driver_helper('BOOTDISK', constraint, logger)
+
+
+@transforms
+def _transform_device_driver(
+    constraint: tmt.hardware.TextConstraint, logger: tmt.log.Logger
+) -> MrackBaseHWElement:
+    return _transform_driver_helper("MODULE", constraint, logger)
+
+
+@transforms
 def _transform_disk_size(
     constraint: tmt.hardware.SizeConstraint, logger: tmt.log.Logger
 ) -> MrackBaseHWElement:
@@ -475,6 +558,7 @@ def _transform_disk_size(
     return MrackHWGroup('disk', children=[MrackHWBinOp('size', beaker_operator, actual_value)])
 
 
+@transforms
 def _transform_disk_model_name(
     constraint: tmt.hardware.TextConstraint, logger: tmt.log.Logger
 ) -> MrackBaseHWElement:
@@ -494,6 +578,7 @@ def _transform_disk_model_name(
     return MrackHWGroup('disk', children=[MrackHWBinOp('model', beaker_operator, actual_value)])
 
 
+@transforms
 def _transform_disk_physical_sector_size(
     constraint: tmt.hardware.SizeConstraint, logger: tmt.log.Logger
 ) -> MrackBaseHWElement:
@@ -506,6 +591,7 @@ def _transform_disk_physical_sector_size(
     )
 
 
+@transforms
 def _transform_disk_logical_sector_size(
     constraint: tmt.hardware.SizeConstraint, logger: tmt.log.Logger
 ) -> MrackBaseHWElement:
@@ -518,6 +604,7 @@ def _transform_disk_logical_sector_size(
     )
 
 
+@transforms
 def _transform_hostname(
     constraint: tmt.hardware.TextConstraint, logger: tmt.log.Logger
 ) -> MrackBaseHWElement:
@@ -531,6 +618,7 @@ def _transform_hostname(
     return MrackHWBinOp('hostname', beaker_operator, actual_value)
 
 
+@transforms
 def _transform_memory(
     constraint: tmt.hardware.SizeConstraint, logger: tmt.log.Logger
 ) -> MrackBaseHWElement:
@@ -541,16 +629,16 @@ def _transform_memory(
     return MrackHWGroup('system', children=[MrackHWBinOp('memory', beaker_operator, actual_value)])
 
 
+@transforms
 def _transform_tpm_version(
     constraint: tmt.hardware.TextConstraint, logger: tmt.log.Logger
 ) -> MrackBaseHWElement:
-    beaker_operator, actual_value, negate = operator_to_beaker_op(
-        constraint.operator, constraint.value
-    )
+    beaker_operator, actual_value, _ = operator_to_beaker_op(constraint.operator, constraint.value)
 
     return MrackHWKeyValue('TPM', beaker_operator, actual_value)
 
 
+@transforms
 def _transform_virtualization_is_virtualized(
     constraint: tmt.hardware.FlagConstraint, logger: tmt.log.Logger
 ) -> BeakerizedConstraint:
@@ -569,6 +657,7 @@ def _transform_virtualization_is_virtualized(
     return _transform_unsupported(constraint)
 
 
+@transforms
 def _transform_virtualization_hypervisor(
     constraint: tmt.hardware.TextConstraint, logger: tmt.log.Logger
 ) -> MrackBaseHWElement:
@@ -590,6 +679,7 @@ def _transform_virtualization_hypervisor(
     )
 
 
+@transforms
 def _transform_zcrypt_adapter(
     constraint: tmt.hardware.TextConstraint, logger: tmt.log.Logger
 ) -> MrackBaseHWElement:
@@ -612,6 +702,7 @@ def _transform_zcrypt_adapter(
     )
 
 
+@transforms
 def _transform_zcrypt_mode(
     constraint: tmt.hardware.TextConstraint, logger: tmt.log.Logger
 ) -> MrackBaseHWElement:
@@ -634,6 +725,7 @@ def _transform_zcrypt_mode(
     )
 
 
+@transforms
 def _transform_iommu_is_supported(
     constraint: tmt.hardware.FlagConstraint, logger: tmt.log.Logger
 ) -> BeakerizedConstraint:
@@ -648,6 +740,7 @@ def _transform_iommu_is_supported(
     return _transform_unsupported(constraint)
 
 
+@transforms
 def _transform_location_lab_controller(
     constraint: tmt.hardware.TextConstraint, logger: tmt.log.Logger
 ) -> MrackBaseHWElement:
@@ -667,6 +760,7 @@ def _transform_location_lab_controller(
     return MrackHWBinOp('labcontroller', beaker_operator, actual_value)
 
 
+@transforms
 def _transform_system_numa_nodes(
     constraint: tmt.hardware.IntegerConstraint, logger: tmt.log.Logger
 ) -> MrackBaseHWElement:
@@ -679,6 +773,7 @@ def _transform_system_numa_nodes(
     )
 
 
+@transforms
 def _transform_system_model_name(
     constraint: tmt.hardware.TextConstraint, logger: tmt.log.Logger
 ) -> MrackBaseHWElement:
@@ -698,6 +793,7 @@ def _transform_system_model_name(
     return MrackHWGroup('system', children=[MrackHWBinOp('model', beaker_operator, actual_value)])
 
 
+@transforms
 def _transform_system_vendor_name(
     constraint: tmt.hardware.TextConstraint, logger: tmt.log.Logger
 ) -> MrackBaseHWElement:
@@ -717,42 +813,6 @@ def _transform_system_vendor_name(
     return MrackHWGroup('system', children=[MrackHWBinOp('vendor', beaker_operator, actual_value)])
 
 
-ConstraintTransformer = Callable[
-    [tmt.hardware.Constraint[Any], tmt.log.Logger], MrackBaseHWElement
-]
-
-_CONSTRAINT_TRANSFORMERS: Mapping[str, ConstraintTransformer] = {
-    'beaker.pool': _transform_beaker_pool,  # type: ignore[dict-item]
-    'cpu.cores': _transform_cpu_cores,  # type: ignore[dict-item]
-    'cpu.family': _transform_cpu_family,  # type: ignore[dict-item]
-    'cpu.flag': _transform_cpu_flag,  # type: ignore[dict-item]
-    'cpu.frequency': _transform_cpu_frequency,  # type: ignore[dict-item]
-    'cpu.hyper_threading': _transform_cpu_hyper_threading,  # type: ignore[dict-item]
-    'cpu.model': _transform_cpu_model,  # type: ignore[dict-item]
-    'cpu.model_name': _transform_cpu_model_name,  # type: ignore[dict-item]
-    'cpu.processors': _transform_cpu_processors,  # type: ignore[dict-item]
-    'cpu.stepping': _transform_cpu_stepping,  # type: ignore[dict-item]
-    'cpu.vendor_name': _transform_cpu_vendor_name,  # type: ignore[dict-item]
-    'disk.driver': _transform_disk_driver,  # type: ignore[dict-item]
-    'disk.model_name': _transform_disk_model_name,  # type: ignore[dict-item]
-    'disk.size': _transform_disk_size,  # type: ignore[dict-item]
-    'disk.physical_sector_size': _transform_disk_physical_sector_size,  # type: ignore[dict-item]
-    'disk.logical_sector_size': _transform_disk_logical_sector_size,  # type: ignore[dict-item]
-    'hostname': _transform_hostname,  # type: ignore[dict-item]
-    'location.lab_controller': _transform_location_lab_controller,  # type: ignore[dict-item]
-    'memory': _transform_memory,  # type: ignore[dict-item]
-    'tpm.version': _transform_tpm_version,  # type: ignore[dict-item]
-    'virtualization.is_virtualized': _transform_virtualization_is_virtualized,  # type: ignore[dict-item]
-    'virtualization.hypervisor': _transform_virtualization_hypervisor,  # type: ignore[dict-item]
-    'zcrypt.adapter': _transform_zcrypt_adapter,  # type: ignore[dict-item]
-    'zcrypt.mode': _transform_zcrypt_mode,  # type: ignore[dict-item]
-    'system.numa_nodes': _transform_system_numa_nodes,  # type: ignore[dict-item]
-    'system.model_name': _transform_system_model_name,  # type: ignore[dict-item]
-    'system.vendor_name': _transform_system_vendor_name,  # type: ignore[dict-item]
-    'iommu.is_supported': _transform_iommu_is_supported,  # type: ignore[dict-item]
-}
-
-
 def constraint_to_beaker_filter(
     constraint: tmt.hardware.BaseConstraint, logger: tmt.log.Logger
 ) -> BeakerizedConstraint:
@@ -783,55 +843,64 @@ def constraint_to_beaker_filter(
     ) or _translate_constraint_by_transformer(constraint, logger)
 
     if not transformed:
-        # Unsupported constraint has been already logged via report_support().
-        # Make sure user is aware it would have no effect.
+        # Make sure user is aware constraint would have no effect.
         logger.warning(f"Hardware requirement '{constraint.printable_name}' will have no effect.")
     return transformed
 
 
-def import_and_load_mrack_deps(workdir: Any, name: str, logger: tmt.log.Logger) -> None:
+# Thread-safe lock for mrack imports and initialization
+_MRACK_IMPORT_LOCK = threading.Lock()
+_MRACK_CONTEXT_LOCK = threading.Lock()
+_MRACK_CONTEXT_CONFIG_PATH: Optional[str] = None
+
+
+def import_and_load_mrack_deps(mrack_log: str, logger: tmt.log.Logger) -> None:
     """
-    Import mrack module only when needed
+    Import mrack module only when needed (thread-safe)
     """
 
     global _MRACK_IMPORTED
 
-    if _MRACK_IMPORTED:
-        return
+    # Use lock to ensure thread-safe initialization
+    with _MRACK_IMPORT_LOCK:
+        if _MRACK_IMPORTED:
+            return
 
-    global MRACK_VERSION
-    global mrack
-    global providers
-    global ProvisioningError
-    global NotAuthenticatedError
-    global BEAKER
-    global BeakerProvider
-    global BeakerTransformer
-    global TmtBeakerTransformer
+        global MRACK_VERSION
+        global mrack
+        global providers
+        global ProvisioningError
+        global NotAuthenticatedError
+        global BEAKER
+        global BeakerProvider
+        global BeakerTransformer
+        global TmtBeakerTransformer
 
-    try:
-        import mrack
-        from mrack.errors import NotAuthenticatedError, ProvisioningError
-        from mrack.providers import providers
-        from mrack.providers.beaker import PROVISIONER_KEY as BEAKER
-        from mrack.providers.beaker import BeakerProvider
-        from mrack.transformers.beaker import BeakerTransformer
+        try:
+            import mrack
+            from mrack.errors import NotAuthenticatedError, ProvisioningError
+            from mrack.providers import providers
+            from mrack.providers.beaker import PROVISIONER_KEY as BEAKER
+            from mrack.providers.beaker import BeakerProvider
+            from mrack.transformers.beaker import BeakerTransformer
 
-        MRACK_VERSION = importlib.metadata.version('mrack')
+            MRACK_VERSION = importlib.metadata.version('mrack')
 
-        # hack: remove mrack stdout and move the logfile to /tmp
-        mrack.logger.removeHandler(mrack.console_handler)
-        mrack.logger.removeHandler(mrack.file_handler)
+            # hack: remove mrack stdout and move the logfile to /tmp
+            if hasattr(mrack, 'console_handler'):
+                mrack.logger.removeHandler(mrack.console_handler)
+            if hasattr(mrack, 'file_handler'):
+                mrack.logger.removeHandler(mrack.file_handler)
 
-        with suppress(OSError):
-            os.remove("mrack.log")
+            with suppress(OSError):
+                if Path("mrack.log").exists():
+                    Path("mrack.log").unlink()
+            mrack.logger.addHandler(logging.FileHandler(mrack_log))
 
-        logging.FileHandler(str(f"{workdir}/{name}-mrack.log"))
+            providers.register(BEAKER, BeakerProvider)
 
-        providers.register(BEAKER, BeakerProvider)
-
-    except ImportError:
-        raise ProvisionError("Install 'tmt+provision-beaker' to provision using this method.")
+        except ImportError:
+            raise ProvisionError("Install 'tmt+provision-beaker' to provision using this method.")
 
     # ignore the misc because mrack sources are not typed and result into
     # error: Class cannot subclass "BeakerTransformer" (has type "Any")
@@ -875,25 +944,46 @@ def import_and_load_mrack_deps(workdir: Any, name: str, logger: tmt.log.Logger)
             if host.beaker_job_owner:
                 req['job_owner'] = host.beaker_job_owner
 
-            if host.kickstart:
-                if 'kernel-options' in host.kickstart:
-                    req['kernel_options'] = host.kickstart['kernel-options']
+            if host.kickstart and not mrack_constructs_ks_pre():
+                ks_components: list[str] = [
+                    host.kickstart[ks_section]
+                    for ks_section in ('pre-install', 'script', 'post-install')
+                    if ks_section in host.kickstart
+                ]
 
-                if 'kernel-options-post' in host.kickstart:
-                    req['kernel_options_post'] = host.kickstart['kernel-options-post']
-
-                if not mrack_constructs_ks_pre():
-                    ks_components: list[str] = []
-
-                    for ks_section in ('pre-install', 'script', 'post-install'):
-                        if ks_section in host.kickstart:
-                            ks_components.append(host.kickstart[ks_section])
-
-                    if ks_components:
-                        req['ks_append'] = ['\n'.join(ks_components)]
+                if ks_components:
+                    req['ks_append'] = ['\n'.join(ks_components)]
 
             # Whiteboard must be added *after* request preparation, to overwrite the default one.
             req['whiteboard'] = host.whiteboard
+            if host.bootc:
+                req['ks_append'] = req['ks_append'] if req.get('ks_append') else []
+                ks_meta = req['ks_meta'] if req.get('ks_meta') else ''
+                req['ks_meta'] = (
+                    'no_default_harness_repo disable_debug_repos disable_onerror ' + ks_meta
+                )
+                ks_list = [
+                    f"""
+ostreecontainer --url {host.bootc_image_url}
+%pre
+#!/bin/sh
+cat > /etc/ostree/auth.json <<EOF
+{json.dumps(host.bootc_credentials)}
+EOF
+%end
+"""
+                ]
+                req['ks_append'].extend(ks_list)
+                req['tasks'] = [
+                    {
+                        "name": "/distribution/check-system",
+                        "role": "None",
+                        "fetch_url": host.bootc_check_system_url,
+                        "params": [
+                            f'BOOTC_IMAGE_URL={host.bootc_image_url}',
+                        ],
+                    },
+                ]
 
             logger.debug('mrack request', req, level=4)
 
@@ -904,6 +994,25 @@ def import_and_load_mrack_deps(workdir: Any, name: str, logger: tmt.log.Logger)
     _MRACK_IMPORTED = True
 
 
+def init_mrack_global_context(config_path: str) -> None:
+    """
+    Initialize mrack global context in a thread-safe manner
+    """
+    global _MRACK_CONTEXT_INITIALIZED, _MRACK_CONTEXT_CONFIG_PATH
+
+    with _MRACK_CONTEXT_LOCK:
+        # If already initialized with the same config, return
+        if _MRACK_CONTEXT_INITIALIZED and config_path == _MRACK_CONTEXT_CONFIG_PATH:
+            return
+
+        # If initialized with different config, reinitialize
+        global_context = mrack.context.global_context
+        global_context.init(config_path)
+
+        _MRACK_CONTEXT_INITIALIZED = True
+        _MRACK_CONTEXT_CONFIG_PATH = config_path
+
+
 def async_run(func: Any) -> Any:
     """
     Decorate click actions to run as async
@@ -918,14 +1027,6 @@ def async_run(func: Any) -> Any:
 
 @container
 class BeakerGuestData(tmt.steps.provision.GuestSshData):
-    # Override parent class with our defaults
-    user: str = field(
-        default=DEFAULT_USER,
-        option=('-u', '--user'),
-        metavar='USERNAME',
-        help='Username to use for all guest operations.',
-    )
-
     # Guest request properties
     whiteboard: Optional[str] = field(
         default=None,
@@ -963,6 +1064,7 @@ class BeakerGuestData(tmt.steps.provision.GuestSshData):
              """,
         normalize=tmt.utils.normalize_int,
     )
+
     provision_tick: int = field(
         default=DEFAULT_PROVISION_TICK,
         option='--provision-tick',
@@ -973,6 +1075,7 @@ class BeakerGuestData(tmt.steps.provision.GuestSshData):
              """,
         normalize=tmt.utils.normalize_int,
     )
+
     api_session_refresh_tick: int = field(
         default=DEFAULT_API_SESSION_REFRESH,
         option='--api-session-refresh-tick',
@@ -1022,6 +1125,42 @@ class BeakerGuestData(tmt.steps.provision.GuestSshData):
              """,
     )
 
+    bootc_check_system_url: Optional[str] = field(
+        default="https://gitlab.com/fedora/bootc/tests/bootc-beaker-test/-/archive/1.8/bootc-beaker-test-1.8.tar.gz#check-system",
+        option='--bootc-check-system-url',
+        metavar='URL',
+        help="""
+             URL to ``check-system`` task, which is needed for bootc on Beaker installation.
+             """,
+    )
+
+    bootc_image_url: Optional[str] = field(
+        default=None,
+        option='--bootc-image-url',
+        metavar='URL',
+        help="""
+             Select bootc image to be used to perform the installation.
+             """,
+    )
+
+    bootc_registry_secret: Optional[str] = field(
+        default=None,
+        option='--bootc-registry-secret',
+        metavar='SECRET',
+        help="""
+             Specify bootc secret, which will be used for fetching from registry.
+             """,
+    )
+
+    bootc: bool = field(
+        default=False,
+        is_flag=True,
+        option='--bootc',
+        help="""
+             If set, bootc on Beaker installation will be performed.
+             """,
+    )
+
 
 @container
 class ProvisionBeakerData(BeakerGuestData, tmt.steps.provision.ProvisionStepData):
@@ -1061,6 +1200,10 @@ class CreateJobParameters:
     beaker_job_owner: Optional[str]
     public_key: list[str]
     group: Optional[str]
+    bootc_credentials: Optional[dict[str, Any]]
+    bootc_image_url: Optional[str]
+    bootc: bool
+    bootc_check_system_url: Optional[str]
 
     def to_mrack(self) -> dict[str, Any]:
         data = dataclasses.asdict(self)
@@ -1072,6 +1215,8 @@ class CreateJobParameters:
 
             if 'metadata' in kickstart:
                 data['beaker']['ks_meta'] = kickstart.pop('metadata')
+            data['beaker']['kernel_options'] = self.kickstart.get('kernel_options')
+            data['beaker']['kernel_options_post'] = self.kickstart.get('kernel_options_post')
 
             # Mrack does not handle metadata-only kickstart nicely, ends
             # up with just an empty string. Don't tempt it, don't let it
@@ -1119,13 +1264,14 @@ class BeakerAPI:
             raise ProvisionError("Configuration file 'mrack.conf' not found.")
 
         try:
-            global_context.init(str(mrack_config))
+            init_mrack_global_context(str(mrack_config))
         except mrack.errors.ConfigError as mrack_conf_err:
             raise ProvisionError(mrack_conf_err)
 
         self._mrack_transformer = TmtBeakerTransformer()
         try:
             await self._mrack_transformer.init(global_context.PROV_CONFIG, {})
+
         except NotAuthenticatedError as kinit_err:
             raise ProvisionError(kinit_err) from kinit_err
         except AttributeError as hub_err:
@@ -1136,6 +1282,8 @@ class BeakerAPI:
             raise ProvisionError(
                 f"Configuration file missing: {missing_conf_err.filename}"
             ) from missing_conf_err
+        except Exception as e:
+            raise ProvisionError("Failed to initialize mrack transformer.") from e
 
         self._mrack_provider = self._mrack_transformer._provider
         self._mrack_provider.poll_sleep = DEFAULT_PROVISION_TICK
@@ -1204,9 +1352,26 @@ class GuestBeaker(tmt.steps.provision.GuestSsh):
     public_key: list[str]
     api_session_refresh_tick: int
 
+    # Bootc-related attributes (injected by Guest.load())
+    bootc: bool
+    bootc_image_url: Optional[str]
+    bootc_check_system_url: Optional[str]
+    bootc_registry_secret: Optional[str]
+
     _api: Optional[BeakerAPI] = None
     _api_timestamp: Optional[datetime.datetime] = None
 
+    def __init__(self, *args: Any, **kwargs: Any):
+        """
+        Make sure that the mrack module is available and imported
+        """
+
+        super().__init__(*args, **kwargs)
+
+        assert isinstance(self.parent, tmt.steps.provision.Provision)
+        self.mrack_log = f"{self.parent.workdir}/{self.parent.name}-mrack.log"
+        import_and_load_mrack_deps(self.mrack_log, self._logger)
+
     @property
     def api(self) -> BeakerAPI:
         """
@@ -1214,10 +1379,6 @@ class GuestBeaker(tmt.steps.provision.GuestSsh):
         """
 
         def _construct_api() -> tuple[BeakerAPI, datetime.datetime]:
-            assert self.parent is not None
-
-            import_and_load_mrack_deps(self.parent.workdir, self.parent.name, self._logger)
-
             return BeakerAPI(self), datetime.datetime.now(datetime.timezone.utc)
 
         if self._api is None:
@@ -1264,11 +1425,28 @@ class GuestBeaker(tmt.steps.provision.GuestSsh):
         except mrack.errors.MrackError:
             return False
 
-    def _create(self, tmt_name: str) -> None:
+    @property
+    def _bootc_registry_credentials(self) -> dict[str, Any]:
+        """Create authentication dictionary with proper credential handling"""
+        if not self.bootc_registry_secret or not self.bootc_image_url:
+            return {}
+        base_repo = _get_registry_from_url(self.bootc_image_url)
+        # Support multiple registries
+        return {"auths": {base_repo: {"auth": self.bootc_registry_secret}}}
+
+    def start(self) -> None:
         """
-        Create beaker job xml request and submit it to Beaker hub
+        Start the guest
+
+        Get a new guest instance running. This should include preparing
+        any configuration necessary to get it started. Called after
+        load() is completed so all guest data should be available.
         """
 
+        if self.job_id and self.primary_address:
+            return
+
+        tmt_name = self._tmt_name()
         data = CreateJobParameters(
             tmt_name=tmt_name,
             hardware=self.hardware,
@@ -1280,6 +1458,10 @@ class GuestBeaker(tmt.steps.provision.GuestSsh):
             beaker_job_owner=self.beaker_job_owner,
             public_key=self.public_key,
             group=self.beaker_job_group,
+            bootc_credentials=self._bootc_registry_credentials if self.bootc else None,
+            bootc_image_url=self.bootc_image_url,
+            bootc=self.bootc,
+            bootc_check_system_url=self.bootc_check_system_url,
         )
 
         if self.is_dry_run:
@@ -1295,7 +1477,6 @@ class GuestBeaker(tmt.steps.provision.GuestSsh):
                     job.toxml(prettyxml=True),
                     flags=re.MULTILINE,
                 ),
-                shift=-2,
             )
             return
 
@@ -1376,17 +1557,14 @@ class GuestBeaker(tmt.steps.provision.GuestSsh):
                         self.warn('No console.log available.')
                     return current
 
-                raise tmt.utils.WaitingIncompleteError
+                raise tmt.utils.wait.WaitingIncompleteError
 
             try:
-                guest_info = tmt.utils.wait(
-                    self,
-                    get_new_state,
-                    datetime.timedelta(seconds=self.provision_timeout),
-                    tick=self.provision_tick,
-                )
+                guest_info = Waiting(
+                    Deadline.from_seconds(self.provision_timeout), tick=self.provision_tick
+                ).wait(get_new_state, self._logger)
 
-            except tmt.utils.WaitingTimedOutError:
+            except tmt.utils.wait.WaitingTimedOutError:
                 response = self.api.delete()
                 raise ProvisionError(
                     f'Failed to provision in the given amount '
@@ -1395,21 +1573,11 @@ class GuestBeaker(tmt.steps.provision.GuestSsh):
 
         self.primary_address = self.topology_address = guest_info['system']
 
-    def start(self) -> None:
-        """
-        Start the guest
-
-        Get a new guest instance running. This should include preparing
-        any configuration necessary to get it started. Called after
-        load() is completed so all guest data should be available.
-        """
-
-        if self.job_id is None or self.primary_address is None:
-            self._create(self._tmt_name())
-
         self.verbose('primary address', self.primary_address, 'green')
         self.verbose('topology address', self.topology_address, 'green')
 
+        self.assert_reachable()
+
     def remove(self) -> None:
         """
         Remove the guest
@@ -1424,9 +1592,7 @@ class GuestBeaker(tmt.steps.provision.GuestSsh):
         self,
         hard: bool = False,
         command: Optional[Union[Command, ShellScript]] = None,
-        timeout: Optional[int] = None,
-        tick: float = tmt.utils.DEFAULT_WAIT_TICK,
-        tick_increase: float = tmt.utils.DEFAULT_WAIT_TICK_INCREASE,
+        waiting: Optional[Waiting] = None,
     ) -> bool:
         """
         Reboot the guest, and wait for the guest to recover.
@@ -1455,6 +1621,8 @@ class GuestBeaker(tmt.steps.provision.GuestSsh):
         :returns: ``True`` if the reboot succeeded, ``False`` otherwise.
         """
 
+        waiting = waiting or tmt.steps.provision.default_reboot_waiting()
+
         if hard:
             self.debug("Hard reboot using the reboot command 'bkr system-power --action reboot'.")
 
@@ -1462,18 +1630,14 @@ class GuestBeaker(tmt.steps.provision.GuestSsh):
 
             return self.perform_reboot(
                 lambda: self._run_guest_command(reboot_script.to_shell_command()),
-                timeout=timeout,
-                tick=tick,
-                tick_increase=tick_increase,
+                waiting,
                 fetch_boot_time=False,
             )
 
         return super().reboot(
             hard=False,
             command=command,
-            timeout=timeout,
-            tick=tick,
-            tick_increase=tick_increase,
+            waiting=waiting,
         )
 
 
@@ -1534,7 +1698,7 @@ class ProvisionBeaker(tmt.steps.provision.ProvisionPlugin[ProvisionBeakerData]):
     _data_class = ProvisionBeakerData
     _guest_class = GuestBeaker
 
-    # _thread_safe = True
+    _thread_safe = True
 
     # Guest instance
     _guest = None
@@ -1556,6 +1720,58 @@ class ProvisionBeaker(tmt.steps.provision.ProvisionPlugin[ProvisionBeakerData]):
                 logger=self._logger,
             )
 
+    @property
+    def _preserved_workdir_members(self) -> set[str]:
+        """
+        A set of members of the step workdir that should not be removed.
+        """
+        assert self._guest
+
+        # Extract just the filename from the full path since
+        # prune_directory compares against member.name
+        mrack_log_filename = Path(self._guest.mrack_log).name
+        return {*super()._preserved_workdir_members, mrack_log_filename}
+
+    def _validate_bootc_configuration(self) -> None:
+        """Comprehensive bootc configuration validation"""
+
+        # Required fields validation
+        required_fields = ['image', 'bootc_image_url']
+
+        missing_fields = [field for field in required_fields if not getattr(self.data, field)]
+        if missing_fields:
+            raise tmt.utils.ProvisionError(
+                f"bootc configuration incomplete. Missing: {', '.join(missing_fields)}"
+            )
+
+        # Validate image URLs
+        if not self._validate_bootc_image_url():
+            raise tmt.utils.ProvisionError(f"Invalid image URL: {self.data.bootc_image_url}")
+
+    def _validate_bootc_image_url(self) -> bool:
+        """Validate container image URL format"""
+
+        if not self.data.bootc_image_url:
+            return False
+        pattern = r'^[a-zA-Z0-9._-]+(/[a-zA-Z0-9._-]+)*:[a-zA-Z0-9._-]+$'
+        return bool(re.match(pattern, self.data.bootc_image_url))
+
+    def _validate_bootc_disabled_configuration(self) -> None:
+        """Verify that bootc_* variables are not specified when bootc is not enabled"""
+
+        # Check all bootc_* variables(bootc_check_system_url has default value, no check)
+        bootc_fields = ['bootc_image_url', 'bootc_registry_secret']
+
+        specified_fields = [
+            field for field in bootc_fields if getattr(self.data, field) is not None
+        ]
+
+        if specified_fields:
+            raise tmt.utils.ProvisionError(
+                f"bootc configuration variables specified when bootc is not enabled. "
+                f"Enable bootc with --bootc, or remove variables: {', '.join(specified_fields)}."
+            )
+
     def go(self, *, logger: Optional[tmt.log.Logger] = None) -> None:
         """
         Provision the guest
@@ -1563,15 +1779,15 @@ class ProvisionBeaker(tmt.steps.provision.ProvisionPlugin[ProvisionBeakerData]):
 
         super().go(logger=logger)
 
+        if self.data.bootc:
+            self._validate_bootc_configuration()
+        else:
+            self._validate_bootc_disabled_configuration()
+
         data = BeakerGuestData.from_plugin(self)
 
         data.show(verbose=self.verbosity_level, logger=self._logger)
 
-        if data.hardware:
-            data.hardware.report_support(
-                names=list(_CONSTRAINT_TRANSFORMERS.keys()), logger=self._logger
-            )
-
         self._guest = GuestBeaker(
             data=data,
             name=self.name,
@@ -1596,5 +1812,10 @@ class GuestLogBeaker(tmt.steps.provision.GuestLog):
         try:
             return tmt.utils.get_url_content(self.url)
         except Exception as error:
-            logger.warning(f'Failed to fetch log: {error}')
+            tmt.utils.show_exception_as_warning(
+                exception=error,
+                message=f"Failed to fetch '{self.url}' log.",
+                logger=self.guest._logger,
+            )
+
             return None
