@@ -9,8 +9,17 @@ import tmt.steps
 import tmt.steps.provision
 import tmt.utils
 from tmt.container import container, field
-from tmt.steps.provision import GuestCapability
-from tmt.utils import Command, OnProcessStartCallback, Path, ShellScript, retry
+from tmt.steps.provision import GuestCapability, TransferOptions
+from tmt.utils import (
+    Command,
+    OnProcessEndCallback,
+    OnProcessStartCallback,
+    Path,
+    ShellScript,
+    retry,
+)
+from tmt.utils.hints import get_hint
+from tmt.utils.wait import Deadline, Waiting
 
 # Timeout in seconds of waiting for a connection
 CONNECTION_TIMEOUT = 60
@@ -242,13 +251,13 @@ class GuestContainer(tmt.Guest):
             )
         )
 
+        self.assert_reachable()
+
     def reboot(
         self,
         hard: bool = False,
         command: Optional[Union[Command, ShellScript]] = None,
-        timeout: Optional[int] = None,
-        tick: float = tmt.utils.DEFAULT_WAIT_TICK,
-        tick_increase: float = tmt.utils.DEFAULT_WAIT_TICK_INCREASE,
+        waiting: Optional[Waiting] = None,
     ) -> bool:
         """
         Reboot the guest, and wait for the guest to recover.
@@ -281,13 +290,15 @@ class GuestContainer(tmt.Guest):
             if self.container is None:
                 raise tmt.utils.ProvisionError("No container initialized.")
 
+            if waiting is None:
+                waiting = tmt.steps.provision.default_reconnect_waiting()
+                waiting.deadline = Deadline.from_seconds(CONNECTION_TIMEOUT)
+
             self.debug("Hard reboot using the reboot command 'container restart'.")
 
             self.podman(Command('container', 'restart', self.container))
 
-            return self.reconnect(
-                timeout=timeout or CONNECTION_TIMEOUT, tick=tick, tick_increase=tick_increase
-            )
+            return self.reconnect(waiting)
 
         if command:
             raise tmt.utils.ProvisionError(
@@ -359,10 +370,11 @@ class GuestContainer(tmt.Guest):
                 silent=silent,
             )
         except tmt.utils.RunError as exc:
-            if "File 'ansible-playbook' not found." in exc.message:
-                from tmt.utils.hints import print_hint
+            hint = get_hint('ansible-not-available', ignore_missing=False)
+
+            if hint.search_cli_patterns(exc.stderr, exc.stdout, exc.message):
+                hint.print(self._logger)
 
-                print_hint(id_='ansible-not-available', logger=self._logger)
             raise exc
 
     def podman(
@@ -399,6 +411,7 @@ class GuestContainer(tmt.Guest):
         log: Optional[tmt.log.LoggingFunction] = None,
         interactive: bool = False,
         on_process_start: Optional[OnProcessStartCallback] = None,
+        on_process_end: Optional[OnProcessEndCallback] = None,
         **kwargs: Any,
     ) -> tmt.utils.CommandOutput:
         """
@@ -447,6 +460,7 @@ class GuestContainer(tmt.Guest):
             silent=silent,
             interactive=interactive,
             on_process_start=on_process_start,
+            on_process_end=on_process_end,
             **kwargs,
         )
 
@@ -454,7 +468,7 @@ class GuestContainer(tmt.Guest):
         self,
         source: Optional[Path] = None,
         destination: Optional[Path] = None,
-        options: Optional[list[str]] = None,
+        options: Optional[TransferOptions] = None,
         superuser: bool = False,
     ) -> None:
         """
@@ -498,8 +512,7 @@ class GuestContainer(tmt.Guest):
         self,
         source: Optional[Path] = None,
         destination: Optional[Path] = None,
-        options: Optional[list[str]] = None,
-        extend_options: Optional[list[str]] = None,
+        options: Optional[TransferOptions] = None,
     ) -> None:
         """
         Nothing to be done to pull workdir
@@ -541,6 +554,8 @@ class GuestContainer(tmt.Guest):
                     # error string:
                     # https://github.com/containers/podman/blob/main/libpod/define/errors.go#L180
                     self.debug(f"Network '{self.network}' is being used, not removing.", level=3)
+                elif err.stderr and 'network not found' in err.stderr:
+                    self.debug(f"Network '{self.network}' has been already removed.", level=3)
                 else:
                     raise err
 
