@@ -1,10 +1,11 @@
 import collections
-import datetime
 import functools
 import itertools
 import os
 import platform
 import re
+import shutil
+import tempfile
 import threading
 import types
 from collections.abc import Iterator
@@ -21,9 +22,10 @@ import tmt.log
 import tmt.steps
 import tmt.steps.provision
 import tmt.utils
+import tmt.utils.wait
 from tmt.container import container, field
+from tmt.steps.provision import CONNECT_TIMEOUT, default_connect_waiting
 from tmt.utils import (
-    WORKDIR_ROOT,
     Command,
     Path,
     ProvisionError,
@@ -31,6 +33,7 @@ from tmt.utils import (
     configure_constant,
     retry_session,
 )
+from tmt.utils.wait import Deadline, Waiting
 
 if TYPE_CHECKING:
     import tmt.base
@@ -91,9 +94,9 @@ def import_testcloud(logger: tmt.log.Logger) -> None:
         )
         from testcloud.workarounds import Workarounds
     except ImportError as error:
-        from tmt.utils.hints import print_hint
+        from tmt.utils.hints import print_hints
 
-        print_hint(id_='provision/virtual.testcloud', logger=logger)
+        print_hints('provision/virtual.testcloud', logger=logger)
 
         raise ProvisionError('Could not import testcloud package.') from error
 
@@ -103,14 +106,11 @@ def import_testcloud(logger: tmt.log.Logger) -> None:
     TPM_CONFIG_ALLOWS_VERSIONS = hasattr(TPMConfiguration(), 'version')
 
 
-# Testcloud cache to our tmt's workdir root
-TESTCLOUD_DATA = (
-    Path(os.environ['TMT_WORKDIR_ROOT']) if os.getenv('TMT_WORKDIR_ROOT') else WORKDIR_ROOT
-) / 'testcloud'
-TESTCLOUD_IMAGES = TESTCLOUD_DATA / 'images'
-
 TESTCLOUD_WORKAROUNDS: list[str] = []  # A list of commands to be executed during guest boot up
 
+# Target filename to store the console log
+CONSOLE_LOG_FILE = "console.txt"
+
 # Userdata for cloud-init
 # TODO: Explore migration to jinja from string.Template
 USER_DATA = """#cloud-config
@@ -136,20 +136,16 @@ passwd:
     - name: ${user_name}
       ssh_authorized_keys:
         - ${public_key}
-systemd:
-  units:
-    - name: ssh_root_login.service
-      enabled: true
-      contents: |
-        [Unit]
-        Before=sshd.service
-        [Service]
-        Type=oneshot
-        ExecStart=/usr/bin/sed -i \
-                  "s|^PermitRootLogin no$|PermitRootLogin yes|g" \
-                  /etc/ssh/sshd_config
-        [Install]
-        WantedBy=multi-user.target
+storage:
+  files:
+    - path: /etc/ssh/sshd_config.d/20-enable-root-login.conf
+      mode: 0644
+      contents:
+        inline: |
+          # CoreOS disables root SSH login by default.
+          # Enable it.
+          # This file must sort before 40-rhcos-defaults.conf.
+          PermitRootLogin yes
 """
 
 # VM defaults
@@ -162,15 +158,6 @@ DEFAULT_BOOT_TIMEOUT: int = 2 * 60
 #: ``TMT_BOOT_TIMEOUT``.
 BOOT_TIMEOUT: int = configure_constant(DEFAULT_BOOT_TIMEOUT, 'TMT_BOOT_TIMEOUT')
 
-#: How many seconds to wait for a connection to succeed after guest boot.
-#: This is the default value tmt would use unless told otherwise.
-DEFAULT_CONNECT_TIMEOUT = 2 * 60
-
-#: How many seconds to wait for a connection to succeed after guest boot.
-#: This is the effective value, combining the default and optional envvar,
-#: ``TMT_CONNECT_TIMEOUT``.
-CONNECT_TIMEOUT: int = configure_constant(DEFAULT_CONNECT_TIMEOUT, 'TMT_CONNECT_TIMEOUT')
-
 #: How many times should the timeouts be multiplied in kvm-less cases.
 #: These include emulating a different architecture than the host,
 #: some nested virtualization cases, and hosts with degraded virt caps.
@@ -179,13 +166,22 @@ NON_KVM_TIMEOUT_COEF = 10  # times
 # SSH key type, set None for ssh-keygen default one
 SSH_KEYGEN_TYPE = "ecdsa"
 
-DEFAULT_USER = 'root'
 DEFAULT_CPU_COUNT = 2
 DEFAULT_MEMORY: 'Size' = tmt.hardware.UNITS('2048 MB')
 DEFAULT_DISK: 'Size' = tmt.hardware.UNITS('40 GB')
 DEFAULT_IMAGE = 'fedora'
 DEFAULT_CONNECTION = 'session'
 DEFAULT_ARCH = platform.machine()
+#: Default number of attempts to stop a VM.
+#:
+#: .. note::
+#:
+#:    The value :py:mod:`testcloud` starts with is ``3``, and we already
+#:    observed some VMs with bootc involved to not shut down in time.
+#:    Therefore starting with increased default on our side.
+DEFAULT_STOP_RETRIES = 10
+#: Default time, in seconds, to wait between attempts to stop a VM.
+DEFAULT_STOP_RETRY_DELAY = 1
 
 # Version-aware TPM configuration is added in
 # https://pagure.io/testcloud/c/89f1c024ca829543de7f74f89329158c6dee3d83
@@ -207,6 +203,10 @@ TPM_VERSION_SUPPORTED_VERSIONS = {
     False: ['2.0', '2'],
 }
 
+#: Image url fetch retry attempts and interval
+IMAGE_URL_FETCH_RETRY_ATTEMPTS = 5
+IMAGE_URL_FETCH_RETRY_INTERVAL = 5
+
 
 def normalize_memory_size(
     key_address: str,
@@ -268,7 +268,7 @@ def normalize_disk_size(key_address: str, value: Any, logger: tmt.log.Logger) ->
     raise tmt.utils.NormalizationError(key_address, value, 'an integer')
 
 
-def _report_hw_requirement_support(constraint: tmt.hardware.Constraint[Any]) -> bool:
+def _report_hw_requirement_support(constraint: tmt.hardware.Constraint) -> bool:
     components = constraint.expand_name()
 
     if components.name == 'memory' and constraint.operator in (
@@ -314,14 +314,6 @@ def _report_hw_requirement_support(constraint: tmt.hardware.Constraint[Any]) ->
 
 @container
 class TestcloudGuestData(tmt.steps.provision.GuestSshData):
-    # Override parent class with our defaults
-    user: str = field(
-        default=DEFAULT_USER,
-        option=('-u', '--user'),
-        metavar='USERNAME',
-        help='Username to use for all guest operations.',
-    )
-
     image: str = field(
         default=DEFAULT_IMAGE,
         option=('-i', '--image'),
@@ -382,6 +374,24 @@ class TestcloudGuestData(tmt.steps.provision.GuestSshData):
         internal=True,
     )
 
+    stop_retries: int = field(
+        default=DEFAULT_STOP_RETRIES,
+        metavar='N',
+        option='--stop-retries',
+        help="""
+             Number of attempts to stop a VM.
+             """,
+    )
+
+    stop_retry_delay: int = field(
+        default=DEFAULT_STOP_RETRY_DELAY,
+        metavar='SECONDS',
+        option='--stop-retry-delay',
+        help="""
+             Time to wait between attempts to stop a VM.
+             """,
+    )
+
     # TODO: custom handling for two fields - when the formatting moves into
     # field(), this should not be necessary.
     def show(
@@ -396,11 +406,6 @@ class TestcloudGuestData(tmt.steps.provision.GuestSshData):
 
         super().show(keys=super_keys, verbose=verbose, logger=logger)
 
-        # TODO: find formatting that would show "MB" instead of "megabyte"
-        # https://github.com/teemtee/tmt/issues/2410
-        logger.info('memory', f'{(self.memory or DEFAULT_MEMORY).to("MB")}', 'green')
-        logger.info('disk', f'{(self.disk or DEFAULT_DISK).to("GB")}', 'green')
-
 
 @container
 class ProvisionTestcloudData(TestcloudGuestData, tmt.steps.provision.ProvisionStepData):
@@ -669,6 +674,9 @@ class GuestTestcloud(tmt.GuestSsh):
     connection: str
     arch: str
 
+    stop_retries: int
+    stop_retry_delay: int
+
     # Not to be saved, recreated from image_url/instance_name/... every
     # time guest is instantiated.
     # FIXME: ignore[name-defined]: https://github.com/teemtee/tmt/issues/1616
@@ -683,6 +691,14 @@ class GuestTestcloud(tmt.GuestSsh):
     #: remove the lock.
     _testcloud_lock = threading.Lock()
 
+    @functools.cached_property
+    def testcloud_data_dirpath(self) -> Path:
+        return self.workdir_root / 'testcloud'
+
+    @functools.cached_property
+    def testcloud_image_dirpath(self) -> Path:
+        return self.testcloud_data_dirpath / 'images'
+
     @property
     def is_ready(self) -> bool:
         if self._instance is None:
@@ -733,14 +749,14 @@ class GuestTestcloud(tmt.GuestSsh):
             except requests.RequestException:
                 pass
             finally:
-                raise tmt.utils.WaitingIncompleteError
+                raise tmt.utils.wait.WaitingIncompleteError
 
         try:
-            return tmt.utils.wait(
-                self, try_get_url, datetime.timedelta(seconds=CONNECT_TIMEOUT), tick=1
+            return Waiting(Deadline.from_seconds(CONNECT_TIMEOUT), tick=1).wait(
+                try_get_url, self._logger
             )
 
-        except tmt.utils.WaitingTimedOutError:
+        except tmt.utils.wait.WaitingTimedOutError:
             raise ProvisionError(f'Failed to {message} in {CONNECT_TIMEOUT}s.')
 
     def _guess_image_url(self, name: str) -> str:
@@ -753,14 +769,23 @@ class GuestTestcloud(tmt.GuestSsh):
         if name_as_path.is_absolute() and name_as_path.is_file():
             return f'file://{name}'
 
+        name = name.lower().strip()
         url: Optional[str] = None
         assert testcloud is not None
 
         with GuestTestcloud._testcloud_lock:
             try:
-                url = testcloud.util.get_image_url(name.lower().strip(), self.arch)
+                url = tmt.utils.retry(
+                    func=testcloud.util.get_image_url,
+                    attempts=IMAGE_URL_FETCH_RETRY_ATTEMPTS,
+                    interval=IMAGE_URL_FETCH_RETRY_INTERVAL,
+                    label=f"Get image url for '{name}'.",
+                    logger=self._logger,
+                    distro_str=name,
+                    arch=self.arch,
+                )
             except Exception as error:
-                raise ProvisionError("Could not get image url.") from error
+                raise ProvisionError(f"Could not get image url for '{name}'.") from error
 
         if not url:
             raise ProvisionError(f"Could not map '{name}' to compose.")
@@ -836,9 +861,17 @@ class GuestTestcloud(tmt.GuestSsh):
         self.config.DOWNLOAD_PROGRESS = self.debug_level > 2
         self.config.DOWNLOAD_PROGRESS_VERBOSE = False
 
+        # We can't assign a not-exists path to STORE_DIR,
+        # so we should make sure required directories exist
+        os.makedirs(self.testcloud_data_dirpath, exist_ok=True)
+        os.makedirs(self.testcloud_image_dirpath, exist_ok=True)
+
         # Configure to tmt's storage directories
-        self.config.DATA_DIR = TESTCLOUD_DATA
-        self.config.STORE_DIR = TESTCLOUD_IMAGES
+        self.config.DATA_DIR = self.testcloud_data_dirpath
+        self.config.STORE_DIR = self.testcloud_image_dirpath
+
+        self.config.STOP_RETRIES = self.stop_retries
+        self.config.STOP_RETRY_WAIT = self.stop_retry_delay
 
     def _combine_hw_memory(self) -> None:
         """
@@ -953,12 +986,20 @@ class GuestTestcloud(tmt.GuestSsh):
 
         if self.is_dry_run:
             return
-        # Make sure required directories exist
-        os.makedirs(TESTCLOUD_DATA, exist_ok=True)
-        os.makedirs(TESTCLOUD_IMAGES, exist_ok=True)
+
+        # Prepare the console log
+        assert self.logdir is not None  # Narrow type
+        console_log = ConsoleLog(
+            name=CONSOLE_LOG_FILE,
+            testcloud_symlink_path=self.logdir / CONSOLE_LOG_FILE,
+            guest=self,
+        )
+        console_log.prepare(logger=self._logger)
+        self.guest_logs.append(console_log)
 
         # Prepare config
         self.prepare_config()
+        self.config.CONSOLE_LOG_DIR = console_log.exchange_directory
 
         # Kick off image URL with the given image
         self.image_url = self.image
@@ -981,7 +1022,7 @@ class GuestTestcloud(tmt.GuestSsh):
             raise ProvisionError(f"Image '{self._image.local_path}' not found.") from error
         except (testcloud.exceptions.TestcloudPermissionsError, PermissionError) as error:
             raise ProvisionError(
-                f"Failed to prepare the image. Check the '{TESTCLOUD_IMAGES}' "
+                f"Failed to prepare the image. Check the '{self.testcloud_image_dirpath}' "
                 f"directory permissions."
             ) from error
         except KeyError as error:
@@ -995,6 +1036,7 @@ class GuestTestcloud(tmt.GuestSsh):
 
         # Prepare DomainConfiguration object before Instance object
         self._domain = DomainConfiguration(self.instance_name)
+        self._domain.console_log_file = console_log.testcloud_symlink_path
 
         # Prepare Workarounds object
         self._workarounds = Workarounds(defaults=True)
@@ -1018,11 +1060,21 @@ class GuestTestcloud(tmt.GuestSsh):
         _apply_hw_disk_size(self.hardware, self._domain, self._logger)
         _apply_hw_tpm(self.hardware, self._domain, self._logger)
 
-        self.debug('final domain memory', str(self._domain.memory_size))
-        self.debug('final domain root disk size', str(self._domain.storage_devices[0].size))
+        self.info(
+            'memory',
+            f'{int(tmt.hardware.UNITS(f"{self._domain.memory_size} kB").to("MB").magnitude)} MB',
+            'green',
+        )
+        self.info(
+            'disk',
+            f'{tmt.hardware.UNITS(f"{self._domain.storage_devices[0].size} GB").to("GB")}',
+            'green',
+        )
 
         for i, device in enumerate(self._domain.storage_devices):
-            self.debug(f'final domain disk #{i} size', str(device.size))
+            self.debug(
+                f'domain disk #{i} size', f'{tmt.hardware.UNITS(f"{device.size} GB").to("GB")}'
+            )
 
         # Is this a CoreOS?
         self._domain.coreos = self.is_coreos
@@ -1078,6 +1130,7 @@ class GuestTestcloud(tmt.GuestSsh):
 
         # Boot the virtual machine
         self.info('progress', 'booting...', 'cyan')
+        self.verbose("console", console_log.testcloud_symlink_path, level=2, color="cyan")
         assert libvirt is not None
 
         try:
@@ -1094,8 +1147,9 @@ class GuestTestcloud(tmt.GuestSsh):
         self._instance.create_ip_file(self.primary_address)
 
         # Wait a bit until the box is up
-        if not self.reconnect(timeout=CONNECT_TIMEOUT * time_coeff, tick=1):
-            raise ProvisionError(f"Failed to connect in {CONNECT_TIMEOUT * time_coeff}s.")
+        waiting = default_connect_waiting()
+        waiting.deadline = Deadline.from_seconds(CONNECT_TIMEOUT * time_coeff)
+        self.assert_reachable(waiting)
 
     def stop(self) -> None:
         """
@@ -1132,9 +1186,7 @@ class GuestTestcloud(tmt.GuestSsh):
         self,
         hard: bool = False,
         command: Optional[Union[Command, ShellScript]] = None,
-        timeout: Optional[int] = None,
-        tick: float = tmt.utils.DEFAULT_WAIT_TICK,
-        tick_increase: float = tmt.utils.DEFAULT_WAIT_TICK_INCREASE,
+        waiting: Optional[Waiting] = None,
     ) -> bool:
         """
         Reboot the guest, and wait for the guest to recover.
@@ -1159,6 +1211,8 @@ class GuestTestcloud(tmt.GuestSsh):
         :returns: ``True`` if the reboot succeeded, ``False`` otherwise.
         """
 
+        waiting = waiting or tmt.steps.provision.default_reboot_waiting()
+
         if hard:
             if self._instance is None:
                 raise tmt.utils.ProvisionError("No instance initialized.")
@@ -1169,9 +1223,7 @@ class GuestTestcloud(tmt.GuestSsh):
             # being `None`, missing the explicit check above.
             return self.perform_reboot(
                 lambda: self._instance.reboot(soft=False),  # type: ignore[union-attr]
-                timeout=timeout,
-                tick=tick,
-                tick_increase=tick_increase,
+                waiting,
                 fetch_boot_time=False,
             )
 
@@ -1179,11 +1231,11 @@ class GuestTestcloud(tmt.GuestSsh):
             return super().reboot(
                 hard=False,
                 command=command,
-                timeout=timeout,
-                tick=tick,
-                tick_increase=tick_increase,
+                waiting=waiting,
             )
 
+        self.debug("Soft reboot using the testcloud API.")
+
         if self._instance is None:
             raise tmt.utils.ProvisionError("No instance initialized.")
 
@@ -1191,9 +1243,7 @@ class GuestTestcloud(tmt.GuestSsh):
         # being `None`, missing the explicit check above.
         return self.perform_reboot(
             lambda: self._instance.reboot(soft=True),  # type: ignore[union-attr]
-            timeout=timeout,
-            tick=tick,
-            tick_increase=tick_increase,
+            waiting,
         )
 
 
@@ -1241,6 +1291,9 @@ class ProvisionTestcloud(tmt.steps.provision.ProvisionPlugin[ProvisionTestcloudD
             # in GB
             disk: 30
 
+    Images
+    ^^^^^^
+
     As the image use ``fedora`` for the latest released Fedora compose,
     ``fedora-rawhide`` for the latest Rawhide compose, short aliases such as
     ``fedora-32``, ``f-32`` or ``f32`` for specific release or a full url to
@@ -1273,9 +1326,22 @@ class ProvisionTestcloud(tmt.steps.provision.ProvisionPlugin[ProvisionTestcloudD
     In addition to the qcow2 format, Vagrant boxes can be used as well,
     testcloud will take care of unpacking the image for you.
 
+    Reboot
+    ^^^^^^
+
     To trigger hard reboot of a guest, plugin uses testcloud API. It is
     also used to trigger soft reboot unless a custom reboot command was
     specified via ``tmt-reboot -c ...``.
+
+    Console
+    ^^^^^^^
+
+    The full console log is available, after the guest is booted, in the
+    ``logs`` directory under the provision step workdir, for example:
+    ``plan/provision/client/logs/console.txt``. Enable verbose mode
+    using ``-vv`` to get the full path printed to the terminal for easy
+    investigation.
+
     """
 
     _data_class = ProvisionTestcloudData
@@ -1356,10 +1422,11 @@ class ProvisionTestcloud(tmt.steps.provision.ProvisionPlugin[ProvisionTestcloudD
         Print images which are already cached
         """
 
+        store_dir = self.workdir_root / 'testcloud/images'
         self.info("Locally available images")
-        for filename in sorted(TESTCLOUD_IMAGES.glob('*.qcow2')):
+        for filename in sorted(store_dir.glob('*.qcow2')):
             self.info(filename.name, shift=1, color='yellow')
-            click.echo(f"{TESTCLOUD_IMAGES / filename}")
+            click.echo(f"{store_dir / filename}")
 
     @classmethod
     def clean_images(cls, clean: 'tmt.base.Clean', dry: bool, workdir_root: Path) -> bool:
@@ -1384,3 +1451,71 @@ class ProvisionTestcloud(tmt.steps.provision.ProvisionPlugin[ProvisionTestcloudD
                     clean.fail(f"Failed to remove '{image}'.", shift=2)
                     successful = False
         return successful
+
+
+@container
+class ConsoleLog(tmt.steps.provision.GuestLog):
+    # Path where testcloud will create the symlink to the console log
+    testcloud_symlink_path: Path
+
+    # Temporary directory for storing the console log content
+    exchange_directory: Optional[Path] = None
+
+    def prepare(self, logger: tmt.log.Logger) -> None:
+        """
+        Prepare temporary directory for the console log.
+
+        Special directory is needed for console logs with the right
+        selinux context so that virtlogd is able to write there.
+        """
+
+        self.exchange_directory = Path(tempfile.mkdtemp(prefix="testcloud-"))
+        logger.debug(f"Created console log directory '{self.exchange_directory}'.", level=3)
+
+        self.exchange_directory.chmod(0o755)
+        self.guest._run_guest_command(
+            Command("chcon", "--type", "virt_log_t", self.exchange_directory), silent=True
+        )
+
+    def cleanup(self, logger: tmt.log.Logger) -> None:
+        """
+        Remove the temporary directory.
+        """
+
+        if self.exchange_directory is None:
+            return
+
+        try:
+            logger.debug(f"Remove console log directory '{self.exchange_directory}'.", level=3)
+            shutil.rmtree(self.exchange_directory)
+            self.exchange_directory = None
+
+        except OSError as error:
+            logger.warning(
+                f"Failed to remove console log directory '{self.exchange_directory}': {error}"
+            )
+
+    def fetch(self, logger: tmt.log.Logger) -> Optional[str]:
+        """
+        Read the content of the symlink target prepared by testcloud.
+        """
+
+        text = None
+
+        try:
+            logger.debug(
+                f"Read the console log content from '{self.testcloud_symlink_path}'.", level=3
+            )
+            text = self.testcloud_symlink_path.read_text(errors="ignore")
+
+        except OSError as error:
+            tmt.utils.show_exception_as_warning(
+                exception=error,
+                message='Failed to read the console log.',
+                logger=self.guest._logger,
+            )
+
+        self.testcloud_symlink_path.unlink()
+        self.cleanup(logger)
+
+        return text
