@@ -7,9 +7,7 @@ from typing import TYPE_CHECKING, Any, Callable, Optional, TypedDict, Union, cas
 from jinja2 import FileSystemLoader, select_autoescape
 
 import tmt
-import tmt.base
 import tmt.log
-import tmt.options
 import tmt.result
 import tmt.steps
 import tmt.steps.report
@@ -307,11 +305,26 @@ def make_junit_xml(
         except tmt.utils.FileError:
             return ''
 
+    def _failures_filter(paths: list[Path]) -> list[str]:
+        """
+        Return a list of failures from a given failure log file
+        """
+
+        failures: list[str] = []
+        for path in paths:
+            try:
+                failures += tmt.utils.yaml_to_list(phase.step.plan.execute.read(path))
+            except tmt.utils.FileError:
+                phase.warn(f"Unable to read failure log file: '{path}'")
+                continue
+
+        return failures
+
     environment.filters.update(
         {
             'read_log': _read_log_filter,
             'duration_to_seconds': _duration_to_seconds_filter,
-            'failures': tmt.result.Result.failures,
+            'failures': _failures_filter,
             # Use a wrapper function to also _escape_control_chars.
             'e': _escape_control_chars_filter(environment.filters['e']),
         }
@@ -336,9 +349,9 @@ def make_junit_xml(
         from lxml import etree
 
     except ImportError:
-        from tmt.utils.hints import print_hint
+        from tmt.utils.hints import print_hints
 
-        print_hint(id_='report/junit', logger=phase._logger)
+        print_hints('report/junit', logger=phase._logger)
 
         return xml_data
 
@@ -371,8 +384,7 @@ def make_junit_xml(
 
     xml_parser = etree.XMLParser(**xml_parser_kwargs)
     try:
-        # S320: Parsing of untrusted data is known to be vulnerable to XML attacks.
-        tree_root: XMLElement = etree.fromstring(xml_data, xml_parser)  # noqa: S320
+        tree_root: XMLElement = etree.fromstring(xml_data, xml_parser)
 
     except etree.XMLSyntaxError as e:
         phase.warn(
@@ -392,7 +404,7 @@ def make_junit_xml(
         xml_parser = etree.XMLParser(**xml_parser_kwargs)
 
         try:
-            tree_root = etree.fromstring(xml_data, xml_parser)  # noqa: S320
+            tree_root = etree.fromstring(xml_data, xml_parser)
         except etree.XMLSyntaxError as error:
             phase.verbose('rendered XML', xml_data, 'red')
             raise tmt.utils.ReportError(
