@@ -1,7 +1,8 @@
 import datetime
 import os
 import re
-from typing import TYPE_CHECKING, Any, Optional, overload
+from re import Pattern
+from typing import TYPE_CHECKING, Any, Optional, Union, overload
 
 import requests
 import urllib3
@@ -11,9 +12,16 @@ import tmt.log
 import tmt.steps.report
 import tmt.utils
 import tmt.utils.templates
+from tmt._compat.pathlib import Path
 from tmt.container import container, field
 from tmt.result import ResultOutcome
-from tmt.utils import ActionType, catch_warnings_safe, format_timestamp, yaml_to_dict
+from tmt.utils import (
+    ActionType,
+    catch_warnings_safe,
+    format_timestamp,
+    sanitize_string,
+    yaml_to_dict,
+)
 
 if TYPE_CHECKING:
     from tmt._compat.typing import TypeAlias
@@ -23,6 +31,16 @@ JSON: 'TypeAlias' = Any
 DEFAULT_LOG_SIZE_LIMIT: 'Size' = tmt.hardware.UNITS('1 MB')
 DEFAULT_TRACEBACK_SIZE_LIMIT: 'Size' = tmt.hardware.UNITS('50 kB')
 
+DEFAULT_LOG_PATTERNS: list[Pattern[str]] = [
+    re.compile(pattern)
+    for pattern in [
+        r'avc\.txt',
+        r'dmesg-.*\.txt',
+        r'output\.txt',
+        r'tmt-watchdog\.txt',
+    ]
+]
+
 
 def _flag_env_to_default(option: str, default: bool) -> bool:
     env_var = 'TMT_PLUGIN_REPORT_REPORTPORTAL_' + option.upper()
@@ -48,6 +66,17 @@ def _str_env_to_default(option: str, default: Optional[str]) -> Optional[str]:
     return str(os.getenv(env_var))
 
 
+def _pattern_list_env_to_default(option: str, default: list[Pattern[str]]) -> list[Pattern[str]]:
+    env_var = 'TMT_PLUGIN_REPORT_REPORTPORTAL_' + option.upper()
+    if env_var not in os.environ or os.getenv(env_var) is None:
+        return default
+    return tmt.utils.normalize_pattern_list(
+        option,
+        [item.strip() for item in str(os.getenv(env_var)).split(',') if item.strip()],
+        tmt.log.Logger.get_bootstrap_logger(),
+    )
+
+
 def _size_env_to_default(option: str, default: 'Size') -> 'Size':
     return tmt.hardware.UNITS(_str_env_to_default(option, str(default)))
 
@@ -59,9 +88,8 @@ class LogFilterSettings:
 
 
 def _filter_invalid_chars(data: str, settings: LogFilterSettings) -> str:
-    return re.sub(
-        '[^\u0020-\ud7ff\u0009\u000a\u000d\ue000-\ufffd\U00010000-\U0010ffff]+', '', data
-    )
+    """Filter out invalid unicode characters"""
+    return sanitize_string(data)
 
 
 def _filter_log_per_size(data: str, settings: LogFilterSettings) -> str:
@@ -281,6 +309,25 @@ class ReportReportPortalData(tmt.steps.report.ReportStepData):
              """,
     )
 
+    upload_log_pattern: list[Pattern[str]] = field(
+        metavar="PATTERN",
+        option="--upload-log-pattern",
+        multiple=True,
+        default_factory=lambda: _pattern_list_env_to_default(
+            'upload_log_pattern', DEFAULT_LOG_PATTERNS[:]
+        ),
+        normalize=tmt.utils.normalize_pattern_list,
+        serialize=lambda patterns: [pattern.pattern for pattern in patterns],
+        unserialize=lambda serialized: [re.compile(pattern) for pattern in serialized],
+        help="""
+             List of regular expressions to look for in result log names. If any of the
+             patterns is found in a log file name, the log will be uploaded to ReportPortal.
+             Check result logs will be uploaded only if the check failed or if an error
+             occurred during the execution. The search mode is used for pattern matching.
+             See the :ref:`regular-expressions` section for details.
+             """,
+    )
+
     launch_url: Optional[str] = None
     launch_uuid: Optional[str] = None
     suite_uuid: Optional[str] = None
@@ -409,6 +456,7 @@ class ReportReportPortal(tmt.steps.report.ReportPlugin[ReportReportPortalData]):
         ResultOutcome.WARN: "FAILED",
         ResultOutcome.ERROR: "FAILED",
         ResultOutcome.SKIP: "SKIPPED",
+        ResultOutcome.PENDING: "SKIPPED",
     }
 
     def handle_response(self, response: requests.Response) -> None:
@@ -474,15 +522,16 @@ class ReportReportPortal(tmt.steps.report.ReportPlugin[ReportReportPortalData]):
 
         # Get common attributes across the plans
         merged_plans = [
-            {key: value[0] for key, value in plan._fmf_context.items()}
+            {key: value[0] for key, value in plan.fmf_context.items()}
             for plan in self.step.plan.my_run.plans
         ]
         result_dict = merged_plans[0]
         for current_plan in merged_plans[1:]:
-            tmp_dict = {}
-            for key, value in current_plan.items():
-                if key in result_dict and result_dict[key] == value:
-                    tmp_dict[key] = value
+            tmp_dict = {
+                key: value
+                for key, value in current_plan.items()
+                if key in result_dict and result_dict[key] == value
+            }
             result_dict = tmp_dict
         return [{'key': key, 'value': value} for key, value in result_dict.items()]
 
@@ -568,7 +617,7 @@ class ReportReportPortal(tmt.steps.report.ReportPlugin[ReportReportPortalData]):
 
     def upload_result_logs(
         self,
-        result: tmt.result.BaseResult,
+        result: Union[tmt.result.Result, tmt.result.SubResult],
         session: requests.Session,
         item_uuid: str,
         launch_uuid: str,
@@ -579,50 +628,60 @@ class ReportReportPortal(tmt.steps.report.ReportPlugin[ReportReportPortalData]):
         Upload all result log files into the ReportPortal instance
         """
 
-        for index, log_path in enumerate(result.log):
+        def upload_log(log_path: Path, is_yaml: bool = False, is_traceback: bool = False) -> None:
             try:
-                log = self.step.plan.execute.read(log_path)
-            except tmt.utils.FileError:
-                continue
+                if is_yaml:
+                    logs = tmt.utils.yaml_to_list(self.step.plan.execute.read(log_path))
+                else:
+                    logs = [self.step.plan.execute.read(log_path)]
+            except (tmt.utils.FileError, tmt.utils.GeneralError) as error:
+                tmt.utils.show_exception_as_warning(
+                    exception=error,
+                    message=f"Failed to read log file '{log_path}' for ReportPortal upload.",
+                    logger=self._logger,
+                )
+                return
 
-            level = "INFO" if index == 0 else "TRACE"
-            message = _filter_log(log, settings=LogFilterSettings(size=self.data.log_size_limit))
-
-            # Upload log
-            self.rp_api_post(
-                session=session,
-                path="log/entry",
-                json={
-                    "message": message,
-                    "itemUuid": item_uuid,
-                    "launchUuid": launch_uuid,
-                    "level": level,
-                    "time": timestamp,
-                },
+            filter_settings = LogFilterSettings(
+                size=self.data.traceback_size_limit if is_traceback else self.data.log_size_limit,
+                is_traceback=is_traceback,
             )
 
-            # Optionally write out failures only for results which implement the failures callable
-            if hasattr(result, "failures") and index == 0 and write_out_failures:
-                message = _filter_log(
-                    result.failures(log),
-                    settings=LogFilterSettings(
-                        size=self.data.traceback_size_limit,
-                        is_traceback=True,
-                    ),
-                )
-
+            for log in logs:
+                # Add file name to the log if it is not a traceback
+                if not is_traceback:
+                    log = f'### `{log_path.name}`\n{log}'
+                # Upload log
                 self.rp_api_post(
                     session=session,
                     path="log/entry",
                     json={
-                        "message": message,
+                        "message": _filter_log(log, filter_settings),
                         "itemUuid": item_uuid,
                         "launchUuid": launch_uuid,
-                        "level": "ERROR",
+                        "level": "ERROR" if is_traceback else "INFO",
                         "time": timestamp,
                     },
                 )
 
+        # Upload result logs
+        for log_path in result.log:
+            if any(pattern.search(log_path.name) for pattern in self.data.upload_log_pattern):
+                upload_log(log_path)
+
+        # Upload check result logs if the check failed
+        for check in result.check:
+            if check.result not in (ResultOutcome.FAIL, ResultOutcome.ERROR):
+                continue
+            for log_path in check.log:
+                if any(pattern.search(log_path.name) for pattern in self.data.upload_log_pattern):
+                    upload_log(log_path, is_traceback=True)
+
+        # Upload failure logs
+        if write_out_failures:
+            for failure_log in result.failure_logs:
+                upload_log(failure_log, is_yaml=True, is_traceback=True)
+
     def execute_rp_import(self) -> None:
         """
         Execute the import of test, results and subresults into ReportPortal
@@ -641,7 +700,7 @@ class ReportReportPortal(tmt.steps.report.ReportPlugin[ReportReportPortalData]):
             # The datetime *strings* are in fact sorted here, but finding the minimum will work,
             # because the datetime in ISO format is designed to be lexicographically sortable.
             launch_start_time = min(
-                [r.start_time or self.datetime for r in self.step.plan.execute.results()]
+                r.start_time or self.datetime for r in self.step.plan.execute.results()
             )
 
         # Create launch, suites (if "--suite_per_plan") and tests;
@@ -676,7 +735,7 @@ class ReportReportPortal(tmt.steps.report.ReportPlugin[ReportReportPortalData]):
         defect_type = self.data.defect_type or ""
 
         attributes = [
-            {'key': key, 'value': value[0]} for key, value in self.step.plan._fmf_context.items()
+            {'key': key, 'value': value[0]} for key, value in self.step.plan.fmf_context.items()
         ]
         launch_attributes = self.construct_launch_attributes(suite_per_plan, attributes)
 
