@@ -498,12 +498,9 @@ def validate_git_status(test: 'tmt.base.Test') -> tuple[bool, str]:
     except RunError as error:
         return (False, f"Failed to run git status: {error.stdout}")
 
-    not_committed: list[str] = []
     assert result.stdout is not None
-    for line in result.stdout.split('\n'):
-        if line:
-            # XY PATH or XY ORIG -> PATH. XY and PATH are separated by space
-            not_committed.append(line[3:])
+    # XY PATH or XY ORIG -> PATH. XY and PATH are separated by space
+    not_committed = [line[3:] for line in result.stdout.split('\n') if line]
 
     if not_committed:
         return (False, "Uncommitted changes in " + " ".join(not_committed))
@@ -598,7 +595,7 @@ class DistGitHandler:
         except Exception as error:
             raise GeneralError(f"Couldn't read '{self.sources_file_name}' file.") from error
         if not ret_values:
-            raise GeneralError("No sources found in '{self.sources_file_name}' file.")
+            raise GeneralError(f"No sources found in '{self.sources_file_name}' file.")
         return ret_values
 
     def its_me(self, remotes: list[str]) -> bool:
@@ -748,9 +745,15 @@ def git_clone(
         """
 
         depth = ['--depth=1'] if shallow else []
-        return Command('git', 'clone', *depth, url, destination).run(
+        output = Command('git', 'clone', *depth, url, destination).run(
             cwd=Path('/'), env=env, timeout=timeout, logger=logger
         )
+        logger.info(
+            'cloned-commit-hash',
+            git_hash(directory=destination, logger=logger),
+            color='green',
+        )
+        return output
 
     from tmt.utils import GIT_CLONE_ATTEMPTS, GIT_CLONE_INTERVAL, GIT_CLONE_TIMEOUT
 
